/**************************************************************************************************
 *
 *	initplace.c
 *
 *	part of the layout program.
 *
 *
 *	Written by Gerard Leurs, NIKHEF-H, Amsterdam.
 *	July 1991.
 *	email: a89@nikhef.nl
 *
 *************************************************************************************************/

# include "layout.h"

	/* get number of links for the nodes and then place them on the window
	   More links for a node -> closer to the center */

extern void place_linked_node(max_x_size,max_y_size,rand)
int max_x_size, max_y_size;		/* maximum sizes for a node */
int rand;				/* all nodes random coordinates or not */
{
	struct hostentity * dummyhost1;	/* node to determine the number of links for */
	struct hostentity * dummyhost2;	/* are there links to this node */
	struct hostentity * newhost;
	int links;			/* number of links for a node */
	int x_center, y_center;
	long x_rand, y_rand;
	long UTC;

	time (&UTC);
	srandom( (UTC % 10000) );
/* Temporarily */
	srandom (number_of_nodes);
/* TEMPORARILY */

	x_center = WINDOWwidth / 2;	y_center = WINDOWheight / 2;

	dummyhost1 = host;		/* only the nodes with more links (including those who have only links to one_link_nodes) */
	newhost = NULL;			/* only the nodes with more links (to more_linked_nodes) with adjusted linkcount */
	host_more_to_one = NULL;

	while (dummyhost1 != NULL)
	{
		/* adjust sizes for a node */

		if (dummyhost1->width > max_x_size)
			dummyhost1->width = max_x_size;
		if (dummyhost1->height > max_y_size)
			dummyhost1->height = max_y_size;
		links = 0;
		dummyhost2 = host;

		while (dummyhost2 != NULL)
		{
			if (linked_or_not(dummyhost1->id, dummyhost2->id))
				links ++;
			dummyhost2 = dummyhost2->next;
		}

		if (Debug)
			fprintf(stderr,"node %s has %d links\n",dummyhost1->hostname,links);

		/* generate random coordinates
		   Nodes with more links get closer to the center of the window */

		x_rand = random() % 100;		y_rand = random() % 100;
		if (x_rand > 50)
			x_rand -= 100;
		if (y_rand > 50)
			y_rand -= 100;

		if (links == 0)
		{
			fprintf(stderr,"place_linked_node: %s has only links to one_link_nodes\n",dummyhost1->hostname);

			/* make list with nodes with only links to one_link_nodes */

			host_more_to_one = addnodestructure (host_more_to_one, dummyhost1->id, dummyhost1->hostname,
							     dummyhost1->x_coord, dummyhost1->y_coord, dummyhost1->width,
							     dummyhost1->height,dummyhost1->link,TRUE);
		}
		else
		{
			/* give more linked nodes a larger size, make them look more important */

			if ((rand) || (dummyhost1->x_coord == 0))
			{
				dummyhost1->x_coord = x_center + x_rand * ((WINDOWwidth  - boundary) / (50 * links));
				if (dummyhost1->x_coord < boundary)
					dummyhost1->x_coord = boundary;
				if ( (dummyhost1->x_coord + dummyhost1->width) > (WINDOWwidth - boundary))
					dummyhost1->x_coord = WINDOWwidth - boundary - dummyhost1->width;
			}

			if ((rand) || (dummyhost1->y_coord == 0))
			{
				dummyhost1->y_coord = y_center + y_rand * ((WINDOWheight - 2 * boundary) / (50 * links));
				if (dummyhost1->y_coord < boundary)
					dummyhost1->y_coord = boundary;
				if ( (dummyhost1->y_coord + dummyhost1->height) > (WINDOWheight - boundary))
					dummyhost1->y_coord = WINDOWheight - boundary - dummyhost1->height;
			}

			if (Debug)
				fprintf(stderr,"place_linked_nodes: coordinates are %d %d\n",dummyhost1->x_coord, dummyhost1->y_coord);

			/* make temp list with nodes with links and copy this back later
			   change number of links so we know how many one_link_node are connected to it */

			links = dummyhost1->link - links;

			newhost = addnodestructure(newhost,dummyhost1->id, dummyhost1->hostname,dummyhost1->x_coord,
						   dummyhost1->y_coord,dummyhost1->width,dummyhost1->height,links,TRUE);
		}

		dummyhost1 = dummyhost1->next;
	}

	/* make changes known to the original hoststructure */

	host = newhost;
}
