/**************************************************************************************************
 *
 *	onelink.c
 *
 *	part of the layout program.
 *
 *
 *	Written by Gerard Leurs, NIKHEF-H, Amsterdam.
 *	July 1991.
 *	email: a89@nikhef.nl
 *
 *
 *	Change 18-07-1991 : nodes with one link connected to a node with more links are placed
 *			    above the connecting node if this node has a y_coordinate above center
 *			    of the 2D window. Otherwise it is placed under the connecting node.
 *
 *************************************************************************************************/

# include "layout.h"

	/* the nodes with only one link have to be placed near the node with more links it is connected to */
	/* the nodes with one link only connected to other nodes with one link are placed in the border */

extern void place_one_link_nodes(sizecheck)
int sizecheck;				/* check sizes for the boxes */
{
	struct hostentity * dummyhost1;		/* node with one link to determine the coordinates for */
	struct hostentity * dummyhost2;		/* node with more than one link */
	struct hostentity * newhost;
	struct hostentity * dummy;		/* nodes with one link connected to another one_link_node */
	int Linked;				/* one_link_node linked to node_with_more_links */
	int Above_center;			/* more_link_node is on tophalf of window or not */
	int x_offset, y_offset;			/* coordinates for one_link_nodes connected to one_link_node */
	int nodecount;				/* how many of these are there */


	/* get one_link_nodes only connected to another one_link_node */

	dummyhost1 = host_one_link;
	newhost = NULL;
	dummy = NULL;

	while (dummyhost1 != NULL)
	{
		Linked = FALSE;
		dummyhost2 = host;

		while (dummyhost2 != NULL)
		{
			if (strcmp(dummyhost1->hostname,dummyhost2->hostname) != 0)
			{
				if (linked_or_not(dummyhost1->id,dummyhost2->id))
					Linked = TRUE;
			}

			dummyhost2 = dummyhost2->next;
		}

		if (! Linked)
			dummy = addnodestructure(dummy, dummyhost1->id, dummyhost1->hostname, dummyhost1->x_coord,
						 dummyhost1->y_coord, dummyhost1->width, dummyhost1->height,
						 dummyhost1->link, TRUE);
		else
			newhost = addnodestructure(newhost, dummyhost1->id, dummyhost1->hostname, dummyhost1->x_coord,
						   dummyhost1->y_coord, dummyhost1->width, dummyhost1->height,
						   dummyhost1->link, TRUE);
		dummyhost1 = dummyhost1->next;
	}

	/* make (reduced) one_link_nodestructure known */

	host_one_link = newhost;

	/* place the one_link_nodes connected to another one_link_node on the topborder */

	x_offset = 0;	y_offset = WINDOWheight - DEFAULTheight - 10;	nodecount = 0;

	dummyhost1 = dummy;
	newhost = host;

	while (dummyhost1 != NULL)
	{
		dummyhost2 = dummy;

		while (dummyhost2 != NULL)
		{
			if ( (strcmp(dummyhost1->hostname,dummyhost2->hostname) != 0)
			  && (linked_or_not(dummyhost1->id,dummyhost2->id)) )
				{
					if (dummyhost2->y_coord == 0)
					{
						if (sizecheck)
							dummyhost1->x_coord = x_offset + (nodecount * ((WINDOWwidth / 8) + 10));
						else
							dummyhost1->x_coord = x_offset + (nodecount * DEFAULTwidth * 2);
						dummyhost1->y_coord = y_offset;
						nodecount ++;
					}
					else
					{
						dummyhost1->x_coord = dummyhost2->x_coord;
						dummyhost1->y_coord = dummyhost2->y_coord - dummyhost1->height - 10;
					}

					if (sizecheck)
					{
						if ((nodecount * ((WINDOWwidth / 8) + 10)) >= WINDOWwidth)
						{
							y_offset -= (2 * (DEFAULTheight + 10));
							nodecount = 0;
						}
					}
					else if ((nodecount * (DEFAULTwidth * 1.5)) >= WINDOWwidth)
					{
						y_offset -= (2 * (DEFAULTheight + 10));
						nodecount = 0;
					}
					

					newhost = addnodestructure(newhost, dummyhost1->id, dummyhost1->hostname,
								   dummyhost1->x_coord, dummyhost1->y_coord,
								   dummyhost1->width, dummyhost1->height,
								   dummyhost1->link, FALSE);
				}

			dummyhost2 = dummyhost2->next;
		}

		dummyhost1 = dummyhost1->next;
	}

	/* save nodes with more links, also the nodes only connected to one_link_nodes
	   and one_link_nodes connected to the "special" more_link_nodes
	   and one_link_nodes only connected to another one_link node */

	host = newhost;
	newhost = NULL;

	/* place the rest of the one_link_nodes close above the more_links_node (tophalf of window)
	   or direct under the more_links_node (bottomhalf of the window) */

	dummyhost1 = host;

	while (dummyhost1 != NULL)
	{
	if ((dummyhost1->y_coord + (dummyhost1->height / 2) + (dummyhost1->link * (DEFAULTheight + 10) / 2)) >=
	    (WINDOWheight / 2))
		Above_center = TRUE;
	else
	{
		Above_center = FALSE;
		dummyhost1->y_coord += dummyhost1->height + (dummyhost1->link * (DEFAULTheight + 10));
	}
		nodecount = 1;
		dummyhost2 = host_one_link;

		while (dummyhost2 != NULL)
		{
			if ( (strcmp(dummyhost1->hostname,dummyhost2->hostname) != 0)
			  && (linked_or_not(dummyhost1->id,dummyhost2->id)) )
			{
				dummyhost2->x_coord = dummyhost1->x_coord;
				if (Above_center)
					dummyhost2->y_coord = dummyhost1->y_coord + dummyhost1->height
						    + (nodecount * (dummyhost2->height + 10));
				else
					dummyhost2->y_coord = dummyhost1->y_coord - dummyhost1->height
						    - (nodecount * (dummyhost2->height + 10));
				newhost = addnodestructure(newhost, dummyhost2->id, dummyhost2->hostname, dummyhost2->x_coord,
							   dummyhost2->y_coord, dummyhost2->width, dummyhost2->height,
							   dummyhost1->link, TRUE);
				nodecount ++;
			}

			dummyhost2 = dummyhost2->next;
		}

		if (Debug)
			fprintf(stderr,"nr of connecting one_link_nodes to %s:%d\n",dummyhost1->hostname,(nodecount - 1));

		dummyhost1 = dummyhost1->next;
	}

	/* make additions of one_link_nodes connected to the more_links_nodes known */

	dummyhost1 = newhost;
	newhost = host;

	while (dummyhost1 != NULL)
	{
		newhost = addnodestructure(newhost, dummyhost1->id, dummyhost1->hostname, dummyhost1->x_coord, dummyhost1->y_coord,
					      dummyhost1->width, dummyhost1->height, dummyhost1->link, FALSE);
		dummyhost1 = dummyhost1->next;
	}
}
