/**************************************************************************************************
 *
 *	output.c
 *
 *	part of the layout program.
 *
 *
 *	Written by Gerard Leurs, NIKHEF-H, Amsterdam.
 *	July 1991.
 *	email: a89@nikhef.nl
 *
 *************************************************************************************************/

# include "layout.h"

	/* printroutine for the nodes, since we now know the new coordinates */

extern void print_node()
{
	struct hostentity * dummyhost1;

	dummyhost1 = host;

	while (dummyhost1 != NULL)
	{
		printf("%s %s %d %d %d %d\n",dummyhost1->id, dummyhost1->hostname, dummyhost1->x_coord,
			dummyhost1->y_coord, dummyhost1->width, dummyhost1->height);
		dummyhost1 = dummyhost1->next;
	}
}


	/* printroutine for the interfaces */

extern void print_link()
{
	struct linkentity * dummylink1;

	dummylink1 = link;

	while (dummylink1 != NULL)
	{
		printf("%s %s %s\n",dummylink1->linktype, dummylink1->id_one , dummylink1->id_two);
		dummylink1 = dummylink1->next;
	}
}


	/* find a good place to position the interface between the two connecting nodes
	   if the new position is a straight line choose that */

static void choose_attachment(x1, y1, w1, h1, x2, y2, w2, h2, x_int, y_int, w_int, h_int)
int x1, y1, x2, y2;		/* coordinates of the two connecting nodes */
int w1, h1, w2, h2;		/* sizes of the two connecting nodes */
int *x_int, *y_int;		/* coordinates of the interface */
int w_int, h_int;		/* sizes of the interface box */
{
	int xr1, yr1, xr2, yr2;

	nearest_point(x1, x2, w1, w2, &xr1, &xr2);
	nearest_point(y1, y2, h1, h2, &yr1, &yr2);

	if (xr1 == xr2)
	{
		if (Debug)
			fprintf(stderr,"choose_attachment (a): straight line (%d,%d) - (%d,%d) [%d %d]\n",
				x1,y1,x2,y2,xr1,yr1);
		*x_int = xr1;		*y_int = (yr1 + yr2) / 2;
	}
	else if (yr1 == yr2)
	{
		if (Debug)
			fprintf(stderr,"choose_attachment (b): straight line (%d,%d) - (%d,%d) [%d %d]\n",
				x1,y1,x2,y2,xr1,yr1);
		*x_int = (xr1 + xr2) / 2;		*y_int = yr1;
	}
	else
	{
		*x_int = xr1 + (xr2 - xr1);
		if (x1 >= x2)
			*x_int -= 10;
		else
			*x_int += 10;
		if (y1 >= y2)
			*y_int = yr1 - (h1 / 2);
		else
			*y_int = yr1 + (h1 / 2);

		if (Debug)
			fprintf(stderr,"choose_attachment: interface placed beside first node (%d %d)\n",
				xr1, yr1, xr2, yr2, *x_int, *y_int);
	}
}


	/* make outputfile with postscript commands */

extern void make_postscript()
{
	struct hostentity * dummyhost1;	/* hoststructure */
	struct hostentity * dummyhost2;	/* hoststructure */
	FILE *psfile;			/* outputfile */
	char outps[MaxEnt];		/* outputfilename */
	int x1, y1, w1, h1;		/* coordinates and sizes */
	int x2, y2, w2, h2;		/* coordinates and sizes */
	int x1_center, y1_center, x2_center, y2_center;	/* center of a node */
	int x_int, y_int;		/* center for a interface between node 1 and node 2 */
	int INTwidth = 4;		/* width for an interface */
	long UTC;
	float xfact, yfact;		/* scalefactor for a nice postscript picture on a a4 */
	int fontsize = DEFAULTheight - 4;	/* fontsize for nodename */
	int nodecount, count;


	fprintf(stderr,"\ncreating postscriptfile /tmp/psfile with the layoutresults\n");

	if ((psfile=fopen("/tmp/psfile","w")) == NULL)
	{
		fprintf(stderr,"I can't create /tmp/psfile !!\n");
		exit(1);
	}

	time(&UTC);

	/* start of postscriptfile */

	fprintf(psfile,"%%!\n");
	fprintf(psfile,"%% postscript output from layout dd. %s",ctime(&UTC));
	fprintf(psfile,"/mt {moveto} def\n");
	fprintf(psfile,"/lt {lineto} def\n");
	fprintf(psfile,"/slw {setlinewidth} def\n");
	fprintf(psfile,"/gy {setgray} def\n");
	fprintf(psfile,"/sk {stroke} def\n");
	fprintf(psfile,"%% Print text in Times-Roman %d\n",fontsize);
	fprintf(psfile,"/Times-Roman findfont %d scalefont setfont\n",fontsize);

	if (WINDOWwidth <= 500 )
		xfact = 1.0;
	else
	{
		xfact = (float) WINDOWwidth / 500;
		xfact = 1.0 / xfact;
	}

	if (WINDOWheight <= 550 )
		yfact = 1.0;
	else
	{
		yfact = (float) WINDOWheight / 550;
		yfact = 1.0 / yfact;
	}

	fprintf(psfile,"%3.2f %3.2f scale\n",xfact, yfact);
	fprintf(psfile,"90 rotate\n");
	fprintf(psfile,"75 -%d translate\n", WINDOWwidth);

	/* windowframe */

	fprintf(psfile,"1 slw\n");
	fprintf(psfile,"0.1 gy\n");
	fprintf(psfile,"0 0 mt\n");
	fprintf(psfile,"%d 0 lt\n",WINDOWwidth);
	fprintf(psfile,"%d %d lt\n",WINDOWwidth,WINDOWheight);
	fprintf(psfile,"0 %d lt\n",WINDOWheight);
	fprintf(psfile,"0 0 lt\n");

	/* border (used for unlinked nodes / one_link_node connected to another one_link_node) */

	fprintf(psfile,"%d %d mt\n",boundary,boundary);
	fprintf(psfile,"%d %d lt\n",(WINDOWwidth - boundary),boundary);
	fprintf(psfile,"%d %d lt\n",(WINDOWwidth - boundary),(WINDOWheight - boundary));
	fprintf(psfile,"%d %d lt\n",boundary,(WINDOWheight - boundary));
	fprintf(psfile,"%d %d lt\n",boundary,boundary);
	fprintf(psfile,"sk\n");

	/* where are the linked nodes */

	nodecount = 0;
	dummyhost1 = host;

	while (dummyhost1 != NULL)
	{
		nodecount ++;
		x1 = dummyhost1->x_coord;	y1 = dummyhost1->y_coord;
		w1 = dummyhost1->width;		h1 = dummyhost1->height;

		fprintf(psfile,"1 slw\n");
		fprintf(psfile,"0.1 gy\n");
		fprintf(psfile,"%5.1f %5.1f mt\n",(float) (x1 + ((w1 - (strlen(dummyhost1->hostname) * 7)) / 2)),
				(float) (y1 + ((h1 - fontsize) / 2)));
		fprintf(psfile,"(%s) show\n",dummyhost1->hostname);
		fprintf(psfile,"%3d %3d mt\n",x1, y1);
		fprintf(psfile,"%3d %3d lt\n",(x1 + w1), y1);
		fprintf(psfile,"%3d %3d lt\n",(x1 + w1), (y1 + h1));
		fprintf(psfile,"%3d %3d lt\n",x1, (y1 + h1));
		fprintf(psfile,"%3d %3d lt\n",x1, y1);
		fprintf(psfile,"sk\n");

		/* lines and interfaces between connecting nodes */

		dummyhost2 = host;

		for (count=1; count <= nodecount ;count++)
			dummyhost2 = dummyhost2->next;

		while (dummyhost2 != NULL)
		{
			if ( (strcmp(dummyhost1->hostname,dummyhost2->hostname) != 0)
			   && (linked_or_not(dummyhost1->id,dummyhost2->id)) )
			{
				x2 = dummyhost2->x_coord;	y2 = dummyhost2->y_coord;
				w2 = dummyhost2->width;		h2 = dummyhost2->height;

				nearest_point(x1, x2, w1, w2, &x1_center, &x2_center);
				nearest_point(y1, y2, h1, h2, &y1_center, &y2_center);

				choose_attachment(x1, y1, w1, h1, x2, y2, w2, h2, &x_int, &y_int, INTwidth, INTwidth);

				fprintf(psfile,"0.5 slw\n");
				fprintf(psfile,"0.3 gy\n");
				fprintf(psfile,"%3d %3d mt\n",x1_center, y1_center );
				fprintf(psfile,"%3d %3d lt\n",x_int, y_int);
				fprintf(psfile,"%3d %3d lt\n",x_int, y2_center);

				fprintf(psfile,"%3d %3d mt\n",x_int , y_int );
				fprintf(psfile,"%3d %3d lt\n",(x_int + INTwidth), y_int);
				fprintf(psfile,"%3d %3d lt\n",(x_int + INTwidth), (y_int + INTwidth));
				fprintf(psfile,"%3d %3d lt\n",x_int, (y_int + INTwidth));
				fprintf(psfile,"%3d %3d lt\n",x_int, y_int);
				fprintf(psfile,"sk\n");
			}

			dummyhost2 = dummyhost2->next;
		}

		dummyhost1 = dummyhost1->next;
	}

	/* where are the unlinked nodes */

	dummyhost1 = host_zero_links;

	while (dummyhost1 != NULL)
	{
		if (Debug)
			fprintf(stderr,"postscript for UNlinked nodes%s\n",dummyhost1->hostname);

		x1 = dummyhost1->x_coord;	y1 = dummyhost1->y_coord;
		w1 = dummyhost1->width;		h1 = dummyhost1->height;

		fprintf(psfile,"1 slw\n");
		fprintf(psfile,"0.1 gy\n");
		fprintf(psfile,"%5.1f %5.1f mt\n",(float) (x1 + ((w1 - (strlen(dummyhost1->hostname) * 7)) / 2)),
				(float) (y1 + ((h1 - fontsize) / 2)));
		fprintf(psfile,"(%s) show\n",dummyhost1->hostname);
		fprintf(psfile,"%3d %3d mt\n",x1, y1);
		fprintf(psfile,"%3d %3d lt\n",(x1 + w1), y1);
		fprintf(psfile,"%3d %3d lt\n",(x1 + w1), (y1 + h1));
		fprintf(psfile,"%3d %3d lt\n",x1, (y1 + h1));
		fprintf(psfile,"%3d %3d lt\n",x1, y1);
		fprintf(psfile,"sk\n");

		dummyhost1 = dummyhost1->next;
	}

	/* end of postscriptfile */

	fprintf(psfile,"sk\n");
	fprintf(psfile,"showpage\n");
}
