/**************************************************************************************************
 *
 *	overlap.c
 *
 *	part of the layout program.
 *
 *
 *	Written by Gerard Leurs, NIKHEF-H, Amsterdam.
 *	July 1991.
 *	a89@nikhef.nl
 *
 *
 *	Change 18-07-91 : left or right placement depends on left / right position (-center)
 *			  of the connecting node(s).
 *	Change 18-07-91 : For the nodes with only one link to another node with more links:
 *			  when it is on the bottompart of the window : one_link_nodes beneath them.
 *	Change 23-07-91 : check overlap for nodes after forcevector determination.
 *
 *************************************************************************************************/

# include "layout.h"

	/* check linked nodes : overlapping new coordinates */
	/* check enough space to place connected one_link_nodes for the more_linked_nodes */

extern void overlap_coordinates(overlap_check)
int *overlap_check;			/* checks number of overlap */
{
	struct hostentity * dummyhost1;	/* host to determine the changes for */
	struct hostentity * dummyhost2;	/* the host that interacts with dummyhost1 */
	struct hostentity * newhost;
	int x1, y1, w1, h1;		/* temp. values for host1 */
	int x2, y2, w2, h2;		/* temp. values for host2 */
	int new_x;			/* new x_coordinate for the node */
	int new_y;			/* new y_coordinate for the node */
	int count;			/* nodes to skip before force determination */
	int nodecount = 0;		/* number of nodes already determined */
	int x_distance, y_distance;	/* minimum distance if overlapping */
	int overlaps = 0;		/* the name says it */
                                                                     
	fprintf(stderr,"Fixing overlapping coordinates\n");

	*overlap_check = FALSE;

	newhost = NULL;
	dummyhost1 = host;

	while (dummyhost1 != NULL)
	{
		if (Debug)
			fprintf(stderr,"overlap_coordinates: working on %s\n",dummyhost1->hostname);

		dummyhost2 = host;

		x_distance = 10;
		y_distance = 10;

		new_x = dummyhost1->x_coord;
		new_y = dummyhost1->y_coord;

		nodecount ++;
		for (count = 1 ; count <= nodecount ; count ++)
			dummyhost2 = dummyhost2->next;

		while (dummyhost2 != NULL)
		{
			if (strcmp(dummyhost1->hostname,dummyhost2->hostname) != 0)
			{
				x1 = dummyhost1->x_coord;	y1 = dummyhost1->y_coord;
				w1 = dummyhost1->width;
				if (dummyhost1->link > 0)
					h1 = dummyhost1->height + (DEFAULTheight * (1 + dummyhost1->link));
				else
					h1 = dummyhost1->height * 2;
				x2 = dummyhost2->x_coord;	y2 = dummyhost2->y_coord;
				w2 = dummyhost2->width;
				if (dummyhost2->link > 0)
					h2 = dummyhost2->height + (DEFAULTheight * (1 + dummyhost2->link));
				else
					h2 = dummyhost2->height * 2;

			/* overlap in x-coordinates (width) -> check y-coordinates */

				if ( ((x1 <= x2) && (x1 + w1 >= x2)) || ((x2 <= x1) && (x2 + w2 >= x1)) )
				{
					if ( (y1 <= y2) && ((y1 + h1) >= y2) )
					{
						if (Debug)
							fprintf(stderr,"y_coord. overlap for %s\n",dummyhost1->hostname);

						*overlap_check = TRUE;		overlaps ++;
						new_y = y1 - ((y1 + h1) - y2) - y_distance;
						if (new_y > (WINDOWheight - boundary - h1))
							new_y = WINDOWheight - boundary - h1 - 50;
						if (new_y < boundary)
							new_y = boundary + 50;
					}
					else if ( (y2 <= y1) && ((y2 + h2) >= y1) )
					{
						if (Debug)
							fprintf(stderr,"y_coord. overlap for %s\n",dummyhost1->hostname);

						*overlap_check = TRUE;		overlaps ++;
						new_y = y1 + ((y2 + h2) - y1) + y_distance;
						if (new_y > (WINDOWheight - boundary - h1))
							new_y = WINDOWheight - boundary - h1 - 50;
						if (new_y < boundary)
							new_y = boundary + 50;
					}

				}

			/* overlap in y-coordinates (height) -> check x-coordinates */

				if ( ((y1 <= y2) && (y1 + h1 >= y2)) || ((y2 <= y1) && (y2 + h2 >= y1)) )
				{
					if ( (x1 <= x2) && ((x1 + w1) >= x2) )
					{
						if (Debug)
							fprintf(stderr,"x_coord. overlap for %s\n",dummyhost1->hostname);

						*overlap_check = TRUE;		overlaps ++;
						new_x = x1 - ((x1 + w1) - x2) - x_distance;
						if (new_x > (WINDOWwidth - boundary - w1))
							new_x = WINDOWwidth - boundary - w1 - 50;
						if (new_x < boundary)
							new_x = boundary + 50;
					}
					else if ( (x2 <= x1) && ((x2 + w2) >= x1) )
					{
						if (Debug)
							fprintf(stderr,"x_coord. overlap for %s\n",dummyhost1->hostname);

						*overlap_check = TRUE;		overlaps ++;
						new_x = x1 + ((x2 + w2) - x1) + x_distance;
						if (new_x > (WINDOWwidth - boundary - w1))
							new_x = WINDOWwidth - boundary - w1 - 50;
						if (new_x < boundary)
							new_x = boundary + 50;
					}
				}
			}

			dummyhost2 = dummyhost2->next;
		}

		newhost = addnodestructure(newhost,dummyhost1->id, dummyhost1->hostname, new_x, new_y,
					   dummyhost1->width, dummyhost1->height, dummyhost1->link, TRUE);

		dummyhost1 = dummyhost1->next;
	}

	/* after the complete cycle : make changes known to the original hoststructure */

	host = newhost;

	fprintf(stderr,"Total number of overlaps (x and y) is %d\n",overlaps);
}


	/* overlap between more_links_node and the more_links_node with two connections to
	   more_links_node */

extern int check_nodes(hoststr,node,x,y)
struct hostentity * hoststr;
struct hostentity * node;
int x, y;
{
	struct hostentity * dummyhost1;
	int w, h;
	int x1, y1, w1, h1;

	if (Debug)
		fprintf(stderr,"check_nodes: trying to place %s at %d %d\n",node->hostname,x,y);

	dummyhost1 = hoststr;
	while (dummyhost1 != NULL)
	{
		if ((strcmp(dummyhost1->hostname,node->hostname)) != 0)
		{
			w = node->width;
			if (node->link > 0)
				h = node->height + (DEFAULTheight * (1 + node->link));
			else
				h = node->height * 2;
			x1 = dummyhost1->x_coord;	y1 = dummyhost1->y_coord;	w1 = dummyhost1->width;
			if (dummyhost1->link > 0)
				h1 = dummyhost1->height + (DEFAULTheight * (1 + dummyhost1->link));
			else
				h1 = dummyhost1->height * 2;

			/* overlap in x-coordinates (width) -> check y-coordinates */

			if ( ((x <= x1) && (x + w >= x1)) || ((x1 <= x) && (x1 + w1 >= x)) )
			{
				if ( (y <= y1) && ((y + h) >= y1) )
					return FALSE;
				else if ( (y1 <= y) && ((y1 + h1) >= y) )
					return FALSE;

			}

			/* overlap in y-coordinates (height) -> check x-coordinates */

			if ( ((y <= y1) && (y + h >= y1)) || ((y1 <= y) && (y1 + h1 >= y)) )
			{
				if ( (x <= x1) && ((x + w) >= x1) )
					return FALSE;
				else if ( (x1 <= x) && ((x1 + w1) >= x) )
					return FALSE;
			}
		}

		dummyhost1 = dummyhost1->next;
	}
	return TRUE;
}


	/* overlap between already changed nodes (after forcevector detemination)
	   and the node of which we have just determined the new coordinates for */

extern void check_overlap_layout(newhost, node, new_x, new_y, x_change, y_change)
struct hostentity * newhost;
struct hostentity * node;
int *new_x, *new_y;
int x_change, y_change;
{
	struct hostentity * dummyhost1;
	int No_Overlap, Overlap;
	int Found = FALSE;
	int save_x, save_y;
	int x, y;			/* temp. coordinates (new -> old) */
	int x_step, step_nr;

	x_step = x_change / 3;
	if (x_step == 0)
	{
		if (x_change < 0)
			x_step = -1;
		else if (x_change >= 0)
			x_step = 1;
	}
	step_nr = 0;

	/* check overlap for all coordinates, starting at the new and ending
	   at the old ones. The first one found without overlapping other
	   already changed nodes is returned */

	x = *new_x;	y = *new_y;
	if (x_change >= 0)
	{
		while ( (! Found) && (x <= (*new_x + x_change)) && (step_nr < 3))
		{
			y += step_nr * (y_change / 3);
			step_nr ++;
			dummyhost1 = newhost;
			if (dummyhost1 == NULL)
			{
				Found = TRUE;	save_x = x;	save_y = y;
			}
			if (!Found)
			{
				No_Overlap = check_nodes(newhost, node, x, y);
				if (No_Overlap)
				{
					Found = TRUE;	save_x = x;	save_y = y;
				}
			}
			x += x_step;
		}
	}
	else
	{
		while ( (! Found) && (x <= (*new_x - x_change)) && (step_nr < 3))
		{
			y = *new_y + step_nr * (y_change / 3);
			step_nr ++;
			dummyhost1 = newhost;
			if (dummyhost1 == NULL)
			{
				Found = TRUE;	save_x = x;	save_y = y;
			}
			if (!Found)
			{
				No_Overlap = check_nodes(newhost, node, x, y);
				if (No_Overlap)
				{
					Found = TRUE;	save_x = x;	save_y = y;
				}
			}
			x += x_step;
		}
	}

	if (Found)
	{
		*new_x = save_x;	*new_y = save_y;
	}
	else if (Debug)
		fprintf(stderr,"Overlap occurs for %s at %d,%d (even with stepwise overlapcheck)\n",
				node->hostname, *new_x, *new_y);
}


	/* for a node with only one link to another node with more links
	   check overlap in an area close to the connecting more_links_node */

extern void check_overlap_1(node1, node2, new_x, new_y)
struct hostentity * node1;
struct hostentity * node2;
int *new_x, *new_y;
{
	int save_x, save_y;
	int cont = 1;
	int No_Overlap;
	int Found = FALSE;

	/* try some very obvious spots : under(3x) / left / right */

	while ((! Found) && (cont++ <= 3))
	{
		*new_x = node2->x_coord;
		if (node2->y_coord >= (WINDOWheight / 2))
		{
			if (node1->link > 0)
				*new_y = node2->y_coord - node1->height - (((2 * cont) + node1->link) * (DEFAULTheight + 10));
			else
	 			*new_y = node2->y_coord - node1->height - ((2 * cont) * (DEFAULTheight + 10));
		}
		else
			*new_y = node2->y_coord - node1->height - (3 * cont * (DEFAULTheight + 10));
		if (Debug)
			fprintf(stderr,"check_overlap: (step 1a-%d) %s at %d %d\n",cont,node1->hostname, *new_x, *new_y);
		No_Overlap = check_nodes(host, node1, *new_x, *new_y);
		if (No_Overlap)
		{
			Found = TRUE;	save_x = *new_x;	save_y = *new_y;
		}
	}

	if (! Found)
	{
		*new_y = node2->y_coord;
		if ((node2->x_coord + (node1->width / 2)) <= (WINDOWwidth / 2))
			*new_x = node2->x_coord - node1->width - 10;
		else
			*new_x = node2->x_coord + node1->width + 10;
		if (Debug)
			fprintf(stderr,"check_overlap: (step 1b) %s at %d %d\n",node1->hostname, *new_x, *new_y);
		No_Overlap = check_nodes(host, node1, *new_x, *new_y);
		if (No_Overlap)
		{
			Found = TRUE;	save_x = *new_x;	save_y = *new_y;
		}
	}

	if (! Found)
	{
		*new_y = node2->y_coord;
		if ((node2->x_coord + (node1->width / 2)) <= (WINDOWwidth / 2))
			*new_x = node2->x_coord + node1->width + 10;
		else
			*new_x = node2->x_coord - node1->width - 10;
		if (Debug)
			fprintf(stderr,"check_overlap: (step 1c) %s at %d %d\n",node1->hostname, *new_x, *new_y);
		No_Overlap = check_nodes(host, node1, *new_x, *new_y);
		if (No_Overlap)
		{
			Found = TRUE;	save_x = *new_x;	save_y = *new_y;
		}
	}

	if (Found)
	{
		*new_x = save_x;	*new_y = save_y;
	}
	else
	{
		*new_x = -100;		*new_y = -100;
	}
}


	/* check overlap in a certain area (used for nodes with two links to more_link_nodes) */

extern void check_overlap_2(node, x1, x2, y1, y2, new_x, new_y,check_or_not)
struct hostentity * node;	/* node to be placed somewhere in this area */
int x1, x2, y1, y2;		/* coordinates of the area in which this node must be placed */
int *new_x, *new_y;		/* coordinates for the node */
int check_or_not;
{
	int possible_x, possible_y, xlow, xhigh, ylow, yhigh;
	int save_x, save_y, x_step, y_step;
	float fact;
	int No_Overlap;
	int Found = FALSE;

	/* first try some spots very close to halfway the two connecting nodes
	   check overlap for node halfway x and y of the two connecting more_links_nodes
	   and check overlap at one-third and also at twothird */

	*new_x = (x1 + x2) / 2 - node->width / 2;		*new_y = (y1 + y2) / 2 - node->height / 2;
	if (Debug)
		fprintf(stderr,"check_overlap: (step 1a) %s at %d %d\n",node->hostname, *new_x, *new_y);
	No_Overlap = check_nodes(host, node, *new_x, *new_y);
	if (No_Overlap)
	{
		Found = TRUE;	save_x = *new_x;	save_y = *new_y;
	}

	if (! Found)
	{
		*new_x = (x1 + x2) / 2 - node->width / 2;
		*new_y = (y1 + y2) / 2 - node->height  - (node->link * (DEFAULTheight + 10));
		if (Debug)
			fprintf(stderr,"check_overlap: (step 1b) %s at %d %d\n",node->hostname, *new_x, *new_y);
		No_Overlap = check_nodes(host, node, *new_x, *new_y);
		if (No_Overlap)
		{
			Found = TRUE;	save_x = *new_x;	save_y = *new_y;
		}
	}

	if (! Found)
	{
		*new_x = (x1 + x2) / 2 - node->width / 2;
		*new_y = (y1 + y2) / 2 + (2 * (DEFAULTheight + 10));
		if (Debug)
			fprintf(stderr,"check_overlap: (step 1c) %s at %d %d\n",node->hostname, *new_x, *new_y);
		No_Overlap = check_nodes(host, node, *new_x, *new_y);
		if (No_Overlap)
		{
			Found = TRUE;	save_x = *new_x;	save_y = *new_y;
		}
	}

	if (! Found)
	{
		if (((x1 + x2) / 2) <= (WINDOWwidth / 2))
			fact = 0.33;
		else
			fact = 0.67;
		if (x2 > x1)
			*new_x = x1 + (x1 - x2) * fact - node->width / 2;
		else
			*new_x = x2 + (x2 - x1) * fact - node->width / 2;
		if (y2 > y1)
			*new_y = y1 + (y1 - y2) * fact - node->height / 2;
		else
			*new_y = y2 + (y2 - y1) * fact - node->height / 2;
		if (Debug)
			fprintf(stderr,"check_overlap: (step 1d) %s at %d %d\n",node->hostname, *new_x, *new_y);
		No_Overlap = check_nodes(host, node, *new_x, *new_y);
		if (No_Overlap)
		{
			Found = TRUE;	save_x = *new_x;	save_y = *new_y;
		}
	}

	if (! Found)
	{
		if (((x1 + x2) / 2) <= (WINDOWwidth / 2))
			fact = 0.67;
		else
			fact = 0.33;
		if (x2 > x1)
			*new_x = x1 + (x1 - x2) * fact - node->width / 2;
		else
			*new_x = x2 + (x2 - x1) * fact - node->width / 2;
		if (y2 > y1)
			*new_y = y1 + (y1 - y2) * fact - node->height / 2;
		else
			*new_y = y2 + (y2 - y1) * fact - node->height / 2;
		if (Debug)
			fprintf(stderr,"check_overlap: (step 1e) %s at %d %d\n",node->hostname, *new_x, *new_y);
		No_Overlap = check_nodes(host, node, *new_x, *new_y);
		if (No_Overlap)
		{
			Found = TRUE;	save_x = *new_x;	save_y = *new_y;
		}
	}

	/* check overlap for node along the x_axis (halfway y_coord of the two connecting nodes) */

	if (check_or_not)
	{
		if (! Found)
		{
			if (Debug)
				fprintf(stderr,"check_overlap: (step 2) %s at %d %d\n",node->hostname, *new_x, *new_y);
			if (x1 < x2)
			{
				xlow = x1;	xhigh = x2;
			}
			else if (x1 > x2)
			{
				xlow = x2;	xhigh = x1;
			}
			else
			{
				xlow = x1 - 25;	xhigh = x2 + 25;
			}
			x_step = (xhigh - xlow) / 7;
			if (x_step < 1)
				x_step = 1;
			*new_y = (y1 + y2) / 2 - (node->height / 2);

			for (possible_x = xlow; possible_x <= xhigh; possible_x += x_step)
			{
				*new_x = possible_x;
				No_Overlap = check_nodes(host, node, *new_x, *new_y);
				if (No_Overlap)
				{
					Found = TRUE;	save_x = *new_x;	save_y = *new_y;
				}
			}
		}

	/* check overlap for node along the y_axis (halfway x_coord of the two connecting nodes) */

		if (! Found)
		{
			if (Debug)
				fprintf(stderr,"check_overlap: (step 3) %s at %d %d\n",node->hostname, *new_x, *new_y);
			if (y1 < y2)
			{
				ylow = y1;	yhigh = y2;
			}
			else if (y1 > y2)
			{
				ylow = y2;	yhigh = y1;
			}
			else
			{
				ylow = y1 - 25;	yhigh = y2 + 25;
			}
			y_step = (yhigh - ylow) / 7;
			if (y_step < 1)
				y_step = 1;
			*new_x = (x1 + x2) / 2 - (node->width / 2);

			for (possible_y = ylow; possible_y <= yhigh; possible_y += y_step)
			{
				*new_y = possible_y;
				No_Overlap = check_nodes(host, node, *new_x, *new_y);
				if (No_Overlap)
				{
					Found = TRUE;	save_x = *new_x;	save_y = *new_y;
				}
			}
		}

	/* check overlap for node at the corners of the box surrounded by the two connecting nodes */

		if (! Found)
		{
			*new_x = x1 - node->width / 2;		*new_y = y2 - node->height / 2;
			if (Debug)
				fprintf(stderr,"check_overlap: (step 4) %s at %d %d\n",node->hostname, *new_x, *new_y);
			No_Overlap = check_nodes(host, node, *new_x, *new_y);
			if (No_Overlap)
			{
				Found = TRUE;	save_x = *new_x;	save_y = *new_y;
			}
			else
			{
				*new_x = x2 - node->width / 2;		*new_y = y1 - node->height / 2;
				if (Debug)
					fprintf(stderr,"check_overlap: (step 5) %s at %d %d\n",node->hostname, *new_x, *new_y);
				No_Overlap = check_nodes(host, node, *new_x, *new_y);
				if (No_Overlap)
				{
					Found = TRUE;	save_x = *new_x;	save_y = *new_y;
				}
			}
		}
	}

	if (Found)
	{
		*new_x = save_x;	*new_y = save_y;
	}
	else
	{
		*new_x = -100;		*new_y = -100;
	}
}
