/**********************************************************/
/**********************************************************/
/*  $Header: //gaia/group.local/computer/monet/Monetd/RCS/request.c,v 1.2 1991/03/22 16:09:27 a38 Exp a38 $  */
/**********************************************************/
/**********************************************************/

#include "monetd.h"

/***************************************************************************/

struct request {
	Connection connection;
	RequestId id;
	Request connection_next;
	Request entity_next;
	Entity entity;
	TTime frequency;       /* frequency of queries */
	TTime critical_after;  /* seconds system will go critical after */
};

extern void return_selected (request)
	Request request;
{
	Connection connection = request->connection;
	char * message;
	message = format_result (time (NULL), request, StatusAllMask);
	debug_message ("Sending [%s] to connection %d", message, connection->fid);
	ResetStatusMask (V_STATUS(request->entity));
	Channel_send (connection->channel, message);
	Channel_end_of_record (connection->channel);
	return;
}

/**********************************************************/
/*  Request have a set of options that must be parsed     */
/**********************************************************/

static void parse_request_options (request, options)
	Request request;
	String options;
{
	String pos;
	if (! options)
		return;

	request -> critical_after = 0;
	request -> frequency = 0;
	pos = options;
	while (*pos) {
		int cc;
		String text;
		char name[1024]; /* !!! */
		unsigned long value;

		text = MonetNextWord (&pos);
		if ((cc = sscanf (text, "%[a-zA-Z_]%U", name, &value)) != 2) {
		} else if (EQS(name, "critical")) {
			request -> critical_after = value;
		} else if (EQS(name, "frequency")) {
			request -> frequency = value;
		}
	}
	debug_message ("options[%s] critical%u frequency%u",
		options, request->critical_after, request->frequency);
	if (request->entity->query && request -> frequency > 0)
		Query_set_frequency (request->entity->query, (TTime) request->frequency);
}

/**********************************************************/
/*  Find or create a entity to associate with a new request  */
/**********************************************************/

extern void make_request (connection, name, id, options)
	Connection connection;
	RequestId id;
	String name, options;
{
	Request request;
	Entity entity;
	String ping;

	if (id == 0) {
		user_error ("Invalid request id [%d]", id);
		return;
	} else 	if (id2request (connection, id) != NULL) {
		user_error ("Request %d:%d already exists", connection->fid, id);
		return;
	} else if ((request = MonetNew (struct request)) == NULL) {
		user_error ("Could not allocate space for new request");
		return;
	}

	/*  Find entity in list or create it  */
	/*  if it has a ping instruction then create a query for it */
	if ( !(entity = Entity_find_or_create (name))) {
		user_error ("Could not create entity [%s]", name);
		MonetFree (request);
		return;
	}
	if (Entity_isClass (entity))
		return;

	if (entity -> query == NULL && (ping = get_ping_instruction (entity))) {
		char class[1024], name[1024], options[1024]; /* !!! */
		int cc;
		cc = sscanf (ping, "%[A-Za-z_0-9.]:%s [^ \n\t]", class, name, options);
		if (cc < 2) {
			user_error ("Invalid ping instruction [%s]", ping);
		} else if (entity -> query = Query_create (class, entity, name)) {
			request->entity = entity;
			if (cc == 3)
				parse_request_options (request, options);
		} else {
			user_error ("Could not create query for [%s:%s]", class, name);
		}
	}
	request -> id = id;
	LIST_ADD (request, connection->requests, connection_next);
	request -> connection = connection;
	LIST_ADD (request, entity->requests, entity_next);
	request -> entity = entity;
/*	if (entity->requests == NULL)
		write_log (PingerLog, "REQUEST", "New Request [%s] for request managed object", request2text (request));
	else if (connection->fid > 0) {
		write_log (PingerLog, "REQUEST", "New Request [%s] for existing managed object", request2text (request));
  	}
*/
	parse_request_options (request, options);

	/* -- Send current status to client straight away  */
	return_selected (request);
}

/****************************************************************************/
/*  Format result message  */
/****************************************************************************/

extern char * format_result (now, r, selection)
	Request r;
	unsigned long int now;
	StatusMask selection;
{
	static char buffer [4096] = { 0 }; /* !!! */
	ReachStatus status;
	Entity entity = r->entity;
	StatusMask mask;
	String message;

	if (r->connection->format == attribute_ascii) {
		(void) strcpy (buffer, Entity2Report (r->entity, False));
		return buffer;
	}
	status = STATUS(entity);
	mask   = MASK(entity);

	MASK(entity) |= selection;	
	if (MASK(entity) & StatusReachMask) {
		if (STATUS(entity) == status_unreachable && now - V_STATUS(entity)->time_stamp >= r->critical_after && r->critical_after > 0)
			STATUS(entity) = status_critical;
	}
	if (message = Entity_encodeCompactAscii (entity, selection))
		(void) sprintf (buffer, "%ld%s", r->id, message);
	MASK(entity)   = mask;
	STATUS(entity) = status;

	return buffer;
}

/****************************************************************************/
/*  Return all the results to the clients */
/****************************************************************************/

extern void return_all_results ()
{
	long now;
	Connection c;
	Request r;
	char * buffer, buf[8192];
	int len, total_len;

	now = time (NULL);
	FORALL (c, ConnectionList, next) {
		if (c ->fid <= 0)
			continue;

		*buf = '\0';
		total_len = 0;
		FORALL (r, c->requests, connection_next) {
			buffer = format_result (now, r, StatusNoneMask);
			len = MonetStrlen (buffer);
			total_len += len;
			if (total_len <= sizeof(buf)) {
				(void) sprintf (buf, "%s%s", buf, buffer);
			} else {
				/*  Buffer too big  */
			}
			if (buffer && *buffer) {
				write_log (PingerLog, "PING", Query2String (r->entity->query));
				Channel_send (c->channel, buffer);
			}
		}
		if (total_len > 0) {
			Channel_end_of_record (c->channel);
			debug_message ("Sending to Client [%d] [%s]", c->fid, buf);
/*			write_log (PingerLog, "SENT", "%s", buf); */
		}
	}
	/* -- Now reset the status masks of all returned values */
	FORALL (c, ConnectionList, next) {
		if (c ->fid <= 0)
			continue;
		FORALL (r, c->requests, connection_next) {
			ResetStatusMask (V_STATUS(r->entity));
		}
	}
}

/**********************************************************/
/*  Convert a request into a text string */
/**********************************************************/

extern char *  request2text (request)
	Request request;
{
	static char buffer [1024];
	Connection connection = request -> connection;
	Entity entity = request -> entity;

	(void) sprintf (buffer, "%3d:%-4lu", connection->fid, request->id);
	(void) sprintf (buffer, "%s %s", buffer, Entity_title (entity));
	if (request->critical_after != 0)
		(void) sprintf (buffer, "%s critical after %d", buffer,  request->critical_after);

	return buffer;
}

/**********************************************************/
/*  Given a connection and a request number, return the request */
/**********************************************************/

extern Request id2request (connection, id)
	RequestId id;
	Connection connection;
{
	Request r;
	FORALL(r, connection->requests, connection_next)
		if (r->id == id)
			return r;
	return NULL;
}

/****************************************************************************/
/*  Load requests from a file  */
/****************************************************************************/

extern Boolean load_requests (connection, filename)
	Connection connection;
	char * filename;
{
	FILE * fid;
	char buffer [1024]; /* !!! */

	filename = (char *) MonetFilename (filename);
	fid = fopen (filename, "r");
	if (fid == NULL) {
		user_error ("Cannot open file [%s]", filename);
		return False;
	}
	while (fgets (buffer, sizeof(buffer), fid) != NULL) {
		char * c = tidy_name (buffer);
		if (c && *c)
			(void) parse_message (connection, c);
	}
	fclose (fid);
	return True;
}

/**********************************************************/
/*  Free a request  */
/**********************************************************/

extern void free_request (request)
	Request request;
{
	Entity entity = request->entity;
	Connection connection = request->connection;

	/* remove request from connections list */
	LIST_DEL(Request, request, connection->requests, connection_next);
	/* remove request from entity list */
	LIST_DEL(Request, request, entity->requests, entity_next);
	MonetFree (request);

	/* If entity has no more requests delete it */
	if (entity->requests == NULL) {
		if (entity->query) {
			Query_free (entity->query);
			entity->query = NULL;
		}
		/* free_entity (entity); !!! */
	}
}

/**********************************************************/
/*  Report state of all requests  */
/**********************************************************/

extern void report_all_requests (reporter)
	void (* reporter) ();
{
	Connection c;
	Request r;

	PROC ("report_all_requests");
	FORALL (c, ConnectionList, next) {
		FORALL (r, c->requests, connection_next) {
			reporter (request2text (r));
		}
	}
}

/****************************************************************************/
/*  return the entity associated with a request  */
/****************************************************************************/

extern Entity request2Entity (r)
	Request r;
{
	return r->entity;
}

/****************************************************************************/
/*  Initialise and finish  */
/****************************************************************************/

extern void init_request ()
{
}

extern void finish_request ()
{
}

/**********************************************************/
/**********************************************************/
/*  $Header: //gaia/group.local/computer/monet/Monetd/RCS/request.c,v 1.2 1991/03/22 16:09:27 a38 Exp a38 $  */
/**********************************************************/
/**********************************************************/

