/******************************************************************************/
/******************************************************************************/
/*  $Header$  */
/******************************************************************************/
/* $Log$
*/
/******************************************************************************/
/*  Creates and handles the widgets of the user interface, the buttons, boxes */
/*  menus, the calling of functions etc.  */
/*  !!! A lot of repeated code in here  */
/*  !!! Would like all this to be written as a more general interface, with a */
/*  !!! a command line equivelent  */
/******************************************************************************/

#include "xmonet.h"
#include "map/Translate.h"
#include "map/Interface.h"
#include "db/Info.h"
#include "general/Gather.h"
#include "map/Script.h"

/******************************************************************************/
/*  Globals  */
/******************************************************************************/

#define EXPECT_SELECTED(num) \
	if (numberOf_Selection () != num) { \
		if (num == 1)                \
			user_error ("You must make exactly one selection"); \
		else                                                         \
			user_error ("You must make exactly %d selections", num);\
		return;                                                          \
	}

/*****************************************************************************/
/*  Check add buffer */
/*****************************************************************************/

static char * get_name(widget_name)
	String widget_name;
{
	char * title;

	title = tidy_name (WidgetName2value (widget_name));
	if (title == NULL || *title == NULL) {
		user_error ("You must specify the name of the Entity");
		return NULL;
	}
	return title;
}

static Entity get_type ()
{
	Drawing type;

	if ((type = current_class ()) == NULL) {
		user_error ("You must select a Class");
		return NULL;
	}
	return ENTITY(type);
}

/******************************************************************************/
/******************************************************************************/
/*  Callback routines   */
/******************************************************************************/
/******************************************************************************/

static void quit_mapper ()
{
	user_inform ("Goodbye.");
	Config_finish_handler ();
	exit(0);
}

CALLBACK(QuitCB)
	if (AreAllMapsSaved ()) {
		quit_mapper ();
	} else {
		confirm_box (quit_mapper, "Some maps unsaved, still want to Quit?");
	}
ENDCB

/*****************************************************************************/
/*          Widget Callback routines                                         */
/*****************************************************************************/

CALLBACK(AddSystemCB)
	char *title, buffer[32]; /* !!! */
	Drawing new;
	Entity class;
	static int ID = 1;

	user_inform ("New/Create Drawing");

	if ((class = get_type ()) == NULL) {
		return;
	} else if (has_anonymous_name (class)) {
		/* !!!!!! This is not the way to do it */
		title = (char *) sprintf (buffer, "%d.%d", ID, (int) class);
		ID ++;
	} else 	if ((title = get_name ((String) client_data)) == NULL) {
		return;
	}
	(void) create_new_drawing_at_cursor (class, title);
ENDCB

ACTION(AddSystemACT, 1)
	if (params == NULL) {
		user_error ("No parameters passed to addsystemACT");
	} else {
		AddSystemCB(widget, params[0], NULL);
	}
ENDACT

CALLBACK(AddNoneClassCB)
	char * title;
	Entity new;
	Entity superclass;

	if ((superclass = get_type ()) == NULL)
		return;
	free_selection ();
	if ((title = get_name ((String) client_data)) == NULL)
		return;
	if (new = Entity_create_class (superclass, title)) {
		add_stringAttribute (new, Std_Attribute.instances, "none");
		ClassMap_changed ();
	} else {
		user_error ("Couldn't create new class [%s], title");
	}
ENDCB

CALLBACK(AddSystemClassCB)
	char * title;
	Entity new;
	Entity superclass;

	if ((superclass = get_type ()) == NULL)
		return;
	free_selection ();
	if ((title = get_name ((String) client_data)) == NULL)
		return;
	if (new = Entity_create_class (superclass, title)) {
		add_stringAttribute (new, Std_Attribute.instances, "systems");
		add_stringAttribute (new, Std_Attribute.shape, "simple");
		add_stringAttribute (new, Std_Attribute.shape, "resizable");
		ClassMap_changed ();
	} else {
		user_error ("Couldn't create new class [%s], title");
	}
ENDCB

CALLBACK(AddMapClassCB)
	char * title;
	Entity new;
	Entity superclass;

	if ((superclass = get_type ()) == NULL)
		return;
	free_selection ();
	if ((title = get_name ((String) client_data)) == NULL)
		return;
	if (new = Entity_create_class (superclass, title)) {
		add_stringAttribute (new, Std_Attribute.instances, "maps");
		add_stringAttribute (new, Std_Attribute.shape, "map");
		ClassMap_changed ();
	} else {
		user_error ("Couldn't create new class [%s], title");
	}
ENDCB

CALLBACK(AddInterfaceClassCB)
	char * title;
	Entity new;
	Entity superclass;

	if ((superclass = get_type ()) == NULL)
		return;
	free_selection ();
	if ((title = get_name ((String) client_data)) == NULL)
		return;
	if (new = Entity_create_class (superclass, title)) {
		add_stringAttribute (new, Std_Attribute.instances, "interfaces");
		add_stringAttribute (new, Std_Attribute.shape, "connector");
		ClassMap_changed ();
	} else {
		user_error ("Couldn't create new class [%s], title");
	}
ENDCB

/*****************************************************************************/
/*    Monitor Box                                                               */
/*****************************************************************************/

/*
CALLBACK(PingSystemCB)
	Entity sys;

	if (numberOf_Selection () != 1) {
		user_error ("You must select exactly one system");
		return;
	}
	if (sys == exactly_one_entity ()) {
	user_inform ("This is a limited utility that currently only does IP pings");
	sys = current_Selection ();

	if (ip_ping (Entity_title (sys))) {
		user_inform ("Reachable by IP");
		set_reachability (sys, status_reachable);
	} else {
		user_inform ("Currently not IP reachable");
		set_reachability (sys, status_unreachable);
	}
	free_selection ();

ENDCB
*/

/*****************************************************************************/
/*** If a '&' is found in the request then substitute the systems title/address ***/
/*****************************************************************************/
 
static char ** prepare_shell_command (client_data)
	caddr_t client_data;
{
	Drawing d;
	String command;
	static char *argv[4];

	if (numberOf_Selection () > 1) {
		user_error ("Can't select more than one system");
		return NULL;
	}
	if (client_data == NULL) {
		user_error ("No shell command given to MonetShellCommandCB");
		return NULL;
	}
	if (d = current_Selection ()) {
		Entity sys;
		sys = ENTITY(d);
		/*  Replace any '&' with the title of the entity  */
		command = MonetReplaceInString ((char *) client_data, "&", TITLE(sys));
	} else {
		char * c;
		command = client_data;
		for (c = command; *c && *c != '&'; ++c);
			if (*c == '&') {
				user_error ("Must select exactly one system");
				return NULL;
			}
	}

	{
	    /*
	     * Build a display string using the current screen number, so that
	     * X programs which get fired up from a menu come up on the screen
	     * that they were invoked from, unless specifically overridden on
	     * their command line.
	     */
	    static char buf[256];
	    char *ds = DisplayString (display);
	    char *colon, *dot1;

	    colon = rindex (ds, ':');
	    if (colon) {			/* if host[:]:dpy */
		(void) strcpy (buf, "DISPLAY=");
		(void) strcat (buf, ds);
		colon = buf + 8 + (colon - ds);	/* use version in buf */
		dot1 = index (colon, '.');	/* first period after colon */
		if (!dot1) dot1 = colon + MonetStrlen (colon);  /* if not there, append */
		(void) sprintf (dot1, ".0");
		putenv (buf);
	    }
	}
	argv[0] = "/bin/sh";
	argv[1] = "-c";
	argv[2] = command;
	argv[3] = NULL;

	return argv;
}


/*****************************************************************************/
/*****************************************************************************/

static String process_command (text)
	String text;
{
	static char buffer[1024] = {0}; /* !!! */
	String buf, c;

	for (c=text, buf = buffer; *c; ++c) {
		String widget_name, insert;
		if (*c == '+' && isgraph (*(c+1))) {
			++c;
			widget_name = buf;
			while (isgraph (*c)) {
				*buf = *c;
				++c;
				++buf;
			}
			*buf = '\0';
			insert = tidy_name (WidgetName2value (widget_name));
			if (insert && *insert)
				buf = MonetStringCopyToEnd (widget_name, insert);
			else
				buf = widget_name;
		} else {
			*buf = *c;
			++buf;
		}
	}
	*buf = '\0';
	if (MonetStrlen (buffer) == 0) {
		user_error ("No command given");
		return NULL;
	}
	return buffer;
}

/***********************************************************************/
/*  Execute a shell command as a child process  */
/***********************************************************************/

CALLBACK(MonetShellCommandCB)
	String command;
	String result, errors;
	extern char *DoCommandToString();
	char ** argv;

	command = process_command ((String) client_data);

	if ( ! command)
		return;

	if (! (argv = prepare_shell_command (command)))
		return;

	start_report ("Executing [%s]\nClick button to stop", argv[2]);
	Map_exec_cursor (NULL);
	result = DoCommandToString(argv, &errors);
	Map_normal_cursor (NULL);
	if (MonetStrlen(errors) > 1)
		writeln_report ("Errors: %s", errors);
	if (MonetStrlen(result) > 1)
		writeln_report ("%s", result);

	if (result)
		XtFree (result);
	if (errors)
		XtFree (errors);
	writeln_report ("Shell command done");
	end_report ();
ENDCB

CALLBACK(MonetScriptCommandCB)
	String command;
	command = process_command ((String) client_data);
	if ( ! command)
		return;
	(void) Script_execute (command);
ENDCB

CALLBACK(MonetShellInterpreterCB)
	String command;
	String result, errors;
	extern char *DoCommandToString();
	char ** argv;

	command = process_command ((String) client_data);
	if ( ! command)
		return;

	if (! (argv = prepare_shell_command (command)))
		return;

	start_report ("Executing [%s]\nClick button to stop", argv[2]);

	Map_exec_cursor (NULL);
	result = DoCommandToString(argv, &errors);
	Map_normal_cursor (NULL);
	if (MonetStrlen(errors) > 1)
		writeln_report ("Errors: %s", errors);
	if (MonetStrlen(result) > 1)
		writeln_report ("%s", result);

	if (result && *result)
		Script_execute (result);
	if (result)
		XtFree (result);
	if (errors)
		XtFree (errors);
	writeln_report ("Shell command done");
	end_report ();
ENDCB

ACTION(MonetScriptCommandACT, 1)
	MonetScriptCommandCB(widget, MonetJoinStrings (*num_params, params, " "), 0);
ENDACT

ACTION(MonetShellCommandACT, 1)
	MonetShellCommandCB(widget, MonetJoinStrings (*num_params, params, " "), 0);
ENDACT

CALLBACK(MonetScriptSetWidgetValueCB)
	/*  Argument is <widget name>, value  */
	/*  Fills in a text or dialog widgets value with the  */
	/*  result of a monet script function.  */

	String title, value;

	if (client_data == NULL) {
		user_error ("MonetScriptSetWidgetValueCB needs 2 args");
		return;
	}
	title = MonetNewString ((String) client_data);
	for (value = title; *value && *value != ';'; ++value);	
	if (*value) {
		*value = '\0';
		++ value;
		TextWidget_set_value (title, Script_execute (value));
	} else {
		user_error ("MonetScriptSetWidgetValueCB needs 2 args [%s]", title);
	}
	MonetFree (title);
ENDCB

ACTION(MonetScriptSetWidgetValueACT, 1)
	MonetScriptSetWidgetValueCB(widget, params[0], 0);
ENDACT

SCRIPTCALLBACK(do_x_shell)
	MonetShellCommandCB (NULL, MonetJoinStringsUntilNull (arg, NULL), 0);
ENDSC

/*****************************************************************************/
/*  Init and finish callback module  */
/*****************************************************************************/

extern void init_frontend ()
{
#define NUM_ACTIONS 4
	static XtActionsRec actions[NUM_ACTIONS] = {
	{"MonetScriptSetWidgetValueACT", MonetScriptSetWidgetValueACT},
	{"MonetScriptCommandACT", MonetScriptCommandACT},
	{"MonetShellCommandACT", MonetShellCommandACT},
	{"AddSystemACT", AddSystemACT}
	};
	(void) XtAppAddActions(app_con, (XtActionList) actions, (Cardinal) NUM_ACTIONS);
	
	/* !!! This lot should be replaced by call to MonetScriptCommand  */
	RCALL("AddSystemCB",		AddSystemCB,		NULL);
	RCALL("AddNoneClassCB",		AddNoneClassCB,		NULL);
	RCALL("AddSystemClassCB",	AddSystemClassCB,	NULL);
	RCALL("AddMapClassCB",		AddMapClassCB,		NULL);
	RCALL("AddInterfaceClassCB",	AddInterfaceClassCB,	NULL);
	RCALL("QuitCB",			QuitCB,			NULL);

	/* -- Command interpreters  */
	RCALL("MonetShellCommandCB",	MonetShellCommandCB,	NULL);
	RCALL("MonetScriptCommandCB",	MonetScriptCommandCB,	NULL);
	RCALL("MonetShellInterpreterCB",MonetShellInterpreterCB,NULL);
	RCALL("MonetScriptSetWidgetValueCB",MonetScriptSetWidgetValueCB,NULL);

	ScriptMacro_create ("xsh",	NULL, do_x_shell,
		"do bourne shell command 'sh' and send to X window",
		"");
}

extern void finish_frontend ()
{
}

/*****************************************************************************/
/*****************************************************************************/
/*  $Header$  */
/*****************************************************************************/
/*****************************************************************************/


