#ifndef MONET_DB_STATUS_H_
#define MONET_DB_STATUS_H_

typedef struct status * VStatus;

#include "db/Entity.h"

/* This is is supposed to define the status of a network object */
/* it is planned to be used by  both monetd and monet to help   */
/* keep things consistent. */

/* This structure contains all the actual core status data for an */
/* object as opposed to the presentation status held in 'drawing' */

/* !!! The inclusion of the monetd stuff is very recent, and is an  */
/* !!! attempt to merge these two things.  */
/* Should contain other stuff from the vobject structure */


#define MonitorEndCode '\n'
#define MonitorSepCode '|'




typedef unsigned long Capacity; 
#define UNKNOWN_CAPACITY ((Capacity) 0)

typedef unsigned long int StatusMask;

typedef char * Contact;

#define StatusTitleMask		(1<<1)
#define StatusReachMask		(1<<2)
#define StatusBehaviourMask	(1<<3)
#define StatusCapacityMask	(1<<4)
#define StatusFutureMask	(1<<5)
#define StatusConnectionsMask	(1<<6)

#define StatusLoadInMask	(1<<7)
#define StatusLoadOutMask	(1<<8)
#define StatusOctInMask		(1<<9)
#define StatusOctOutMask	(1<<10)
#define StatusPktInMask		(1<<11)
#define StatusPktOutMask	(1<<12)

#define StatusNeighbourMask	(1<<13)
#define StatusContactMask	(1<<14)
#define StatusPingMask 		(1<<15)
#define StatusActionMask	(1<<16)
#define StatusInstancesMask	(1<<17)
#define StatusInterfaceMask	(1<<18)

#define StatusUnderlyingMask	(1<<19)
#define StatusUnbendingMask	(1<<20)
#define StatusAnonMask		(1<<21)
#define StatusFormatMask	(1<<22)
#define StatusShapeMask		(1<<23)
#define StatusResizeMask	(1<<24)
#define StatusTranslationMask	(1<<25)
#define StatusComponentMask	(1<<26)


#define StatusNoneMask 		0
#define StatusAllMask		((StatusMask) 0xffffffff)

#define SetStatusMask(vstatus,var,val,vmask, DO) \
	if (vstatus->var != val) { \
		{ DO; }; \
		vstatus->var = val; \
		vstatus->mask |= vmask; \
	}

#define SetTitleStatusMask(vstatus,val)             \
	if ( ! EQS(vstatus->title, val)) {          \
		if (vstatus->title != NULL)         \
			MonetFree (vstatus->title); \
		vstatus->title = MonetNewString(val);     \
		vstatus->mask |= StatusTitleMask;   \
	} 
#define SetContactStatusMask(vstatus,val)             \
	if ( ! EQS(vstatus->contact, val)) {          \
		if (vstatus->contact != NULL)         \
			MonetFree (vstatus->contact); \
		vstatus->contact = MonetNewString(val);     \
		vstatus->mask |= StatusContactMask;   \
	} 
#define SetReachStatusMask(vstatus,val)  SetStatusMask(vstatus, status, val, StatusReachMask, 0)

#define SetFutureStatusMask(vstatus,val) \
		SetStatusMask(vstatus, future, val, StatusFutureMask, \
		if (vstatus->future == future_planned) \
			vstatus->status = status_unknown; \
		)

#define SetCapacityStatusMask(vstatus,val) SetStatusMask(vstatus, capacity, val, StatusCapacityMask, 0 )
#define SetBehaviourStatusMask(vstatus,val) SetStatusMask(vstatus, behaviour, val, StatusBehaviourMask, 0)
#define SetConnectionsStatusMask(vstatus,val) SetStatusMask(vstatus, num_connections, val, StatusConnectionsMask, 0)
#define SetLoadInStatusMask(vstatus,val) SetStatusMask(vstatus, load_in, val, StatusLoadInMask, 0)
#define SetLoadOutStatusMask(vstatus,val) SetStatusMask(vstatus, load_out, val, StatusLoadOutMask, 0)
#define ResetStatusMask(vstatus) {vstatus->mask = StatusNoneMask; }

typedef enum {
	behaviour_static = 'S',
	behaviour_normal = 'N',
	behaviour_odd = 'O',
	behaviour_chaotic = 'C',
	behaviour_unknown = '?'
	} Behaviour;

typedef enum {
	status_critical = 'C',
	status_reachable = 'R',
	status_unreachable = 'U',
	status_unknown = '?',
	status_invalid = '-'
	}
	ReachStatus ;

typedef enum {
	future_certain = 'c',
	future_temporary = 't',
	future_planned = 'p',
	future_unknown = 'u'
#define FUTURE_DEFAULT future_certain
	}
	FutureStatus;

#ifdef MONET_PROTOTYPES
typedef void (* StatusTrigger) (VStatus, VStatus, caddr_t);
#else
typedef void (* StatusTrigger) ();
#endif

typedef struct status {
	caddr_t		change_data;  /*  For trigger routine */
	Cardinal	ping_id;
	char           *title;		/* Should maybe generalise this */
	char           *alias;		/* Should maybe generalise this */
	unsigned long	time_stamp;	/* When was this changed */
	ReachStatus	status;		/* !!! Should merge ReachStatus and Reachable indicator */
	Behaviour	behaviour;
	FutureStatus	future;
	StatusMask	mask;		/* Changes since last change */	
	Load		load_in, load_out;  /* Percentage */
	Contact		contact;
	/*  The number of connections to this system */
	Cardinal	num_connections;
#define NUM_CONNECTIONS_UNKNOWN ((Cardinal) 0xffff)
#define MAX_CONNECTIONS	((Cardinal) 0xffe)
	/* Saveable state */
	char *		ping;		/* Ping instrution */
	Capacity	capacity;	/* Should specify units */
	Capacity	octets_in, octets_out, pkts_in, pkts_out;
	String		action;
} EntityStatus;

/**************************************************************************/
/*  Communications stuff */
/**************************************************************************/
#ifdef ksdjksjk
#define RequestLength 128

/* reachable/unreachable etc */
#define request_status      1
/* Load */
/* Speed, etc */
#define request_capacity    8
/* The official title */
#define request_title       16
/* Is there something odd about this mob, or is it normal */
#define request_behaviour   32

#define request_default request_status
typedef unsigned long RequestedInfo;

#endif
/**************************************************************************/
/*  Functions  */
/**************************************************************************/

#ifdef MONET_PROTOTYPES

extern void		init_status			(StatusTrigger);
extern void		finish_status			(void);
extern VStatus		create_status			(caddr_t);
extern void		free_status			(VStatus);

extern char *		pretty_status_report_heading	(void);
extern char *		simple_status_report		(VStatus);
extern char *		pretty_simple_status_report	(VStatus);

extern String		capacity2mega_string		(Capacity);
extern String		capacity2string			(Capacity);
extern String		cardinal2string			(Cardinal);
extern String		status2compactascii		(VStatus, StatusMask);
extern String		future2string			(FutureStatus);
extern String		behaviour2string		(Behaviour);
extern String		load2string			(Load);
extern String		reachstatus2string		(ReachStatus);
extern FutureStatus	string2future			(String);
extern String		format_text_status		();
extern Boolean		parse_text_status		();
extern Boolean		set_capacity			(VStatus, Capacity);
extern Boolean		set_future			(VStatus, FutureStatus);
extern Boolean		set_reachability		(VStatus, ReachStatus);
extern Boolean		set_load_in			(VStatus, Load);
extern Boolean		set_load_out			(VStatus, Load);
extern Boolean		set_behaviour			(VStatus, Behaviour);
extern Boolean		set_contact			(VStatus, String);
extern Boolean		set_neighbour			(VStatus);
extern Boolean		set_ping			(VStatus, String);


#else

extern void		init_status			();
extern void		finish_status			();
extern String		capacity2mega_string		();
extern String		capacity2string			();
extern String		cardinal2string			();
extern char *		pretty_status_report_heading	();
extern VStatus		create_status			();
extern char *		simple_status_report		();
extern char *		pretty_simple_status_report	();
extern char *		status2compactascii		();
extern void		free_status			();
extern String		future2string			();
extern String		load2string			();
extern String		behaviour2string		();
extern String		reachstatus2string		();
extern FutureStatus	string2future			();
extern Boolean		message2status			();
extern String		format_text_status		();
extern Boolean		parse_text_status		();
extern Boolean		set_capacity			();
extern Boolean		set_future			();
extern Boolean		set_reachability		();
extern Boolean		set_load_in			();
extern Boolean		set_load_out			();
extern Boolean		set_behaviour			();
extern Boolean		set_contact			();
extern Boolean		set_neighbour			();
extern Boolean		set_neighbour			();
extern Boolean		set_ping			();


#endif /* MONET_PROTOTYPES */

#endif /* _STATUS_H_ */

