/****************************************************************/
/****************************************************************/
/*  $Header$   */
/****************************************************************/
/*  The model used for the network consists of a set of nodes   */
/*  or nodes connected to other nodes via interfaces, an    */
/*  interface can have up to two connections.  Both nodes and */
/*  interfaces are entities.                                    */
/*  Entity classes can be networked in exactly the same way,    */
/*  this is useful because it defines which entities can be     */
/*  connected via an interface.  For instance we could prevent  */
/*  two ip hosts being connected together directly, by only     */
/*  defining an interface between an IP network and an IP host. */

/*  The make_interface and can_interface are a total mess. !!!  */
/****************************************************************/

#include "db/Entity.h"

/****************************************************************/
/*  Definitions  */
/****************************************************************/

/***************************************************************/
/*  Query for interfaces state  */
/***************************************************************/

extern Entity from_interface (interface)
	Entity interface;
{
	if (isInterface (interface) == False) {
		sys_error ("Not an interface");
		return (NULL);
	}
	return FROM (interface);
}

extern Entity to_interface (interface)
	Entity interface;
{
	if (isInterface (interface) == False) {
		sys_error ("Not an interface");
		return (NULL);
	}
	return TO (interface);
}

/***************************************************************/
/*  Various Tests  */
/***************************************************************/

extern Boolean isInterface (s)
	Entity s;
{
	if (Entity_typeOf(s) == o_interface && !Entity_isClass(s))
		return True;
	return False;
}

/***************************************************************/
/*  Can two objects be interfaced together ?  */
/***************************************************************/

extern Boolean can_interface (interface, node)
	Entity interface, node;
{
	Boolean fromClass, toClass, fromFree, toFree;
	Entity node_class;

	if (Entity_isClass (node))
		node_class = node;
	else
		node_class = Entity_classOf (node);
	if (isInterfaceClass (interface)) {
		fromClass = Entity_subclass (get_fromClass(interface), node_class);
		toClass   = Entity_subclass (get_toClass(interface),   node_class);
		if (fromClass && toClass)
			return True;
		user_error ("Interfaces of class [%s] cannot connect these two nodes", Entity_title (interface));
		return False;
	} else if (isInterface (interface)) {
		fromFree  = FROM(interface) == NULL;
		toFree    = TO(interface) == NULL;
	} else {
		sys_error ("[%s] is not an interface", Entity_title (interface));
		return False;
	}
	fromClass = Entity_subclass (get_fromClass(interface), node_class);
	toClass   = Entity_subclass (get_toClass(interface),   node_class);

	if (fromFree && fromClass) {
		if (!Entity_isClass (node) && TO(interface) == node) {
			user_error ("Cannot interface [%s] to itself", Entity_title (node));
			return False;
		}
	} else 	if (toFree && toClass) {
		if (!Entity_isClass (node) && FROM(interface) == node) {
			user_error ("Cannot interface [%s] to itself", Entity_title (node));
			return False;
		}
	} else if (!toFree && !fromFree) {
		user_error ("[%s] is already connected to two nodes", Entity_title (interface));
		return False;
	} else {
		user_error ("Cannot connect interface [%s] because of class conflict", Entity_title (interface));
		return False;
	}
	return True;
}

static Boolean interfaced (interface, node)
	Entity interface, node;
{
	Entity other;

	/* First check to see if already directly connected */
	if (FROM(interface) == node || TO(interface) == node)
		return True;
	if (FROM(interface) == NULL)
		other = TO(interface);
	else if (TO(interface) == NULL)
		other = FROM(interface);
	else
		return False;


	if (other == NULL)
		return False;


	/* !!! Temporary, disallow multiple joins */
	return False;
	/* NOTREACHED */

	/* !!! We allow nodes to be interfaced more than once !!! */
	/* Check the 'other' side of the interface is not interfaced to node via another interface */
/*	FOR_ENTITYLIST (i, other->interfaces)
		if (FROM(i) == node || TO(i) == node)
			return True;
	return False;
*/
}

extern Boolean are_interfaced (sys1, sys2)
	Entity sys1, sys2;
{
	if (isInterface (sys1))
		return interfaced (sys1, sys2);
	else if (isInterface (sys2))
		return interfaced (sys2, sys1);

/* !!!  This would check for things being interfaced more than once 
	FOR_ENTITYLIST (interface, sys1->interfaces)
		if (interfaced (interface, sys2))
			return True;
*/
	return False;
}

/***************************************************************/
/*  Connect and disconnect Systems and Interfaces */
/***************************************************************/

extern void make_interface (interface, node)
	Entity interface, node;
{
	Boolean fromClass, toClass, fromFree, toFree;
	String title_interface, title_node; /*!!!*/
	Boolean already_in = False;

	title_interface = Entity_title(interface);
	if (isInterface (interface) == False && isInterfaceClass (interface) == False) {
		sys_error ("[%s] is not an interface", title_interface);
		return;
	}
	title_node      = Entity_title(node);

	if (Entity_isClass (interface))
		if (Entity_isClass (node)) {
			fromClass = True;
			toClass   = True;
		} else {
			user_error ("Cannot interface a class [%s] to an instance [%s]",
				title_interface, title_node);
			return;
		}
	else if (Entity_isClass (node)) {
			user_error ("Cannot interface a class [%s] to an instance [%s]",
				title_node, title_interface);
			return;
	} else {
		fromClass = Entity_subclass (get_fromClass(interface), Entity_classOf(node));
		toClass   = Entity_subclass (get_toClass(interface),   Entity_classOf(node));
	}
	fromFree  = FROM(interface) == NULL;
	toFree    = TO(interface) == NULL;

	if (fromFree && fromClass) {
		if (TO(interface) == node) {
			if (Entity_isClass (node)) {
				already_in = True;
			} else {
				sys_error ("Cannot interface [%s] to itself", title_node);
				return;
			}
		}

		FROM(interface) = node;
	} else 	if (toFree && toClass) {
		if (FROM(interface) == node) {
			if (Entity_isClass (node)) {
				already_in = True;
			} else {
				sys_error ("Cannot interface [%s] to itself", title_node);
				return;
			}
		}
		TO(interface) = node;
	} else if (!toFree && !fromFree) {
		user_error ("[%s] already has two interfaces", title_interface);
		return;
	} else {
		user_error ("Can't interface [%s] to objects of class [%s]",
			title_interface, Entity_title (Entity_classOf(node)));
		return;
	}
	if (! already_in)
		insert_entityList (node->interfaces, interface);
	set_neighbour (V_STATUS(interface));
}


extern void unmake_interface (interface, node)
	Entity interface, node;
{
	if (node==NULL)
		  return;
	if (FROM(interface) == node) {
		FROM(interface) = NULL;
	} else 	if (TO(interface) == node) {
		TO(interface) = NULL;
	} else {
		sys_error ("Not interfaced");
		return;
	}
	extract_entityList (node->interfaces, interface);
	set_neighbour (V_STATUS(interface));
}

/***************************************************************/
/***  Disconnects all the interfaces of an object ****/
/*  !!! Should be called something else */

extern void free_interfaces (node)
	Entity node;
{
	Entity interface;

	if (Entity_isClass (node)) {
		return;
	} else if (isInterface (node)) {
		unmake_interface (node, FROM(node));
		unmake_interface (node, TO(node));
		return;
	}
	FOR_ENTITYLIST (interface, Entity_interfaces(node)) {
		if (FROM(interface) == node) {
			FROM(interface) = NULL;
		} else 	if (TO(interface) == node) {
      			TO(interface) = NULL;
		}
		set_neighbour (V_STATUS(interface));
	}
	reset_entityList (node->interfaces);
}

/****************************************************************/
/****************************************************************/
/*  $Header$   */
/****************************************************************/
/****************************************************************/


