/***************************************************************/
/***************************************************************/
/* $Header$ */
/***************************************************************/
/* $Log$ */
/***************************************************************/

#include "db/Entity.h"
#include "parser.h"
#include "general/Log.h"
#include "sys/param.h"  /* For MAXPATHLEN */


static String my_fgets(buffer, len, f)
	FILE *f;
	String buffer;
	int len;
{
	int found = 0, ic;
	while ((ic = getc(f)) != EOF) {
		char c = (char) ic;
		if (c == '\n') {
			if (found) {
				if (buffer[found - 1] == CONTINUATION_CHAR)
					buffer[found] = ' ';
				else {
					buffer[found] = NULL;
					return buffer;
				}
			} else {
				continue;
				/* Ignore empty lines */
			}
		} else if (c == ' ' && !isprint (c)) {
			if (found)
				buffer[found] = ' ';
			else
				continue;
		} else {
			buffer[found] = c;
		}
		found ++;
	}
	return NULL;
}

/***************************************************************/
/* Process one line of input */
/***************************************************************/

extern Boolean Entity_load_line (entity_ref, line)
	Entity * entity_ref;
	String line;
{
	Entity entity, class;
	Boolean isclass;
	AttributeID id;
	int cc;
	char sub [1024], super [1024], *title, attribute[1024]; /* !!! */
	char help[1024]; /* !!! */

	entity = *entity_ref;
	if (line == NULL || *line == NULL || *line == '\n' || *line == COMMENT_CHAR) {
	} else if (sscanf(line, "Attribute %s %s", line, help) == 1) {
		if (create_attributeID (line, 0, ' ', False, type_string, a_all, NULL, True, help) == NULL)
			parse_error ("Attribute [%s] has already been declared", line);
	} else if (sscanf(line, "Object %s", line) == 1) {
		entity = Entity_find_or_create (line);
		if ( ! entity)
			return False;
	} else if ((cc = sscanf (line, "%[a-zA-Z_]=%[^\n]", attribute, line)) > 0) {
		String text;
		if (! entity)
			return False;
		if (cc == 1)
			text = NULL;
		else
			text = tidy_name (line);
		if ((id = String2AttributeID (attribute)) && entity)
			add_stringAttribute(entity, id, text);
		else {
         		user_error ("No such attribute [%s]", attribute);
			return False;
		}		
	} else {
		user_error ("Don't understand [%s]", line);
		return False;
	}
	if (entity)
		*entity_ref = entity;
	return True;
}

/***************************************************************/
/*  Load the entity database                                   */
/***************************************************************/

extern Boolean ClassEntity_load_db (filename)
	String filename;
{
	FILE *f;
	char buffer_space[1024], *buffer = buffer_space;
	Entity entity = NULL;

	filename = MonetFilename (filename);
	if ((f = fopen (filename, "r")) == NULL) {
		user_error ("Sorry, can't open file [%s]", filename);
		return False;
	}
	user_inform ("Loading entity database from file [%s]", filename);
	while (my_fgets(buffer, sizeof(buffer_space), f)) {
		if ( ! Entity_load_line (&entity, buffer)) {

		}
		buffer = buffer_space;
	}
	fclose (f);
	return True;
}

/***************************************************************/
/*  Save a list of entitys                                     */
/*  Need to save connections after systems, not really tru now  */
/*  Does not save classes, this is hack for just now !!! */
/*  Only writes things if they have attributes set.  */
/***************************************************************/

static void save_entities (f, entitys)
	FILE * f;
	EntityList entitys;
{
	Entity s;

	FOR_ENTITYLIST (s, entitys)
		if (!s->standard && !Entity_isClass (s)) {
			String report;
			if (report = Entity2Report (s, True))
				(void) fputs (report, f);
		}
}

extern Boolean ClassEntity_save (f)
	FILE * f;
{
	save_entities (f, Entities);
}

/***************************************************************/
/*  Save the whole database  */
/*  Must create tempfiles in current directory  */
/*  because you can't link across devices.  */
/*  Save to temp file first then link to saved file to avoid */
/*  problems with crashes.  */
/***************************************************************/

extern Boolean ClassEntity_save_db (filename)
	String filename;
{
	FILE *f;
	int fd;
	String tempname1 = "./tmpedbXXXXXX";
	char  tempname2 [MAXPATHLEN];

	filename = MonetFilename (filename);
	if ( ! (fd = mkstemp (tempname1))) {
		user_error ("Sorry, can't open temp file [%s]", tempname1);
		return False;
	}
	f = fdopen (fd, "w");

	(void) sprintf (tempname2, "%s_XXXXXX", filename);
	(void) mktemp (tempname2);
	(void) fprintf (f, "%c Saved by [%s] on %s\n", COMMENT_CHAR, Config_program_name, pretty_time_stamp ());
	ClassAttribute_save (f);
	save_entities (f, Entities);

	fclose (f);

	if (link (filename, tempname2) != 0) {
		user_error ("Could not link [%s] to [%s], errno %d", filename, tempname2, errno);
	} else if (unlink (filename) != 0) {
		user_error ("Could not unlink [%s], errno %d", filename, errno);
		(void) unlink (tempname2);
	} else if (link (tempname1, filename) != 0) {
		user_error ("Could not link [%s] to [%s], errno %d", tempname1, filename, errno);
		if (link (tempname2, filename) != 0) {
			user_error ("Lost original, could not link [%s] to [%s], errno %d", tempname2, filename, errno);
       		} else {
	       		user_inform ("Restored original");
                 }
	} else {
		if (unlink (tempname1) != 0) {
			user_warning ("Could not unlink [%s], errno %d", tempname1, errno);
       		}
		user_inform ("Saved entity database to file [%s]", filename);
		user_inform ("Backed up in [%s]", tempname2);
	}
}
/***************************************************************/
/*  Init and finish  */
/***************************************************************/

extern void init_parser ()
{
}

extern void finish_parser ()
{
}

/***************************************************************/
/***************************************************************/
/* $Header$ */
/***************************************************************/
/***************************************************************/

