/***************************************************************/
/***************************************************************/
/*  $Header: //gaia/group.local/computer/monet/lib/db/RCS/status.c,v 1.1 1991/06/04 07:10:28 a38 Exp a38 $  */
/***************************************************************/
/***************************************************************/

#include "general/General.h"
#include "db/Entity.h"

/***************************************************************/
/*  Matching names to enum values */
/*  Last in list must have the default enum value and a NULL name */
/***************************************************************/

static struct { String name; FutureStatus val; }
	FutureTable[] = {
	{ "certain", future_certain },
	{ "temp", future_temporary },
	{ "planned", future_planned },
	{ "unknown", future_unknown },
	{ NULL, FUTURE_DEFAULT}
};

static struct { String name; ReachStatus val; }
	StatusTable[] = {
	{ "critical",  status_critical },
	{ "reachable", status_reachable },
	{ "unreachable",  status_unreachable},
	{ "unknown",  status_unknown },
	{ "invalid",  status_invalid },
	{ NULL, status_unknown }
};

static struct { String name; Behaviour val; }
	BehaviourTable[] = {
	{ "static",   behaviour_static },
	{ "normal",   behaviour_normal },
	{ "odd",      behaviour_odd },
	{ "chaotic",  behaviour_chaotic },
	{ "unknown",  behaviour_unknown },
	{ NULL, behaviour_normal }
};

#define TOSTRING(s,table) { int i; for (i=0;table[i].name != NULL; i++) \
				if (table[i].val == s) return table[i].name; \
				return NULL; }
#define TOVAL(s,table)    { int i; for (i=0;table[i].name != NULL; i++) \
				if (EQS(table[i].name, s)) return table[i].val; \
				return table[i].val; }

StatusTrigger ChangeTrigger = NULL;

/***************************************************************/
/*  Init and finish  */
/***************************************************************/

extern void init_status (changed)
	StatusTrigger changed;
{
	ChangeTrigger = changed;
}

extern void finish_status ()
{
	ChangeTrigger = NULL;
}

/***************************************************************/
/*  Create and Free */
/***************************************************************/

extern VStatus create_status (trigger_data)
	caddr_t trigger_data;
{
	VStatus new;

	new = MonetNew (struct status);
	new->change_data = trigger_data;
	new->ping_id = 0;
	new->alias = NULL;
	new->title = NULL;
	new->status = status_unknown;
	new->future = FUTURE_DEFAULT;
	new->capacity = UNKNOWN_CAPACITY;
	new->ping = NULL;
	new->contact = NULL;
	new->behaviour = behaviour_normal;
	new->load_in  = UNKNOWN_LOAD;
	new->load_out = UNKNOWN_LOAD;
	new->num_connections    = NUM_CONNECTIONS_UNKNOWN;
	new->time_stamp = time(NULL);
	new->action = NULL;
	return new;
}

extern void free_status (s)
	VStatus s;
{
	MonetFree (s);
}

/***************************************************************/
/*  Simple Report */
/***************************************************************/

#define LOAD(cap,in,period) ((cap && period) ? ((8*100*in)/(cap*period)) : 0)

extern char * simple_status_report (o)
	VStatus o;
{
	return pretty_simple_status_report (o);
}

extern char * simple_status_report_heading ()
{
	return pretty_status_report_heading ();
}

/***************************************************************/

/***************************************************************/

extern char * pretty_status_report_heading ()
{
	static char Heading [512]; /* !!! */

	(void) sprintf (Heading,
			"%12s | %9s | %8s | %8s | %8s | %9s | %12s",
			"Reachability",
			"Behaviour",
			"Future",
			"Capacity",
			"Load In%",
			"Load Out%",
			"Connections"
			);
	return Heading;
}

/***************************************************************/

/***************************************************************/

extern char * pretty_simple_status_report (o)
	VStatus o;
{
	static char report[1024]; /* !!! */
	char buffer[1024]; /* !!! */
	char * num_connections;

	if (o->num_connections == NUM_CONNECTIONS_UNKNOWN)
		num_connections = " ------ ";
	else {
		(void) sprintf (buffer, "%-8u", o->num_connections);
		num_connections = buffer;
	}

	(void) sprintf (report,
			"%12s | %9s | %8s | %8s | %8s | %9s | %12s",
			reachstatus2string (o->status),
			behaviour2string (o->behaviour),
			future2string (o->future),
			capacity2mega_string (o->capacity),
			load2string (o->load_in),
			load2string (o->load_out),
			num_connections
		);

	return report;
}

/***************************************************************/
/*  Exported functions  */
/***************************************************************/

extern String load2string (l)
	Load l;
{
	static char buffer [32]; /* !!! */
	if (l == UNKNOWN_LOAD)
		return "---";
	(void) sprintf (buffer, "%u%%", ((int) l * 100) / FULL_LOAD);
	return buffer;
}

extern Load string2load (s)
	String s;
{
	Load l;
	int i, cc;
	
	cc = sscanf (s, "%u%%", &i);
	if (cc == 1 && i <= 100)
		l = i * FULL_LOAD / 100;
	else
		l = UNKNOWN_LOAD;
	return l;
}

/***************************************************************/

/***************************************************************/

extern String future2string (f)
	FutureStatus f;
{
	TOSTRING (f, FutureTable);
}

extern String behaviour2string (b)
	Behaviour b;
{
	TOSTRING (b, BehaviourTable);
}

extern String reachstatus2string (r)
	ReachStatus r;
{                                                           
	TOSTRING (r, StatusTable);
}

extern ReachStatus string2reachstatus (s)
	String s;
{                                                           
	TOVAL (s, StatusTable);
}

extern Behaviour string2behaviour (s)
	String s;
{
	TOVAL (s, BehaviourTable);
}

extern FutureStatus string2future (s)
	String s;
{
	TOVAL (s, FutureTable);
}

/*****************************************************************************/
/*  Utility to print capacity as K, M or G instaed of lots of Zeros  */
/*****************************************************************************/

static void capacity2text (capacity, buffer, exp, letter)
	char * buffer;
	Capacity exp, capacity;
	char letter;
{
	Capacity rem, value;

	rem   = capacity % exp;
	value = capacity / exp;
	if (rem == 0) {
		(void) sprintf (buffer, "%lu%c", value, letter);
	} else {
		(void) sprintf (buffer, "%.1f%c", (float) capacity / (float) exp, letter);
	}
}

extern String capacity2mega_string (c)
	Capacity c;
{
	static char buffer[32];

	if (c == UNKNOWN_CAPACITY)
		return "";

	if (c >= 1000000000)
		capacity2text (c, buffer, (Capacity) 1000000000, 'G');
	else if (c >= 1000000)
		capacity2text (c, buffer, (Capacity) 1000000, 'M');
	else if (c >= 1000)
		capacity2text (c, buffer, (Capacity) 1000, 'K');
	else
		capacity2text (c, buffer, (Capacity) 1, ' ');
	return buffer;
}

extern String capacity2string (c)
	Capacity c;
{
	static char buffer[32];

	if (c == UNKNOWN_CAPACITY)
		return "";
	(void) sprintf (buffer, "%lu", c);
	return buffer;
}

extern Capacity string2capacity (s)
	String s;
{
	Capacity c;
	int cc;
	cc = sscanf (s, "%lu", &c);
	if (cc != 1)
		c = UNKNOWN_CAPACITY;
	return c;
}

extern String cardinal2string (c)
	Cardinal c;
{
	static char buffer[32];

	(void) sprintf (buffer, "%u", c);
	return buffer;
}

extern Cardinal string2cardinal (s)
	String s;
{
	Cardinal c;
	int cc;
	cc = sscanf (s, "%lu", &c);
	if (cc != 1)
		c = 0;
	return c;
}

/***************************************************************/
/***************************************************************/
/*  $Header: //gaia/group.local/computer/monet/lib/db/RCS/status.c,v 1.1 1991/06/04 07:10:28 a38 Exp a38 $  */
/***************************************************************/
/***************************************************************/

