
#ifndef MONET_CHANNEL_H
#define MONET_CHANNEL_H

/***************************************************************************/
/*  Select Scheduler routines                                              */
/***************************************************************************/
/*  The following routines must be supplied by some external module, this  */
/*  module will be responsible for listening to sockets using 'select' and */
/*  calling the supplied handler routines when a connection becomes pending*/
/*  !!! Needs to define an event loop routine  */
/***************************************************************************/

typedef struct input_fid * InputFid;
typedef enum {fid_write, fid_read} FidType;

/***************************************************************************/
/*  Prototypes for server  */
/***************************************************************************/

#ifdef MONET_PROTOTYPES

/* Returns false if it wants to end the connection */
typedef Boolean (* FidHandler) (int, caddr_t);

extern void close_server (void);
extern InputFid create_input_fid (int, FidType, FidHandler, caddr_t);
extern void free_input_fid (int);
extern Boolean start_server (void);

#else

typedef Boolean (* FidHandler) ();

extern void close_server ();
extern InputFid create_input_fid ();
extern void free_input_fid ();
extern Boolean start_server ();

#endif

/********************************************************************************/
/*  Shome simple to use comms routines between to applications  */
/*  Currently implemented for tcp/ip only */
/********************************************************************************/

typedef struct channel * Channel;
#define NULL_CHANNEL (Channel) 0

#ifdef MONET_PROTOTYPES

/* -- Channel handler returns false to disconnect */
typedef Boolean	(* ChannelHandler)    	(caddr_t, String);
typedef void	(* ChannelDisconnect) 	(caddr_t);
extern void	Channel_free		(Channel);
extern Channel	Channel_create		(int fid, ChannelHandler, ChannelDisconnect, caddr_t data);
extern Channel	Channel_create_client	(int server_port, String server_address, ChannelHandler, ChannelDisconnect, caddr_t);
extern void	Channel_flush		(Channel);
extern void	Channel_end_of_record	(Channel);
extern void	Channel_send		(Channel, String);
extern int	Channel2fid		(Channel);
extern Channel	fid2Channel		(int fid);
extern void	Channel_init		(void);
extern void	Channel_finish		(void);

#else

typedef Boolean	(* ChannelHandler)    	();
typedef void	(* ChannelDisconnect) 	();
extern void	Channel_free		();
extern Channel	Channel_create		();
extern Channel	Channel_create_client	();
extern void	Channel_flush		();
extern void	Channel_end_of_record	();
extern void	Channel_send		();
extern int	Channel2fid		();
extern Channel	fid2Channel		();
extern void	Channel_init		();
extern void	Channel_finish		();

#endif

#endif
