
#ifndef MONET_DRAWLIB_H_
#define MONET_DRAWLIB_H_

#include "general/General.h"

/***********************************************************************/
/*  Scale pictures are to be drawn at                                      */
/***********************************************************************/

typedef struct scale * Scale;

#define BASE_CHAR_WIDTH  6.0
#define BASE_CHAR_HEIGHT 10.0
#define MAXSCALENAMELEN  81

/***********************************************************************/
/* Shade can mean, dashing, grey scale, or colours */
/***********************************************************************/


typedef enum {
	shade_0,
	shade_1,
	shade_2,
	shade_3,
	shade_4,
	shade_5
#define NUM_SHADE 6
	}
	Shade;
#define shade_background  shade_0
#define shade_dark   shade_4
#define shade_grey   shade_2
#define shade_heavy  shade_5

typedef Cardinal BitmapId;
/***********************************************************************/

/***********************************************************************/

typedef struct drawLibrary {
#ifdef MONET_PROTOTYPES_THISBIT_DONT_WORK_ON_APOLLOS
	/* !!! Not allowed apparently  */
	void (* end_draw)		(void);
	void (* draw_percentage_line)	(Shade, Load, Load, Position, Position, Position, Position);
	void (* draw_string)		(Shade, Position, Position, String);
	void (* draw_line)		(Shade, Position, Position, Position, Position);
	void (* draw_elipse)		(Shade, Position, Position, Dimension, Dimension);
	void (* draw_rectangle)		(Shade, Position, Position, Dimension, Dimension);
	void (* draw_box)		(Shade, Position, Position, Dimension, Dimension, Position, Position);
	void (* draw_roundedRectangle)	(Shade, Position, Position, Dimension, Dimension);
	void (* fill_elipse)		(Shade, Position, Position, Dimension, Dimension);
	void (* fill_rectangle)		(Shade, Position, Position, Dimension, Dimension);
	void (* fill_roundedRectangle)	(Shade, Position, Position, Dimension, Dimension);
	void (* draw_bitmap)		(BitmapId, Position, Position);
	void (* register_bitmap)	(BitmapId, Dimension, Dimension, char * bits);
	void (* set_scale)		(float, float);
	void (* set_reverse)		(void);
#else
	void (* end_draw) ();
	void (* draw_percentage_line) ();
	void (* draw_string) ();
	void (* draw_line) ();
	void (* draw_elipse) ();
	void (* draw_rectangle) ();
	void (* draw_box) ();
	void (* draw_roundedRectangle) ();
	void (* fill_elipse) ();
	void (* fill_rectangle) ();
	void (* fill_roundedRectangle) ();
	void (* draw_bitmap)		();
	void (* register_bitmap)	();
	/* -- set_scale is really set text scale  */
	void (* set_scale)		();
	/* -- draw in reverse video, to emulate XOR drawing */
	void (* set_reverse)		();
#endif
} * DrawLibrary;

#define NEWDRAWLIB(Lib) \
	Lib = MonetNew (struct drawLibrary);   \
	Lib->end_draw = end_draw;               \
	Lib->draw_percentage_line = draw_percentage_line; \
	Lib->draw_string = draw_string; \
	Lib->draw_line = draw_line;      \
	Lib->draw_elipse = draw_elipse;   \
	Lib->draw_box = draw_box;           \
	Lib->draw_rectangle = draw_rectangle;\
	Lib->draw_roundedRectangle = draw_roundedRectangle;\
	Lib->fill_elipse = fill_elipse;               \
	Lib->fill_rectangle = fill_rectangle;          \
	Lib->fill_roundedRectangle = fill_roundedRectangle;    \
	Lib->draw_bitmap = draw_bitmap;                  \
	Lib->register_bitmap = register_bitmap;   \
	Lib->set_scale = set_scale; \
	Lib->set_reverse = set_reverse;

extern DrawLibrary default_drawLibrary;

#define create_drawLibrary()  (MonetNew (struct draw_library));
#define free_drawLibrary(l)    MonetFree (l);


/*********************************************************/
/*  Text  */
/*********************************************************/

extern DrawLibrary TextLib;

#ifdef MONET_PROTOTYPES
extern void finish_draw_text (void);
extern void init_draw_text (void);
extern Boolean start_draw_text (String);


#else

extern void finish_draw_text ();
extern void init_draw_text ();
extern Boolean start_draw_text ();

#endif

/*********************************************************/
/*  PS  */
/*********************************************************/

extern DrawLibrary DrawPSLib;

#ifdef MONET_PROTOTYPES
extern void finish_draw_postscript (void);
extern void init_draw_postscript (void);
extern Boolean start_draw_postscript (String, String, String, Position, Position, Position, Position);
#else
extern void finish_draw_postscript ();
extern void init_draw_postscript ();
extern Boolean start_draw_postscript ();
#endif

/*********************************************************/
/*  General  */
/*********************************************************/

#ifdef MONET_PROTOTYPES

extern void	init_drawlib	(void);
extern void	finish_drawlib	(void);

extern String	Scale2String	(Scale);
extern Scale	String2Scale	(String);
extern Scale	Scale_create	(Dimension w, Dimension h);
extern void	Scale_free	(Scale);
extern void	Scale_xy	(Scale, float *, float *);

extern void Draw_start_draw		(DrawLibrary, float x_scale, float y_scale);
extern void Draw_end_draw		(void);
extern void Draw_draw_percentage_line	(Shade, Load, Load, Position, Position, Position, Position);
extern void Draw_draw_string		(Shade, Position, Position, String);
extern void Draw_draw_line		(Shade, Position, Position, Position, Position);
extern void Draw_draw_elipse		(Shade, Position, Position, Dimension, Dimension);
extern void Draw_draw_circle		(Shade, Position, Position, Dimension);
extern void Draw_draw_rectangle		(Shade, Position, Position, Dimension, Dimension);
extern void Draw_draw_box		(Shade, Position, Position, Dimension, Dimension, Position, Position);
extern void Draw_draw_roundedRectangle	(Shade, Position, Position, Dimension, Dimension);
extern void Draw_fill_circle		(Shade, Position, Position, Dimension);
extern void Draw_fill_elipse		(Shade, Position, Position, Dimension, Dimension);
extern void Draw_fill_rectangle		(Shade, Position, Position, Dimension, Dimension);
extern void Draw_fill_roundedRectangle	(Shade, Position, Position, Dimension, Dimension);
extern void Draw_draw_bitmap		(BitmapId, Position, Position);
extern void Draw_register_bitmap	(BitmapId, Dimension, Dimension, char * bits);
extern void Draw_set_box		(Position x, Position y, Dimension in_w, Dimension in_h, Dimension out_w, Dimension out_h);
extern void Draw_reset_box		(Position x, Position y, Dimension in_w, Dimension in_h, Dimension out_w, Dimension out_h);
extern void Draw_set_reverse		(void);

#else

extern String	Scale2String	();
extern Scale	String2Scale	();
extern Scale	Scale_create	();
extern void	Scale_free	();
extern void	Scale_xy	();

extern void init_drawlib	();
extern void finish_drawlib	();

extern void Draw_start_draw();
extern void Draw_end_draw();
extern void Draw_draw_percentage_line();
extern void Draw_draw_string();
extern void Draw_draw_line();
extern void Draw_draw_elipse();
extern void Draw_draw_circle		();
extern void Draw_draw_rectangle		();
extern void Draw_draw_box		();
extern void Draw_draw_roundedRectangle	();
extern void Draw_fill_circle		();
extern void Draw_fill_elipse		();
extern void Draw_fill_rectangle		();
extern void Draw_fill_roundedRectangle	();
extern void Draw_draw_bitmap		();
extern void Draw_register_bitmap	();
extern void Draw_set_box		();
extern void Draw_reset_box		();
extern void Draw_set_reverse		();

#endif

#endif

