/***************************************************************/
/***************************************************************/
/*  $Header$  */
/***************************************************************/
/***************************************************************/

#ifndef  MONET_GENERAL_H
#define  MONET_GENERAL_H

#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <assert.h>
#include <ctype.h>
extern int errno;


#ifndef MONET_PROTOTYPES
#if defined(__STDC__) || defined(__cplusplus) || defined(c_plusplus)
#define MONET_PROTOTYPES
#endif
#endif

/* !!! Causes problems on apollo because does not handle prototypes in
   !!! typedefs.
#ifdef MONET_PROTOTYPES
#define XTFUNCPROTO
#endif
*/

#ifdef MONET_PROTOTYPES
#define PR(prototype) prototype
#else
#define PR(prototype) ()
#endif


#include "general/Config.h"


typedef unsigned char Load;  /*  Load as a fraction of FULL_LOAD  */
#define FULL_LOAD ((Load) 254)
#define ZERO_LOAD ((Load) 0)
#define UNKNOWN_LOAD ((Load) 255)


/**********************************************/
/*  X11R3/X11R4/DECwindows/ compatibility    */
/*  or for programs with no Windows           */
/**********************************************/

/* According to the documentation XtRealloc will correctly handle */
/* a NULL first parameter */

#if defined(NOX11)
typedef int Boolean;
typedef char * String;
typedef unsigned int Cardinal;
#else
#include <X11/Intrinsic.h>
#endif

#if !defined(X11R4)
typedef Opaque XtPointer;
#endif

/****************************************************************
 *
 * Memory Management
 *
 * The memory management routines are not defined in the general
 * library because they may be replaced by something more suitable.
 * When using X11 it is best to use the X11 allocation routines.
 *
 ****************************************************************/

#define MonetNew(type) ((type *) MonetMalloc((unsigned) sizeof(type)))
#define MonetNewString(str) \
    ((str) == NULL ? NULL : (strcpy(MonetMalloc((unsigned)strlen(str) + 1), str)))

extern char * MonetMalloc (); /* size */
    /* Cardinal size; */

extern char * MonetRealloc (); /* ptr, num */
    /* char     *ptr; */
    /* Cardinal num; */

extern void MonetFree (); /* ptr */
        /* char  *ptr */

#include "general/UserInform.h"
#include "general/Help.h"

/********************************************************************************************/
/*  Some machines crash when NULL passed to string routine  */
/********************************************************************************************/

#define MonetStrlen(s)  (s ? strlen (s) : 0)

/********************************************************************************************/
/*  String functions  */
/*  strcmp is not used because it breaks on some machines with parameters == NULL           */
/*  Simply testing for NULLness is not sufficient because "" == NULL should be True         */
/********************************************************************************************/

#define EQS(a,b) strings_are_equal (a,b)
#define NES(a,b) (! EQS(a,b))
#define SPACEORTAB(c) ((c) == ' ' || (c) == '\t')
#define WHITESPACE(c) ((c) == ' ' || (c) == '\n' || (c) == '\t')

/**********************************************/
/* List handling  macros */
/**********************************************/

#define FORALL(v,head,next) for (v=(head); (v)!=NULL; v=(v->next))
#define LIST_ADD(new,head,next) { if (head==NULL) {new->next=NULL;} else {new->next=head;}; head=new;}
#define LIST_DEL(type, old,head,next) \
		if (head == old) \
			head = old->next; \
		else { \
			type v; \
			FORALL(v,head,next) \
				if (v->next == old) { \
					v->next = old->next; \
					break; \
				} \
		} \
		old->next = NULL;

/**********************************************/
/*  Debuging stuff  */
/**********************************************/

extern char * CurrentFunction;
#define PROC(function)   { enter_procedure (function); }
#define CHECKER  int Check  
#define CHECKNUMBER   19999
#define FREED         19788
#define SETCHECK(s)     {s -> Check = CHECKNUMBER; }
#define BADCHECK(s)  (s -> Check != CHECKNUMBER)
#define CHECKERROR(s) {sys_error ("Bad CHECKNUMBER in [%s]", "s");} 
#define UNSETCHECK(s) {s -> Check = FREED; }
#define CHECKSTRUCTURE(s)  { if ((char *) s == (char *) NULL) \
	{ sys_warning ("Structure s == NULL"); } \
	else if (BADCHECK(s)) { CHECKERROR(s); }  }

/**********************************************/
/*  Test mode  */
/**********************************************/

#define IF_TESTING  if (Config_testing)

/**********************************************/
/*  C++  ANSI-C  Prototypes                   */
/*  Not really tested                         */
/**********************************************/

#ifdef MONET_PROTOTYPES

typedef void (* MonetFinishRoutine) (void);
extern void	Monet_init		(String program, 
	MonetFinishRoutine finish,
	int argc, char ** argv);
extern void	Monet_finish		(void);
extern String MonetJoinStrings		(int argc, char ** argv, char * between);
extern String MonetJoinStringsUntilNull (char ** argv, char * between);
extern String MonetReplaceInString	(String source, String search, String replacement);
extern String MonetStringCopyToEnd	(String dest, String src);
extern String MonetTidyString		(String source);
extern Boolean MonetIsWord		(String source);
/* -- looks in 'place' for next non whitespace char and returns
	pointer to it.  'place' is set to next whitespace(or null)
	 char after this. */
static String MonetNextWord		(String *place);
extern Boolean configure_string		(String variable, String value);
extern void copyright_message		(void);
extern void version_message		(void);
extern void config_options_message	(void);
extern void config_params_message	(void);
extern Boolean strings_are_equal	(String, String);
extern char * MonetFilename		(char *);
extern char * MonetStrcat		(char *, char *);
extern void enter_procedure		(char *);
extern char * tidy_name			(String);
extern void my_error			();
extern String Monet_findfile		(String dir, String pattern);
extern Boolean Monet_list_dir		(String dir, String  pattern);
extern FILE * MonetFopen		(String name, String mode);

#ifndef ultrix
/* Haven't used varargs with the implementation !!! */

extern void sys_error			(char *, ...);
extern void sys_warning			(char *, ...);
extern void debug_message		(char *, ...);
extern void debug			(char *, ...);
extern void debug_on			(void);
extern void debug_off			(void);

#endif

#else  /* MONET_PROTOTYPES */

typedef void (* MonetFinishRoutine) ();

extern void	Monet_init		();
extern void	Monet_finish		();
extern String MonetJoinStrings		();
extern String MonetJoinStringsUntilNull ();
extern String MonetReplaceInString	();
extern String MonetStringCopyToEnd	();
extern String MonetNextWord		();
extern String MonetTidyString		();
extern Boolean MonetIsWord		();
extern Boolean Monet_list_dir		();
extern String Monet_findfile		();
extern Boolean configure_string		();
extern void config_options_message	();
extern void config_params_message	();
extern void copyright_message		();
extern void copyright_message		();
extern Boolean strings_are_equal	();
extern char * MonetFilename		();
extern char * MonetStrcat		();
extern void sys_error			();
extern void sys_warning			();
extern void debug_message		();
extern void enter_procedure		();
extern char * tidy_name			();
extern char  * my_malloc		();
extern char * copy_string		();
extern void my_error			();
extern void sys_error			();
extern void sys_warning			();
extern void debug			();
extern void debug_on			();
extern void debug_off			();
extern char * newstring			();
extern FILE * MonetFopen		();

#endif /* MONET_PROTOTYPES */


/**********************************************************/
/**********************************************************/
/*  Config values used by the system  */
/*  Use these if you need something   */
/**********************************************************/
/**********************************************************/

extern String Config_snmp_mib_file;
extern String Config_x121_file;

extern long Config_monetd_delay;
extern long Config_monetd_writeout_period;
extern long Config_monetd_check_X25_file_delay;
extern long Config_monetd_reconnect_redirects_delay;

extern String Config_local_host;

extern long Config_pinger_port;
extern String Config_pinger_address;
extern long Config_mapserver_port;
extern String Config_mapserver_address;

extern String Config_map_directory;
extern String Config_class_edb;
extern long Config_reverse_video;	/* Boolean:White on Black */
extern long Config_max_map_depth;  /* !!! */

extern String  Config_log_file;   /* !!! */
extern String  Config_ideas_log;
extern String  Config_log_directory;
extern String  Config_program_name;
extern long	Config_testing; /* Boolean */
extern long	Config_debug; /* Boolean */
extern String	Config_monetd_init_file;
/* General directory for related files */
extern String  Config_etc_directory;

/* Information about the software itself, built in */
extern String  Config_software_author;
extern String  Config_software_contact;
extern String  Config_software_source;
extern String  Config_software_title;
extern String  Config_software_version;

/* Information about maps, configurable */
extern String  Config_maps_author;
extern String  Config_maps_source;
extern String  Config_maps_contact;
extern String  Config_maps_postscript_prolog;


/* -- Execute this command at start up  */
extern String  Config_startup_command;

extern String  Config_postscript_printer_command;
extern String  Config_rping;

extern String  Config_user_id;

extern String  Config_entity_database_file;

/* Use a tcp connection with the nameserver instaed of udp */
extern long Config_tcp_dns;  /* Boolean */


extern MonetFinishRoutine Config_finish_handler;

/**********************************************************/
/**********************************************************/

#endif  /* MONET_GENERAL_H */

/***************************************************************/
/***************************************************************/
/*  $Header$  */
/***************************************************************/
/***************************************************************/

