/***********************************************************************/
/***********************************************************************/
/*  $Header$  */
/***********************************************************************/
/***********************************************************************/

#include "General.h"
#include "Help.h"
#include <sys/param.h> /* For MAXHOSTNAMELEN */
#include <sys/types.h>
#include <netdb.h>
#include <pwd.h>

#define DEFAULT_CONFIG_FILE "monetrc"

/***********************************************************************/
/*  Configuration files:                                               */
/*  Lowest level system file, ~/.<config>, and those on command line     */
/*  Can configure from the command line too                            */
/***********************************************************************/

int ABC = 0;

#define SET(config,value) if (config != NULL) MonetFree(config); config = MonetNewString (value);

MonetFinishRoutine Config_finish_handler = NULL;

long	Config_monetd_check_X25_file_delay;
long	Config_monetd_reconnect_redirects_delay;
long	Config_monetd_writeout_period;
long	Config_monetd_delay;
long	Config_pinger_port;
long	Config_mapserver_port;
long	Config_max_map_depth;
long	Config_testing;
long	Config_debug;
long	Config_reverse_video;
long	Config_tcp_dns;
String	Config_monetd_init_file;
String	Config_entity_database_file;
String   Config_user_id		;
String   Config_local_host	;
String   Config_snmp_mib_file	;
String   Config_x121_file	;
String   Config_pinger_address	;
String   Config_mapserver_address;
String   Config_map_directory	;
String   Config_class_edb	;
String   Config_log_file	;
String   Config_ideas_log	;
String   Config_program_name	;
String   Config_log_directory	;
String   Config_software_author	;
String   Config_software_contact;
String   Config_software_source	;
String   Config_software_title	;
String   Config_software_version;
String   Config_maps_author	;
String   Config_maps_contact	;
String   Config_maps_source	;
String   Config_etc_directory	;
String   Config_maps_postscript_prolog;
String   Config_postscript_printer_command;
String   Config_rping;
String	Config_startup_command;

/***********************************************************************/
/*  Configuration defaults table  */
/***********************************************************************/

static struct {
	long	default_value;
	String name;	/*  If NULL cannot be configured  */
	long * value;
	String option;
	String help;
} IntConfig [] = {
	/* -- Obsolete parameters */
	{ 0,		NULL,				&Config_max_map_depth, 		NULL, NULL },
	/* -- Integer valued parameters */
	{ 300,		"x25_delay",		&Config_monetd_check_X25_file_delay, "-x25delay",
		"Period between checking x25 host/address file has changed" },
	{ 660,		"reconnect_delay",	&Config_monetd_reconnect_redirects_delay, "-reconnectdelay",
		"Period(secs) between trying to reconnect to monetd servers" },
	{ 24*30*60,	"log_period",			&Config_monetd_writeout_period, "-logperiod",
		"Period(secs) between writing out batched status to log file" },
	{ 30,		"monetd_delay",			&Config_monetd_delay,		"-monetddelay",
		"Period(secs) between sending out network queries" },
	{ 2567,		"monetd_port",			&Config_pinger_port,		"-p",
		"Tcp port of monetd server" },
	{ 2568,		"monetmapd_port",		&Config_mapserver_port, 	"-mp",
		"Tcp port of map server" },
	/* -- Boolean valued parameters */
	{ 0,		"reverse_video",		&Config_reverse_video, 		"-map_rv",
		"Put maps in reverse video" },
	{ 0,		"testing",			&Config_testing, 		"-testing",
		"Enable untested/dangerous features" },
	{ 0,		"debug",			&Config_debug,	 		"-debug",
		"Debug mode" },
	{ 0,		"tcp_dns",			&Config_tcp_dns, 		"-tcp_dns",
		"Set up tcp connection to nameserver, instead of using IP" },
	{ 0,		NULL, NULL, NULL }
};

static struct {
	String default_value;
	String name;	/*  If NULL cannot be configured  */
	String * value;
	String option;
	String help;
} StrConfig [] = {
	/* -- Obsolete parameters */
	{ "logfile",	NULL,		&Config_log_file,		"-logfile", NULL},

	/* -- These cannot be reset by the user */	
	{ "ftp nikhefh.nikhef.nl",NULL, 	&Config_software_source,	NULL,
		"Where Monet Software is available from" },
	{ "James Barr",	NULL,			&Config_software_author,	NULL,
		"Author of Monet" },
	{ "a38@nikhef.nl",NULL,	 		&Config_software_contact,	NULL,
		"Where to send queries to" },
	{ "Monet", 	NULL,			&Config_software_title,		NULL,
		"Title of Software package" },
	{ "0.2-Beta",	NULL,			&Config_software_version,	NULL,
		"Current Version" },
	/* -- Set up from the system  */
	{ "",		NULL,			&Config_user_id,		NULL, NULL },
	{ "localhost",	NULL,			&Config_local_host,		NULL, NULL },
	{ "",		NULL,			&Config_program_name,		NULL, NULL },
	/* -- String valued parameters */
	{ "",		"maps_source",		&Config_maps_source,		"-mapssource",
		"Where maps are available from" },
	{ "",		"maps_author",		&Config_maps_author,		"-mapsauthor",
		"Who drew the maps" },
	{ "",		"maps_contact",		&Config_maps_contact,		"-mapscontact",
		"Who to contact about the maps" },
	{ "monetd.conf","monetd_conf",		&Config_monetd_init_file,	"-monetdconf",
		"Default requests file for monetd" },
	{ "monet.edb",	"edb_file",		&Config_entity_database_file,	"-edbfile",
		"Cache/default Entity Database file" },
	{ "maps",	"map_directory",	&Config_map_directory,		"-maps",
		"Directory where maps are kept" },
	{ "class.edb",	"class_edb",		&Config_class_edb,		"-classedb",
		"File defining entity classes" },
	{ "localhost",	"monetd_host",		&Config_pinger_address,		"-a",
		"Host name of monetd server" },
	{ "localhost",	"monetmapd_host",	&Config_mapserver_address,	"-ma",
		"Host name of map server" },
	{ C_ETC_DIRECTORY,"etc_directory",	&Config_etc_directory,		"-etc",
		"Where monet specific files are kept" },
	{ "maps.PS",	"map_ps_prolog",	&Config_maps_postscript_prolog,	"-mappspro",
		"Postscript prologue for maps" },
	{ "log",	"log_directory",	&Config_log_directory,		"-logdir",
		"Directory where logs are kept" },
	{ "ideas.txt",	"ideas_log",		&Config_ideas_log,		"-ideaslog",
		"Name of ideas log file" },
	{ "mib.txt",	"snmp_mib_file",	&Config_snmp_mib_file,		"-snmpmib",
		"File name of Snmp MIB in RFCxxxx format" },
	{ "x25hosts",	"x121_file",		&Config_x121_file,		"-x121file",
		"File name of x25 host/address translation table" },
	{ "pslpr",	"ps_lpr",		&Config_postscript_printer_command, "-pslpr",
		"Command for sending files to a postscript printer" },
	{ "/etc/rping",	"rping",		&Config_rping,			"-rping",
		"Name of a program for doing one-off IP pings" },
	{ "",	"startup_command",		&Config_startup_command,	"-exec",
		"A monet script that will be executed at startup" },
	{ NULL, 	NULL, NULL, NULL }
};

#define FORCONFIG(array) { Cardinal c; 	for (c=0; array[c].value != NULL; ++c) {
#define ENDFORCONFIG }}
#define FORNAMEDCONFIG(array) FORCONFIG(array) if (array[c].name) {
#define ENDFORNAMEDCONFIG  ENDFORCONFIG }
#define STRC StrConfig[c]
#define INTC IntConfig[c]

/***********************************************************************/
/*  Set config values  */
/***********************************************************************/

static Boolean configure (variable, string_value, integer_value)
	String variable, string_value;
	long integer_value;
{
	int c;

	if (!variable || ! (*variable))
		return False;
	debug_message ("Configure [%s] = [%s]", variable, string_value);

	FORNAMEDCONFIG(StrConfig)
		if (EQS(variable, STRC.name) || EQS(variable, STRC.option)) {
			SET (*(STRC.value), string_value);
			return True;
		}
	ENDFORNAMEDCONFIG
	FORNAMEDCONFIG(IntConfig)
		if (EQS(variable, INTC.name) || EQS(variable, INTC.option)) {
			*(INTC.value) = integer_value;
			return True;
		}
	ENDFORNAMEDCONFIG
	/*  Do not warn for command line options, done elsewhere  */
	if (*variable != '-')
		user_error ("Unknown configuration parameter [%s]", variable);
	return False;
}

/***********************************************************************/

extern Boolean configure_string (variable, value)
	String variable, value;
{
	if (value) {
		long i = atol (value);
		return configure (variable, value, i);
	}
	user_error ("Missing value for attribute [%s]", variable);
	return False;
}

/***********************************************************************/

static Boolean configure_integer (variable, value)
	String variable;
	long value;                                       
{
	static char string[64]; /* !!! Should be enough */

	(void) sprintf (string, "%ld", value);
	return configure (variable, string, value);
}

/***********************************************************************/
/*  Display configuration options                                      */
/***********************************************************************/

extern void config_options_message ()
{
	FORNAMEDCONFIG(StrConfig)
		user_inform ("\t%-15s\t-\t%s\t", STRC.option, STRC.help);
	ENDFORNAMEDCONFIG
	FORNAMEDCONFIG(IntConfig)
		user_inform ("\t%-15s\t-\t%s\t", INTC.option, INTC.help);
	ENDFORNAMEDCONFIG
}

/***********************************************************************/
/*  Display configuration parameters                                   */
/***********************************************************************/

extern void config_params_message ()
{
	FORNAMEDCONFIG(StrConfig)
		user_inform ("\t%-15s\t- %s", STRC.name, STRC.help);
		user_inform ("\t\t(%s)\t= [%s]", STRC.option, *(STRC.value));
	ENDFORNAMEDCONFIG
	FORNAMEDCONFIG(IntConfig)
		user_inform ("\t%-15s\t- %s", INTC.name, INTC.help);
		user_inform ("\t\t(%s)\t= [%ld]", INTC.option, *(INTC.value));
	ENDFORNAMEDCONFIG
}

/***********************************************************************/
/*  Display version  */
/***********************************************************************/

extern void version_message ()
{
	user_inform ("[%s] Version %s", Config_program_name, Config_software_version);
	user_inform ("The [%s] package is available from [%s]", Config_software_title, Config_software_source);
	user_inform ("For more information contact [%s]", Config_software_contact);
	user_inform ("");
}

/***********************************************************************/
/*  Display copyright  */
/***********************************************************************/

extern void copyright_message ()
{
	String name;

	name = MonetNewString (Config_program_name);
	if (islower(name[0]))
		name[0] = toupper(name[0]);
	else if (islower(name[1]))
		name[1] = toupper(name[1]);

	user_inform ("***********************************************************************");
	user_inform ("%s version 0.2-Beta, Copyright (C) 1991 NIKHEF-H, Amsterdam", name);
	user_inform ("%s comes with ABSOLUTELY NO WARRANTY.", name);
	user_inform ("This is free software, and you are welcome to redistribute it");
	user_inform ("under certain conditions.  For details see documentation.");
	user_inform ("***********************************************************************");
	user_inform ("");

	MonetFree (name);
}

/***********************************************************************/
/*  Load in named config file, will optionally display a warning if it */
/*  can't open the desired file.                                       */
/* !!! Does not truncate lines longer than 256 characters              */
/***********************************************************************/

extern Boolean load_config_file (name, warn)
	String name;
	Boolean warn;
{
	FILE *f;
	char buffer[256], attribute [256], value [256];  /* !!! */
	char * text;
	int cc;
	
	if ((f = fopen ((char *) MonetFilename (name), "r")) == 0) {
		if (warn)
			user_error ("Sorry, can't open configuration file [%s]", name);
		return False;
	}
	while ((text = fgets (buffer, sizeof(buffer)-1, f)) != NULL) {
		cc = sscanf (text, "%s %[^\n]", attribute, value);
		if (*attribute == '-' || *attribute == '#') {
			/* Comment */
		} else if (cc <= 0) {
			/* Blank line */
		} else if (cc != 2) {
			user_error ("Error in config file [%s] Line [%s]", name, text);
		} else {
			configure_string (attribute, value);
		}
	}
	fclose (f);
	return True;
}

/***********************************************************************/
/*  usage message  */
/***********************************************************************/

extern void usage_message ()
{
	user_inform ("usage: %s [-help [1|2]]  <monet options> ...",
		Config_program_name);
}

/***********************************************************************/
/*  Handle command line options                                        */
/***********************************************************************/

static void command_line_options (argc, argv)
	int argc;
	char **argv;
{
	Cardinal i;

	for (i = 0; i < argc; ++i) {
		String option = argv[i], value = argv[i+1];
		if (EQS(option, "-config") || EQS (option, "-c")) {
			if (i + 2 < argc)
				configure_string (value, argv[argc+2]);
		} else if (EQS(option, "-f")) {
			if (i + 1 < argc)
				load_config_file (value);
		} else if (EQS (option, "-help")) {
			usage_message ();
			if (EQS(value, "1"))
				config_options_message ();
			else if (EQS(value, "2"))
				config_params_message ();
			exit (0);                               
		} else if (*option == '-') {
			if (! value || *value == '-') {
				usage_message ();
				user_error ("Option [%s] must be followed by a value", option);
				exit (1);
			} else 	if (i + 1 < argc)
				configure_string (option, value);
		}
	}
}

/***********************************************************************/
/*  Initialise everything, read in config files  */
/***********************************************************************/

extern void init_configure (program_name, argc, argv)
	String program_name;
	int argc;
	char **argv;
{
	char buffer [MAXHOSTNAMELEN+32];
	struct hostent *host;
	int uid;
	struct passwd *pass;
 
	/*  -- Set up default_value values for all the parameters using the default_values table  */
	FORCONFIG(StrConfig)
		SET (*(STRC.value), STRC.default_value);
	ENDFORCONFIG
	FORCONFIG(IntConfig)
		*(INTC.value) = INTC.default_value;
	ENDFORCONFIG

	SET(Config_program_name, program_name);
	copyright_message ();

	/*  -- Set the local host name  */
	if (gethostname (buffer, sizeof(buffer)) != 0) {
		user_error ("Get Host Name Failed, don't know local host");
	} else if ((host = gethostbyname (buffer)) == NULL) {
		user_error ("Get Host By Name Failed, don't know local host");
	} else {
		SET(Config_local_host, host->h_name);
	}

	/* -- Set user id */
	uid = getuid ();
	pass = getpwuid (uid);
	if (pass == NULL) {
		sys_error ("Could not find password entry of uid [%d]", uid);
		SET(Config_user_id, "unknown");
	} else {
		SET(Config_user_id, pass->pw_name);
	}
	SET(Config_maps_author, Config_user_id);
	(void) sprintf (buffer, "%s@%s", Config_user_id, Config_local_host);
	SET(Config_maps_contact, buffer);
	(void) sprintf (buffer, "ftp %s", Config_local_host);
	SET(Config_maps_source, buffer);

	/* -- Do options once in case config file is mentioned  */
	command_line_options (argc, argv);

	/* -- Load config files  */
	{	char buffer [1024]; /* !!! */

		/* -- system config file */
		(void) sprintf (buffer, "%s/%s", Config_etc_directory, DEFAULT_CONFIG_FILE);
		(void) load_config_file (buffer, True);
		/* -- system config file */
		(void) sprintf (buffer, "%s/%src", Config_etc_directory, program_name);
		(void) load_config_file (buffer, False);
		/* -- user config file */
		(void) sprintf (buffer, "~/.%s", DEFAULT_CONFIG_FILE);
		if ( ! load_config_file (buffer, False)) {
			FILE * f;
			if (f = fopen ((char *) MonetFilename (buffer), "w")) {
				(void) fprintf (f, "# Created by [%s] on %s\n", Config_program_name,
						pretty_time_stamp ());
				(void) fprintf (f, "maps_author\t%s\n", Config_maps_author);
				(void) fprintf (f, "maps_contact\t%s\n", Config_maps_contact);
				(void) fprintf (f, "maps_source\t%s\n", Config_maps_source);
				user_inform ("Created new config file [%s]", MonetFilename (buffer));
				fclose (f);
			} else {
				user_error ("Sorry, can't open or create config file [%s]", MonetFilename (buffer));
 			}
		}
		/* -- user config file */
		(void) sprintf (buffer, "~/.%src", program_name);
		(void) load_config_file (buffer, False);

	}
	/* -- Do options second time, since they have priority  */
	command_line_options (argc, argv);
}

/***********************************************************************/
/*  Finish  */
/***********************************************************************/

extern void finish_configure ()
{
	/*  -- Free allocated values */
	FORCONFIG(StrConfig)
		MonetFree (*(STRC.value));
	ENDFORCONFIG
}
/***********************************************************************/
/***********************************************************************/
/*  $Header$  */
/***********************************************************************/
/***********************************************************************/


