/******************************************************************/
/******************************************************************/
/* $Header: //gaia/group.local/computer/monet/Draw/RCS/lib.c,v 1.1 1991/03/26 10:52:24 a38 Exp a38 $ */
/******************************************************************/
/******************************************************************/

#include "general/Draw.h"

DrawLibrary default_drawLibrary, Draw;

static Position x_offset, y_offset;
static float x_scale, y_scale;

extern void init_drawlib ()
{
	init_draw_text();
	init_draw_postscript();
	default_drawLibrary = NULL;
}

extern void finish_drawlib ()
{
	finish_draw_postscript();
	finish_draw_text();
}

/******************************************************************/
/* Adjust position depending on scale of map */
/******************************************************************/

#define SCALE(a, offset, factor) (offset + a * factor)
#define DESCALE(a, offset, factor) ((-offset + a) / factor)
#define x_adjust(x) SCALE(x, x_offset, x_scale)
#define y_adjust(y) SCALE(y, y_offset, y_scale)
#define x_readjust(x) DESCALE(x, x_offset, x_scale)
#define y_readjust(y) DESCALE(y, y_offset, y_scale)

#define w_adjust(w) SCALE(w, 0, x_scale)
#define h_adjust(h) SCALE(h, 0, y_scale)

/*****************************************************************/
/*  Return Pixel height and widths for strings */
/*  !!! We assume everything is 6x10, so no need to use XTxtWidth? */
/*****************************************************************/

extern Cardinal width_of_string (s)
	char * s;
{
	return 6 * MonetStrlen (s);
}

extern Cardinal height_of_string (s)
	char * s;
{
	return 8;
}

/*****************************************************************/
/*  Start up drawing batch  */
/*****************************************************************/

extern void Draw_start_draw (lib, xscale, yscale)
	DrawLibrary lib;
	float xscale, yscale;
{
	Draw = lib;
	x_offset = 0;
	y_offset = 0;
	x_scale = xscale;
	y_scale = yscale;
	Draw->set_scale (x_scale, y_scale);
}

extern void Draw_end_draw ()
{
	Draw->end_draw ();
}

extern void Draw_draw_percentage_line (shade,  n, d, xl, yl, xr, yr)
	Position        xl, yl, xr, yr;
	Shade shade;
	Load n , d;
{
	xl = x_adjust(xl);
	yl = y_adjust(yl);
	xr = x_adjust(xr);
	yr = y_adjust(yr);

	Draw->draw_percentage_line (shade,  n, d, xl, yl, xr, yr);
}

extern void Draw_draw_line (shade,  xl, yl, xr, yr)
	Position        xl, yl, xr, yr;
	Shade shade;
{
	xl = x_adjust(xl);
	yl = y_adjust(yl);
	xr = x_adjust(xr);
	yr = y_adjust(yr);

	Draw->draw_line (shade,  xl, yl, xr, yr);
}

extern void Draw_fill_elipse (shade, x, y, width, height)
	Shade shade;
	Position        x, y;
	Dimension       width, height;
{
	x = x_adjust(x);
	y = y_adjust(y);
	width = w_adjust(width);
	height = h_adjust(height);


	Draw->fill_elipse (shade, x, y, width, height);
}

extern void Draw_draw_elipse (shade, x, y, width, height)
	Shade shade;
	Position        x, y;
	Dimension       width, height;
{
	x = x_adjust(x);
	y = y_adjust(y);
	width = w_adjust(width);
	height = h_adjust(height);

	Draw->draw_elipse (shade, x, y, width, height);
}


extern void Draw_draw_rectangle (shade, x, y, width, height)
	Shade shade;
	Position        x, y;
	Dimension       width, height;
{
	x = x_adjust(x);
	y = y_adjust(y);
	width = w_adjust(width);
	height = h_adjust(height);

	Draw->draw_rectangle (shade, x, y, width, height);
}

extern void Draw_draw_roundedRectangle (shade, x, y, width, height)
	Shade shade;
	Position        x, y;
	Dimension       width, height;
{
	x = x_adjust(x);
	y = y_adjust(y);
	width = w_adjust(width);
	height = h_adjust(height);

	Draw->draw_roundedRectangle (shade, x, y, width, height);
}

extern void Draw_draw_box (shade, x, y, width, height, xspace, yspace)
	Shade shade;
	Position        x, y, xspace, yspace;
	Dimension       width, height;
{
	x = x_adjust(x);
	y = y_adjust(y);
	width = w_adjust(width);
	height = h_adjust(height);
	xspace = w_adjust(xspace);
	yspace = h_adjust(yspace);

	Draw->draw_box (shade, x, y, width, height, xspace, yspace);
}


extern void Draw_fill_rectangle (shade, x, y, width, height)
	Shade shade;
	Position        x, y;
	Dimension       width, height;
{
	x = x_adjust(x);
	y = y_adjust(y);
	width = w_adjust(width);
	height = h_adjust(height);

	Draw->fill_rectangle (shade, x, y, width, height);
}

extern void Draw_fill_circle (shade, x, y, diameter)
	Shade shade;
	Position        x, y;
	Dimension       diameter;
{
	Dimension width, height;
	x = x_adjust(x);
	y = y_adjust(y);
	width = w_adjust(diameter);
	height = h_adjust(diameter);

	Draw->fill_elipse (shade, x, y, width, height);
}

extern void Draw_draw_circle (shade, x, y, diameter)
	Shade shade;
	Position        x, y;
	Dimension       diameter;
{
	Dimension width, height;
	x = x_adjust(x);
	y = y_adjust(y);
	width = w_adjust(diameter);
	height = h_adjust(diameter);

	Draw->fill_elipse (shade, x, y, width, height);
}

extern void Draw_draw_string (shade, x, y, string)
	Shade shade;
	Position        x, y;
	String string;
{
	x = x_adjust(x);
	y = y_adjust(y);

	Draw->draw_string (shade, x, y, string);
}

extern void Draw_draw_bitmap (id, x,y)
	BitmapId id;
	Position x, y;
{
	Draw->draw_bitmap (id, x,y);
}

extern void Draw_register_bitmap	(id, w,h, bits)
	BitmapId id;
	Dimension w, h;
	char * bits;
{
	Draw->register_bitmap	(id, w,h, bits);
}

extern void Draw_set_box (x,y,in_w, in_h, out_w, out_h)
	Position x,y;
	Dimension in_w, in_h, out_w, out_h;
{
	x_offset = x_adjust(x);
	y_offset = y_adjust(y);

	if (in_w && in_h) {
		float w, h;
		w = (float) out_w / (float) in_w;
		h = (float) out_h / (float) in_h;
		if (w < h) {
			x_scale *= w;
			y_scale *= w;
		} else {
			x_scale *= h;
			y_scale *= h;
		}
		Draw->set_scale (x_scale, y_scale);
	} else {
		sys_error ("zero inside box %d %d", in_w, in_h);
	}
}

extern void Draw_reset_box (x,y,in_w, in_h, out_w, out_h)
	Position x,y;
	Dimension in_w, in_h, out_w, out_h;
{
	if (in_w && in_h) {
		float w, h;
		w = (float) out_w / (float) in_w;
		h = (float) out_h / (float) in_h;
		if (w < h) {
			x_scale /= w;
			y_scale /= w;
		} else {
			x_scale /= h;
			y_scale /= h;
		}
		Draw->set_scale (x_scale, y_scale);
	} else {
		sys_error ("zero inside box %d %d", in_w, in_h);
	}
	x_offset = x_readjust(x);
	y_offset = y_readjust(y);
}

extern void Draw_set_reverse ()
{
	Draw->set_reverse ();
}

/******************************************************************/
/******************************************************************/
/* $Header: //gaia/group.local/computer/monet/Draw/RCS/lib.c,v 1.1 1991/03/26 10:52:24 a38 Exp a38 $ */
/******************************************************************/
/******************************************************************/

