/****************************************************************************/
/****************************************************************************/
/*  $Header$  */
/****************************************************************************/
/*  Maintain key sequential lists of objects.  Can be used for any type  */
/*  of object since generic pointers 'caddr_t' are used.  Create a new      */
/*  header file for each new type of list.                                  */
/****************************************************************************/

#include "General.h"
#include "KeyList.h"

/****************************************************************************/
/*  Definitions  */
/****************************************************************************/

struct KeyList {
	Cardinal size, count;
	caddr_t * head;
	String search_key;
};

/****************************************************************************/

/****************************************************************************/

extern Boolean isMember_KeyList (olist, o)
	KeyList olist;
	caddr_t o;
{
	caddr_t * list = olist->head;
	Cardinal  num = olist->size, i;

	for (i=0; i<num; i++)
		if (o == list[i])
  			return True;
	return False;
}


extern Boolean extract_KeyList (olist, o)
	KeyList olist;
	caddr_t o;
{
	caddr_t * list = olist->head;
	Cardinal  num = olist->size, i;

	if (olist == NULL) {
		sys_error ("list is null");
		return False;
	}
	for (i=0; i<num; i++) {
		if (o == list[i]) {
			for (i++;i<num; i++)
				list[i-1] = list[i];
			olist->size --;
			num = olist -> size;
			/* MonetRealloc will correctly handle NULL first parameter */
			olist->head = (caddr_t *) MallocRealloc (olist->head, num*sizeof(caddr_t));
			/* !!! Just Testing */
			for (i=0; i<num; i++)
				if (olist->head[i] == o)
					sys_error ("Serious, object not removed");
  			return True;
		}
	}
	return False;
}

extern KeyList create_KeyList ()
{
	KeyList new;

	new = MonetNew (struct KeyList);
	new -> size = 0;
	new -> count = 0;
	new -> head = (caddr_t *) 0;
	new -> search_key = NULL;
	return new;
}

extern void reset_KeyList (new)
	KeyList new;
{
	MonetFree (new->head);
	new->head = (caddr_t *) 0;
	new->size = 0;
}

extern Cardinal sizeof_KeyList (list)
	KeyList list;
{
	return list->size;
}

extern Boolean isEmpty_KeyList (list)
	KeyList list;
{
	return list->size == 0;
}



extern caddr_t getN_KeyList (list, n)
	KeyList list;
	Cardinal n;
{
	if (n>list->size) {
		return NULL;
	}
	return list->head[n-1];
}



extern void free_KeyList (new)
	KeyList new;
{
	  if (new == NULL)
		    return;
  if (new->size != 0) {
     sys_error ("Freeing an object list before it is empty");
  }
	  MonetFree (new -> head);
	  MonetFree (new);
}

extern Cardinal  sizeOf_KeyList (olist)
	KeyList olist;
{
	return olist->size;
}

/* Guaranteed to handle null olist or null o properly */
extern void insert_KeyList (olist, o, key)
	KeyList olist;
	caddr_t o;
	char ** key;
{
	if (olist == (KeyList) 0) {
		sys_error ("insert_KeyList caddr_t (key) list is NULL");
		return;
	}
	if (o==NULL) {
		sys_error ("caddr_t is NULL");
		return;
	}
	if (isMember_KeyList (olist, o)) {
		sys_error ("[%d] is already a member of list", o);
		return;
	}
	olist->size ++;
	olist->head = (caddr_t *) MonetRealloc (olist->head, (olist->size)*sizeof(caddr_t));  /* See note above */
	olist->head[olist->size-1] = o;
}


extern caddr_t first_KeyList (l)
	KeyList l;
{
	if (l-> size == 0)
		return NULL;

	l -> count = 0;
	return l->head [0];
}


/* !!! This is so commonly used it is worth having a special routine */
extern caddr_t second_KeyList (l)
	KeyList l;
{
	if (l-> size < 2)
		return NULL;

	l -> count = 1;
	return l->head [1];
}


extern caddr_t next_KeyList (l)
	KeyList l;
{
	l->count++;
	if (l->size <= l->count)
		return (NULL);
	return l->head[l->count];
}

extern KeyList KeyList_search (list, key)
	KeyList list;
	char * key;
{
	caddr_t o;
	list -> search_key = key;
	return NULL;
}

extern KeyList KeyList_search_next (list)
	KeyList list;
{
	return NULL;
}

/****************************************************************************/
/****************************************************************************/
/*  $Header$  */
/****************************************************************************/
/****************************************************************************/

