/****************************************************************************/
/*									    */
/* Author : 	James Barr & Alena Lacova, NIKHEF-H			    */
/* Version :	10-Jul-1991						    */
/*									    */
/****************************************************************************/
/*  $Header$  */
/****************************************************************************/
/*  Maintain simple sequential lists of objects.  Can be used for any type  */
/*  of object since generic pointers 'caddr_t' are used.  Create a new      */
/*  header file for each new type of list.                                  */
/*  This should really be called ordered-set                                */
/****************************************************************************/

#include "General.h"
#include "SimpleList.h"

/****************************************************************************/
/*  Definitions  */
/****************************************************************************/
#define SIZE 16

struct simpleList {
	Cardinal size, count, available;
	caddr_t * head;
};

/****************************************************************************/

/****************************************************************************/

static Boolean Find(head, o, size, position)
caddr_t * head, o;
Cardinal size;
Cardinal * position;
{
	int min = 0, max = size, mid = max/2;
	if(head == NULL)
		return False;
	if (o==NULL) {
		(void) fprintf (stderr,"caddr_t is NULL\n");
		return False;
	}
	while(1)
	{
		if(o < head[mid])
		{
			max = mid;
			mid = (mid + min)/2;
			if(max == mid)
			{
				*position = max;
				break;
			}
		}
		else if (o > head[mid])
		{
			min = mid;
			mid = (max + mid)/2;
			if(min == mid)
			{
				*position = min + 1;
				break;
			}
		}
		else
		{
			*position = mid;
			return True;
		}
	}
	return False;
}
extern Boolean isMember_simpleList (olist, o)
	SimpleList olist;
	caddr_t o;
{
	int  position ;
 	
	if (olist == (SimpleList) 0) {
		(void) fprintf (stderr,"insert_simpleList caddr_t (simple) list is NULL\n");
		return False;
	}

	return(Find(olist->head, o, olist->size, &position)) ;
}

extern Boolean extract_simpleList (olist, o)
	SimpleList olist;
	caddr_t o;
{
	int  position ,i ;

	if (olist == NULL) {
		(void) fprintf (stderr,"list is null\n");
		return False;
	}

	if(Find(olist->head, o, olist->size, &position))
	{
		olist->size --;
		if(olist->size == 0)
		{
			MonetFree(olist->head);
			olist->head = NULL;
		}
		else 
			for(i=position; i<olist->size; i++)
				olist->head[i] = olist->head[i+1];
			if (olist->count > position)
				-- (olist->count);
		return True;
	}
	else
		return False;

}

extern SimpleList create_simpleList ()
{
	SimpleList new;

	new = MonetNew (struct simpleList);
	new -> size = 0;
	new -> count = 0;
	new -> available = 0;
	new -> head = (caddr_t *) 0;
	return new;
}

extern void reset_simpleList (new)
	SimpleList new;
{
	if (new -> head) {
		MonetFree (new->head);
		new->head = (caddr_t *) 0;
		new->size = 0;
	}
}

extern Cardinal sizeof_simpleList (list)
	SimpleList list;
{
	return list->size;
}

extern Boolean isEmpty_simpleList (list)
	SimpleList list;
{
	return list->size == 0;
}

extern caddr_t getN_simpleList (list, n)
	SimpleList list;
	Cardinal n;
{
	if (n>list->size) {
		return NULL;
	}
	return list->head[n-1];
}



extern void free_simpleList (new)
	SimpleList new;
{
	  if (new == NULL)
		    return;
  if (new->size != 0) {
     fprintf (stderr,"Freeing an object list before it is empty\n");
  }
	  MonetFree (new -> head);
	  MonetFree (new);
}

extern Cardinal  sizeOf_simpleList (olist)
	SimpleList olist;
{
	return olist->size;
}

/* Guaranteed to handle null olist or null o properly */

extern void insert_simpleList(olist, o)
SimpleList olist;
caddr_t o;
{
	caddr_t * newspace;
	int position;
	int i;
 	
	if (olist == (SimpleList) 0) {
		(void) fprintf (stderr,"insert_simpleList caddr_t (simple) list is NULL\n");
		return;
	}
	if (o==NULL) {
		fprintf (stderr,"caddr_t is NULL\n");
		return;
	}
	if(olist->head == NULL) {
		olist->head = (caddr_t *)MonetMalloc(SIZE * sizeof(caddr_t));
		olist->available = SIZE;
		olist->head[0] = o;
		olist->size = 1;
		return;
	}

	if(Find(olist->head, o, olist->size, &position)) 
	{
		fprintf(stderr,"There is such a member: %d [%s]\n",o, o);
		return;
	}
		
	olist->size ++;	

	if (olist->size > olist->available)
	{
		olist->available *= 2;
		newspace = (caddr_t *)MonetMalloc((olist->available) * sizeof(caddr_t));
		bcopy(olist->head, newspace, olist->available*sizeof(caddr_t));
		MonetFree(olist->head);
		olist->head = newspace;
	}
		
	for(i=olist->size-1; i>position; i--)
	{
		olist->head[i] = olist->head[i-1];
	}
	olist->head[i] = o;

	if (olist->count > position)
		++ (olist->count);

/*	printf("&o = %d, o = %s, olist->head[%d] = %d\n",o, o, olist->size-1, olist->head);  */

}

extern caddr_t first_simpleList (l)
	SimpleList l;
{
	if (l-> size == 0)
		return NULL;

	l -> count = 0;
	return l->head [0];
}


/* !!! This is so commonly used it is worth having a special routine */
extern caddr_t second_simpleList (l)
	SimpleList l;
{
	if (l-> size < 2)
		return NULL;

	l -> count = 1;
	return l->head [1];
}


extern caddr_t next_simpleList (l)
	SimpleList l;
{
	l->count++;
	if (l->size <= l->count)
		return (NULL);
	return l->head[l->count];
}

/****************************************************************************/
/****************************************************************************/
/*  $Header$  */
/****************************************************************************/
/****************************************************************************/

