/****************************************************************************************************/
/****************************************************************************************************/
/*  $Header$  */
/****************************************************************************************************/
/*  A set of routines for presenting user messages, requires a routine 'user_message'               */
/*  Could be cleaned up a bit */
/****************************************************************************************************/

#include "general/General.h"
#include "general/Log.h"

static UserMessageRoutine user_message = NULL;
ConfirmBoxRoutine confirm_box;
WriteReportRoutine write_report = NULL;

int yylineno;

/****************************************************************************************************/
/*  Globals  */
/****************************************************************************************************/

static Log TestLog;

/****************************************************************************************************/
/*  If testing send everything to a log file  */
/****************************************************************************************************/

static void TESTLOG (m, a,b,c,d,e,f,g,h,i)
	String m,a;
{
	IF_TESTING {
		write_log (TestLog, m, a,b,c,d,e,f,g,h,i);
		flush_log (TestLog);
	}
}

/*****************************************************************************/
/*  Standard print routine  */
/*****************************************************************************/

static void stdio_user_message (label0, a,b,c,d,e,f,g,h,i)
     char * a, * label0;
{
	char * label;

	if (label0 == NULL)
		label = "Inform";
	else {
		label = label0;
		printf ("[%s] ", label);
	}
	printf (a,b,c,d,e,f,g,h,i);
	printf ("\n");
}

/*************************************************************************/
/*  Write to the report  */
/*************************************************************************/

static void stdio_write_report (a,b,c,d,e,f,g,h,i)
	char * a;
{
	printf (a,b,c,d,e,f,g,h,i);
}

/*************************************************************************/
/*  Set writing routines  */
/*************************************************************************/

extern UserMessageRoutine set_user_inform_routine (writer)
	UserMessageRoutine writer;
{
	UserMessageRoutine previous = user_message;
	user_message = writer;
	return previous;
}

extern ConfirmBoxRoutine set_confirm_box_routine (confirmer)
	ConfirmBoxRoutine confirmer;
{
	ConfirmRoutine previous = confirm_box;
	confirm_box = confirmer;
	return previous;
}

extern WriteReportRoutine set_write_report_routine (reporter)
	WriteReportRoutine reporter;
{
	WriteReportRoutine previous = write_report;
	write_report = reporter;
	return previous;
}

/*************************************************************************/
/*  report routines */
/*************************************************************************/

extern void start_report (title,a,b,c,d,e,f,g,h)
char * title;
{
	char buffer [1024];  /* !!! */

	(void) sprintf (buffer, title, a,b,c,d,e,f,g,h);
	write_report ("Reporting on [%s]\n", buffer);
}

extern void writeln_report (a,b,c,d,e,f,g,h,i)
	char * a;
{
	write_report (a,b,c,d,e,f,g,h,i);
	write_report ("\n");
}

extern void end_report ()
{
}

static void stdio_confirm_box (proc, message)
	ConfirmRoutine proc;
	char * message;
{
	user_inform ("%s [yn]", message);
	if (getchar() == 'y')
		proc ();
} 

/****************************************************************************************************/
/*  Exported Message Routines
/****************************************************************************************************/

extern void user_warning (a,b,c,d,e,f,g,h,i)
     char * a;
{
	TESTLOG ("User Warning", a,b,c,d,e,f,g,h,i);
	user_message ("User Warning", a,b,c,d,e,f,g,h,i);
}

extern void user_error (a,b,c,d,e,f,g,h,i)
     char * a;
{
	TESTLOG ("User Error", a,b,c,d,e,f,g,h,i);
	user_message ("User Error", a,b,c,d,e,f,g,h,i);
}

extern void user_inform (a,b,c,d,e,f,g,h,i)
     char * a;
{
	TESTLOG ("User Inform", a,b,c,d,e,f,g,h,i);
	user_message (NULL, a,b,c,d,e,f,g,h,i);
}

extern void parse_error (a,b,c,d,e,f,g,h,i)
     char * a;
{
	char buffer [1024]; /* !!! */
	(void) sprintf (buffer, "Error at line %d", yylineno);
	user_message (buffer, a,b,c,d,e,f,g,h,i);
	TESTLOG (buffer, a,b,c,d,e,f,g,h,i);
}

extern void parse_warning (a,b,c,d,e,f,g,h,i)
     char * a;
{
	char buffer [1024]; /* !!! */
	(void) sprintf (buffer, "Warning at line %d", yylineno);
	user_message (buffer, a,b,c,d,e,f,g,h,i);
	TESTLOG (buffer, a,b,c,d,e,f,g,h,i);
}

/****************************************************************************************************/
/*  Init and finish  */
/****************************************************************************************************/

extern void init_user_inform ()
{
	user_message = stdio_user_message;
	write_report = stdio_write_report;
	confirm_box  = stdio_confirm_box;
	IF_TESTING {
		TestLog = create_log (Config_program_name, 0 , True);
	}
}

extern void finish_user_inform ()
{
	IF_TESTING {
		free_log (TestLog);
	}
}

/****************************************************************************************************/
/****************************************************************************************************/
/*  $Header$  */
/****************************************************************************************************/
/****************************************************************************************************/


