
/******************************************************************/
/******************************************************************/
/*  $Header: //gaia/group.local/computer/monet/lib/map/RCS/draw.c,v 1.1 1991/07/22 11:37:05 a38 Exp a38 $  */
/******************************************************************/
/*  Does all the drawing of maps, should really be split into two */
/*  parts, one for drawing the maps, and the other for actually   */
/*  the drawings.                                                 */
/******************************************************************/

#include "map/Drawing.h"
#include "general/Shape.h"
#include "map/Layout.h"

/************************************************************************/
/*  Globals  */
/************************************************************************/

#ifdef MONET_PROTOTYPES
static void     draw_connection(Drawing, Drawing);
static void	draw_map (Map);
static void     draw_interface(Drawing);
static void     draw_system(Drawing);
#else
static void     draw_connection();
static void	draw_map ();
static void     draw_interface();
static void     draw_system();
#endif

/************************************************************************/
/*  Recursize procedure for drawing maps  */
/************************************************************************/

static void draw_map (map)
	Map map;
{
	Drawing drawing;

	FOR_DRAWINGLIST(drawing, Map_elements (map))
		if (isInterface(ENTITY(drawing)))
			draw_interface (drawing);
		else if (drawing->submap) {
			Position xl, yl, xr, yr;
			draw_system (drawing);
			Map_corners (drawing->submap, &xl, &yl, &xr, &yr, False);
			Draw_set_box (X(drawing)+1, Y(drawing)+1,
					(Dimension) xr, (Dimension) yr,
					WIDTH(drawing)-2, HEIGHT(drawing)-2);
			Draw_set_reverse ();
			draw_map(drawing->submap);
			Draw_set_reverse ();
			Draw_reset_box (X(drawing)+1, Y(drawing)+1,
				(Dimension) xr, (Dimension) yr,
				WIDTH(drawing)-2, HEIGHT(drawing)-2);
		} else
			draw_system (drawing);
}

/************************************************************************/
/*  We only show emphasized items the rest are hiden */
/*  This is an hack !!! */
/************************************************************************/

static Boolean ShowEmphasizedOnly = False;
extern void set_showEmphasizedOnly (state)
	Boolean state;
{ ShowEmphasizedOnly = state; }

static Boolean isHidden (o)
	Drawing o;
{
	Entity e = ENTITY(o);

	if (Entity_isClass (e) || Drawing_isButton (o))
		return False;
	if (ShowEmphasizedOnly && SHOW(o) != show_emphasised)
		return True;
	if (SHOW(o) == show_hiden)
		return True;
	return False;
}

/************************************************************************/
/*  Draw connection between interface and systems                       */
/*  !!! replace this with draw connecting line                          */
/************************************************************************/

static void draw_connection(interface, sys)
	Drawing          interface, sys;
{
	Boolean straight, xstraight;
	Dimension length;
	Shade shade;
	Position        xl, yl, xr, yr;
	Load load;
	Map map = Drawing_mapOf(interface);
	Entity ei = ENTITY(interface);

	if (isHidden (interface) || isHidden (sys))
		return;

	choose_attachment(interface, sys, &xl, &yl, &xr, &yr, &straight, &xstraight, &length);
	if (STATUS(ei) != status_reachable && STATUS(ENTITY(sys)) != status_reachable)
		shade = shade_grey;
	else
		shade = shade_dark;
	if (Drawing_isSelected (interface))
		shade = shade_heavy;

	if (Drawing_from_interface(interface) == sys) {
		load = LOADIN  (ei);
	} else {
		load = LOADOUT (ei);
	}
	if (Map_is_load_shown (map) && STATUS(ei) == status_reachable && load != UNKNOWN_LOAD) {
		Draw_draw_percentage_line(shade, load, FULL_LOAD, xl, yl, xr, yr);
	} else
		Draw_draw_line(shade, xl, yl, xr, yr);
}

/*****************************************************************/
/* Draw interface */
/*****************************************************************/

static void draw_interface(s)
	Drawing          s;
{
	Boolean straight, xstraight;
	Dimension length;
	Position xl, yl, xr, yr;

	if (isHidden (s))
		  return;

	/*  If cannot bend always use a straight line,  !!! This is awful */
	if (!isBendingInterface (ENTITY(s)) && s->from != NULL && s->to != NULL) {
		choose_attachment(s->from, s->to, &xl, &yl, &xr, &yr, &straight, &xstraight, &length);
		X(s) = (xl+xr)/2;
		Y(s) = (yl+yr)/2;
	}

	if (s->from != NULL) {
		  draw_connection(s, s->from);
	}
	if (s->to != NULL) {
		  draw_connection(s, s->to);
	}

	draw_system(s);
}

/*****************************************************************/
/*  Draw a single object                                         */
/*****************************************************************/

static void redraw_drawing(drawing, moved)
	Drawing          drawing;
	Boolean          moved;
{
	Position xl, yl, xr, yr;
	Map map;

	if ( ! (map = Drawing_mapOf(drawing))) {
		sys_error ("[%s] has no map", Drawing_title (drawing));
		return;
	}
	while (map) {
		if (map->mapWidget)
			break;
		else if (map->parent) {
			Map_corners (map, &xl, &yl, &xr, &yr, False);
			Draw_set_box (X(map->parent)+1, Y(map->parent)+1,
				(Dimension) xr, (Dimension) yr,
				WIDTH(map->parent)-2, HEIGHT(map->parent)-2);
			map = Drawing_mapOf (map->parent);
		 } else
			break;
	}
	if (drawing->submap) {
		map = drawing->submap;
		Map_corners (map, &xl, &yl, &xr, &yr, False);
		Draw_set_box (X(map->parent)+1, Y(map->parent)+1,
				(Dimension) xr, (Dimension) yr,
				WIDTH(map->parent)-2, HEIGHT(map->parent)-2);
		draw_map (drawing->submap);
		Draw_reset_box (X(map->parent)+1, Y(map->parent)+1,
				(Dimension) xr, (Dimension) yr,
				WIDTH(map->parent)-2, HEIGHT(map->parent)-2);
	}

	if (isInterface(ENTITY(drawing)))
		draw_interface(drawing);
	else {
		Drawing          interface;
		Entity e;
		if (moved) {
			FOR_ENTITYLIST(e, Entity_interfaces (ENTITY(drawing))) {
				if (interface = get_drawing (e, Drawing_mapOf(drawing)))
					draw_connection(interface, drawing);
			}
		}
		draw_system(drawing);
	}

	map = Drawing_mapOf(drawing);
	while (map) {
		if (map->mapWidget)
			break;
		else if (map->parent) {
			Map_corners (map, &xl, &yl, &xr, &yr, False);
			Draw_reset_box (X(map->parent)+1, Y(map->parent)+1,
				(Dimension) xr, (Dimension) yr,
				WIDTH(map->parent)-2, HEIGHT(map->parent)-2);
			map = Drawing_mapOf (map->parent);
		 } else
			break;
	}
}

extern void Drawing_outline (drawing, insert_map, x,y)
	Drawing          drawing;
	Position x,y;
	Map insert_map;
{
	Map map;
	Position xl, yl, xr, yr;

	map = insert_map;
	while (map) {
		if (map->mapWidget)
			break;
		else if (map->parent) {
			Map_corners (map, &xl, &yl, &xr, &yr, False);
			Draw_set_box (X(map->parent)+1, Y(map->parent)+1,
				(Dimension) xr, (Dimension) yr,
				WIDTH(map->parent)-2, HEIGHT(map->parent)-2);
			map = Drawing_mapOf (map->parent);
		 } else
			break;
	}
	Draw_draw_rectangle(shade_dark, x, y, WIDTH(drawing), HEIGHT(drawing));
	map = insert_map;
	while (map) {
		if (map->mapWidget)
			break;
		else if (map->parent) {
			Map_corners (map, &xl, &yl, &xr, &yr, False);
			Draw_reset_box (X(map->parent)+1, Y(map->parent)+1,
				(Dimension) xr, (Dimension) yr,
				WIDTH(map->parent)-2, HEIGHT(map->parent)-2);
			map = Drawing_mapOf (map->parent);
		 } else
			break;
	}
}

/************************************************************************/
/*  Draw the systems on the screen                                      */
/*  !!! Very untidy routine needs to be re thought out and fixed        */
/*  !!! Is split in two parts the first could be done once, every time  */
/*  !!! an drawing attribute changes, not every time it is drawn         */
/************************************************************************/

static void draw_system(drawing)
	Drawing          drawing;
{
	Shade           border, text;
	Position        x, y;
	Dimension       width, height;

	if (isHidden (drawing))
		return;

	/************************************************************/
	/*  Set up shade/colour and geometry of drawing             */
	/************************************************************/

	border = SHADE(drawing);
	text   = shade_dark;
	width = WIDTH(drawing);
	height = HEIGHT(drawing);
	x = X(drawing);
	y = Y(drawing);

	/************************************************************/
	/*  Draw the drawings basic shape                           */
	/************************************************************/

	Shape_draw (shapeOf_entity(ENTITY(drawing)),
		Drawing_isClass (drawing),
		border, text,
		REVERSEVIDEO(drawing),
		x,y,width,height,
		BASE_WIDTH(drawing), BASE_HEIGHT(drawing),
		LABEL(drawing), LABEL2(drawing));

	/************************************************************/

	switch (EXTRA1(drawing)) {
	case extra_left:
		Draw_draw_line(border, x-2,y, x-2,y+height);
		break;
	case extra_double_border:
		Draw_draw_rectangle(border, x-2,y-2, width+4,height+4);
		Draw_draw_rectangle(border, x-4,y-4, width+8,height+8);
		break;
	case extra_border:
		Draw_draw_rectangle(border, x-2,y-2, width+4,height+4);
		break;
	case extra_leftbelow:
		Draw_draw_line(border, x-2,y, x-2,y+height);
		Draw_draw_line(border, x,y+height+2, x+width,y+height+2);
		break;
	case extra_none:
	default:
		break;
	}

	/************************************************************/

	switch (EXTRA2(drawing)) {
  	default:
	case extra2_none:
	case extra2_upFlag:
	case extra2_downFlag:
		break;
	case extra2_highlight:
		Draw_draw_line(border, x-1,y, x-1,y+height);
		Draw_draw_line(border, x,y-1, x+width,y-1);
		break;
	case extra2_lowlight:
		Draw_draw_line(border, x+width+1,y,  x+width+1,y+height);
		Draw_draw_line(border, x,y+height+1, x+width, y+height+1);
		break;
	case extra2_crosstwice:
		Draw_draw_line(border, x, y, x + width, y + height);
		Draw_draw_line(border, x, y + height, x + width, y);
		break;
	case extra2_crossonce:
		Draw_draw_line(border, x, y, x + width, y + height);
		break;
	}

	switch (EXTRAFLAG(drawing)) {
  	default:
	case flag_none:
		break;
	case flag_halfmast:
		Draw_draw_line(border, x, y, x - 4, y - 4);
		Draw_draw_line(border, x-5, y-5, x - 5, y - 5);
		break;
	case flag_up:
		Draw_draw_line(border, x, y, x, y - 4);
		Draw_draw_rectangle(border, x-10, y - 10, 10, 5);
		break;
	case flag_down:
		Draw_draw_line(border, x-4, y, x, y);
		Draw_draw_rectangle(border, x-10, y, 5, 10);
		break;
	}
	switch (TOPRIGHT(drawing)) {
  	default:
	case topright_none:
		break;
	case topright_cross:
		Draw_draw_line(border, x+ width, y-8, x + width+10, y);
		Draw_draw_line(border, x+ width, y, x + width+10, y-8);
		break;
	case topright_U:
		Draw_draw_line(border, x+ width, y-10, x + width, y);
		Draw_draw_line(border, x+ width+10, y-10, x + width, y+10);
		Draw_draw_line(border, x+ width, y, x + width+10, y);
		break;
	case topright_C:
		Draw_draw_line(border, x+ width, y-10, x + width, y);
		Draw_draw_line(border, x+ width, y-10, x + width+10, y-10);
		Draw_draw_line(border, x+ width, y, x + width+10, y);
		break;
	case topright_T:
		Draw_draw_line(border, x+ width, y-10, x + width+10, y - 10);
		Draw_draw_line(border, x+ width+5, y - 8, x + width+5, y);
		break;
	case topright_upFlag:
		Draw_draw_line(border, x+ width, y, x + width, y - 4);
		Draw_draw_rectangle(border, x+ width, y - 10, 10, 5);
		break;
	case topright_downFlag:
		Draw_draw_line(border, x+ width, y, x + width+4, y);
		Draw_draw_rectangle(border, x+ width+5, y, 5, 10);
		break;
	case topright_P:
		Draw_draw_line(border, x+ width + 1, y - 1, x + width + 1, y - 3);
		Draw_draw_rectangle(border, x+ width + 1, y - 10, 8, 5);
		break;
	}
}

/*************************************************************************/
/*  Draw map as a list  */
/*************************************************************************/

static void draw_list_drawing (o)
	Drawing o;
{
	Position x,y, nx,ny;
	Dimension w,h;

	x = LISTBOX(o).x;
	y = LISTBOX(o).y;
	w = LISTBOX(o).width;
	h = LISTBOX(o).height;
	nx = 2;
	ny = 2;

	if (Drawing_isSelected (o)) {
		Draw_draw_box(shade_dark, x, y, w,h, nx, ny);
	}
	Draw_draw_string(shade_dark, x + nx, y + ny + height_of_string (NULL), LISTBOX(o).label);
}

/*************************************************************************/
/* Class map */
/*************************************************************************/

#define CLASS_X_SPACING 16
#define CLASS_Y_SPACING 5

static Position draw_class(map, class, x,y)
	Entity          class;
	Position        x, y;
	Map map;
{
	Drawing		draw;
	Entity		ce;
	Position        x1, x2, yc;

	if (class == NULL) {
		sys_error ("draw_class class is NULL");
		return y;
	}
	draw = Map_insert_drawing_class (map, class, x,y);
	if (!draw) {
		sys_error ("could not get drawing for class [%s]", Entity_title (class));
		return y;
	}
	X(draw) = x;
	Y(draw) = y;
	draw_system (draw);

	yc = Y(draw) + HEIGHT(draw) / 2;
	if (class != TopClass)
		Draw_draw_line(shade_dark, X(draw) - 1, yc, X(draw) - CLASS_X_SPACING / 2 + 1, yc);
	if (! isEmpty_entityList(Entity_subclasses(class))) {
		x1 = X(draw) + WIDTH(draw);
		x2 = x1 + CLASS_X_SPACING / 2;
		Draw_draw_line(shade_dark, x1, yc, x2, yc);
		FOR_ENTITYLIST(ce, Entity_subclasses(class)) {
			Position oldyc;
			y = draw_class(map, ce, x1 + CLASS_X_SPACING, y);
			oldyc = yc;
			yc = y + HEIGHT(ce->drawing) / 2;
			Draw_draw_line(shade_dark, x2, oldyc, x2, yc);
			y += HEIGHT(ce->drawing) + CLASS_Y_SPACING;
		}
		y -= HEIGHT(draw) + CLASS_Y_SPACING;	/* !!! Very bad style */
	}
	return (y);
}

/************************************************************************/
/* Actually draw the map                                                */
/* Draws a line between two systems. May later allow different line     */
/* styles. Very non-optimal method employed, need not do so much        */
/* calcutlation each time. Attempts to be clever with lines and draw a  */
/* straight one when it can                                             */
/* !!! May later also try 45 degree lines if nothing better available   */
/************************************************************************/

extern void Map_draw(map)
	Map          map;
{
	Drawing          s;

	switch (map->drawnas) {
	case drawnas_list:
		if (!Map_classesOnly(map)) {
			Dimension w, h;
			char heading[1024]; /* !!! */
			(void) sprintf (heading, "%s | Title", pretty_status_report_heading ());
			w = width_of_string (heading);
			h = height_of_string (heading);
			Draw_draw_box(shade_dark, 19, 12, w+6,h+6, 3, 3);
			Draw_draw_string (shade_dark, 22, 15 + h, heading);
		}
		FOR_DRAWINGLIST(s, Map_elements (map))
			draw_list_drawing (s);
		break;
	default:
	case drawnas_map:
		if (Map_classesOnly(map)) {
			(void) draw_class(map, TopClass, 10, 10);
		} else {
			draw_map (map);
		}
		break;
	}
	Draw_end_draw ();
}

/******************************************************************/
/* Temporarily Switch off drawing
/******************************************************************/

static Boolean NoDrawing = False;
extern void temporary_noDrawing ()
{ NoDrawing = True; }
extern void allow_drawing ()
{ NoDrawing = False; }
extern Boolean isDrawingAllowed ()
{ return NoDrawing == False; }


/***************************************************************/
/*  Redraw parts of map                                        */
/*  Since we use XOR for drawing, undraw and redraw are        */
/*  nearly equivalent.                                         */
/***************************************************************/

extern void Drawing_draw(s, map, moved)
	Drawing s;
	Map map;
	Boolean moved;
{

	switch (map->drawnas) {
	case drawnas_list:
		draw_list_drawing (s);
		break;
	case drawnas_map:
		redraw_drawing(s, moved);
		break;
	}
	Draw_end_draw ();
}

/******************************************************************/
/*  Draw a text map  */
/******************************************************************/

extern void Map_draw_text (map)
	Map map;
{
	if (map == NULL) {
		sys_error ("Map Area is NULL");
		return;
	}
	if (map == NULL) {
		user_error ("Map area has no map");
		return;
	}

	if (start_draw_text (Map_title (map)))
		Map_draw (map);
}

/******************************************************************/
/*  Draw a postscript Map  */
/******************************************************************/

extern String Map_draw_postscript (map, prologue)
	Map map;
	String prologue;
{
	String title;
	Position xl,xr, yl,yr;
	static char  filename[1024]; /* !!! */

	if (map == NULL) {
		sys_error ("Map Area is NULL");
		return NULL;
	}
	if (map == NULL) {
		user_error ("Map area has no map");
		return NULL;
	}
	title = Map_title (map);

	if (Map_corners (map, &xl,&yl, &xr,&yr, True) == False) {
		user_warning ("Map [%s] is empty", title);
		return NULL;
	}
	(void) sprintf (filename, "%s.ps", title);
	if (start_draw_postscript (prologue, title, filename, xl,xr, yl, yr))
		Map_draw (map);
	return filename;
}

/************************************************************************/
/*  Init and Finish  */
/************************************************************************/

extern void init_drawMaps ()
{
}

extern void finish_drawMaps ()
{
}

/******************************************************************/
/******************************************************************/
/*  $Header: //gaia/group.local/computer/monet/lib/map/RCS/draw.c,v 1.1 1991/07/22 11:37:05 a38 Exp a38 $  */
/******************************************************************/
/******************************************************************/

