/****************************************************************/
/****************************************************************/
/*  $Header: //gaia/group.local/computer/monet/lib/map/RCS/interface.c,v 1.1 1991/06/11 11:26:10 a38 Exp a38 $ */
/****************************************************************/
/*  These routines are used for creating and positioning        */
/*  interfaces on the screen.  This is a layer above the        */
/*  interface module provided by _db/interface.c                */
/****************************************************************/

#include "map/Drawing.h"
#include "map/Layout.h"
#include "map/Interface.h"

/****************************************************************/
/*  Definitions  */
/****************************************************************/


/***************************************************************/
/*  Work out the position of interfaces on the map */
/***************************************************************/

extern void get_newInterfacePosition (map, interface, x, y)
	Drawing interface;
	Map map;
	Position *x, *y;
{
	Drawing from , to;

	from = Drawing_from_interface (interface);
	to   = Drawing_to_interface   (interface);

	if (from == NULL || to == NULL) {
		/* !!! There is a big fuck up with loading interfaces must use value no > than 0 */
		*x = 0;
		*y = 0;
	} else {
		choose_midpoint_attachment (interface, from, to, x, y);
	}
}

/***************************************************************/
/*  Create an interface between two systems, create a unique */
/*  name for it */
/***************************************************************/

extern Drawing make_anonymousInterface (class, from, to)
	Drawing from, to;
	Entity class;
{
	Drawing new;

	static long int Linkid = 0;
	char * title;
	Position x,y;

	PROC("anonymous_interface");  

	if (Drawing_mapOf(from) != Drawing_mapOf(to)) {
		user_error ("Cannot create an interface between objects on different maps");
		return NULL;
	}
	if (!can_interface (class, Drawing_classOf(from)) || !can_interface (class, Drawing_classOf(to))) {
		user_error ("Can't create an interface between these two objects");
		return NULL;
	}
	/* !!!!!! This is not the way to do it */
	title = (char *) sprintf (MonetMalloc (26), "link%ld.%d.%d", Linkid, (int) from, (int) to);
	Linkid ++;
	choose_midpoint_position (from, to, &x, &y);
	if ((new = Drawing_create (title, class, Drawing_mapOf(from), x,y, 0,0)) == NULL) {
		sys_error ("Anonymous interface already exists");
		return NULL;
	}
	make_interface (ENTITY(new), ENTITY(from));
	make_interface (ENTITY(new), ENTITY(to));
	return new;
}

/***************************************************************/
/* Position the interfaces of a system in a nice way */
/* There are more general solutions to this problem */
/* Hopefully this one is good enough for the time being */
/***************************************************************/

static Boolean is_acute (i)
	Drawing i;
{
	int xf,yf, xt,yt;
	int above;
	Drawing from, to;

	from = Drawing_from_interface (i);
	to   = Drawing_to_interface (i);
	if (from == NULL || to == NULL)
		return False;

	yf = Y(from) - Y(i);
	xf = X(from) - X(i);
	yt = Y(to)   - Y(i);
	xt = X(to)   - X(i);

/*	below = ( sqrt (xf*xf+xt*xt) * sqrt (yf*yf+yt*yt)) >= 0; */
	above = (xf*xt + yf*yt) >= 0;
	return above;
}

extern void straighten_acute_interfaces (drawing)
	Drawing drawing;
{
	Drawing i;
	Entity ei, sys = ENTITY(drawing);

	if (isSystem (sys))
		FOR_ENTITYLIST (ei, sys->interfaces)
			if (i = get_drawing (ei, Drawing_mapOf(drawing)))
				if (is_acute (i))
					place_interface (Drawing_mapOf(drawing), i);
	else if (isInterface (sys))
		if (is_acute (drawing))
			place_interface (NULL, drawing);
}

extern Boolean position_interfaces (drawing)
	Drawing drawing;
{
	Drawing i;
	Entity ei, sys = ENTITY(drawing);

	if (isSystem (sys))
		FOR_ENTITYLIST (ei, sys->interfaces)
			if (i = get_drawing (ei, Drawing_mapOf(drawing)))
				place_interface (Drawing_mapOf(drawing), i);
	else if (isInterface (sys))
		place_interface (NULL, drawing);
	else
		return False;
	return True;
}

extern void place_interface (map, interface)
	Drawing interface;
	Map map;
{
	Position x,y;

	if (map == NULL)
		map = Drawing_mapOf(interface);
	get_newInterfacePosition  (map, interface, &x, &y);
	Map_insert_drawing (map, interface, x,y, 0,0);
}

/****************************************************************/
/****************************************************************/
/*  $Header: //gaia/group.local/computer/monet/lib/map/RCS/interface.c,v 1.1 1991/06/11 11:26:10 a38 Exp a38 $ */
/****************************************************************/
/****************************************************************/


