/*****************************************************************************************/
/*****************************************************************************************/
/* $Header$ */
/*****************************************************************************************/
/*  Does pinging of IP hosts.  Would like it to be clever and ping         */
/*  the addresses to infer the hosts status.                                             */

/*  Much of this stuff taken from the code for ping.                                     */
/*  !!! Originally a local program 'rping' was used for doing one shot pinging.          */
/*  !!! this was developed form the original code for 'ping'.                            */
/*  !!! As yet untested but included is the code from rping so that a 'system' call      */
/*  !!! is not needed, some changes have been made to suit it better to this application */
/*  !!! Once this is working, will try to put the 'select statement elsewhere so that    */
/*  !!! other connections can be serviced, the select statement is in 'server.c'.        */
/*  !!! This is very powerfull because many pings could be done at once.                 */
/*  !!! Need to split into two parts a request sender and a handler.                     */
/*  !!! No need to set timeouts short in this program.                                   */
/*  !!! Must give an id to each request so that they are not confused.                   */
/*****************************************************************************************/

#include "general/General.h"
#include "general/Gather.h"
#include "ip.h"

#include <signal.h>
#include <sys/errno.h>
#include <sys/wait.h>

#include <errno.h>
#include <sys/param.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <ctype.h>

extern int errno;

#ifndef SGI
extern char *inet_ntoa();
#endif

/*
**  Parts copied from public domain stuff with header below
**
**     Written by Steven Grimm (sgrimm@sun.com) on 11-26-87
**     Please distribute widely, but leave my name here.
**
*/

#ifndef FD_SETSIZE	/* for 4.2BSD */
#define FD_SETSIZE	(sizeof(fd_set) * 8)
#define NFDBITS		32		/* # of bits in fds_bits[0] */
#define FD_SET(n, p)	((p)->fds_bits[(n)/NFDBITS] |= (1 << ((n) % NFDBITS)))
#define FD_CLR(n, p)	((p)->fds_bits[(n)/NFDBITS] &= ~(1 << ((n) % NFDBITS)))
#define FD_ISSET(n, p)	((p)->fds_bits[(n)/NFDBITS] & (1 << ((n) % NFDBITS)))
#define FD_ZERO(p)	bzero((char *)(p), sizeof(*(p)))
#endif

/*****************************************************************************************/

/*****************************************************************************************/

#define	MAXPACKET	4096			/* max packet size */
#define PACKETSIZE	32			/* packet total size */
#define HDRLEN		8			/* ICMP header length */
#define DATALEN		PACKETSIZE-HDRLEN	/* data size */

/*****************************************************************************************/
/*  Globals  */
/*****************************************************************************************/

static u_char packet[MAXPACKET];
static struct icmp *ICP = (struct icmp *) packet;
static Boolean Pinging = True, CanPing = False;
static int MyPID;
static int IcmpSocket;		/* Socket file descriptor */
static struct sockaddr icmp_to;	/* Who to ping */
static int uniqizer = 0;	/* Used in "seq" field to distinguish
				 * current polls from old ones		*/

/*****************************************************************************************/
/*  IP Queries  */
/*****************************************************************************************/

static QueryClass QP;

typedef struct ip_query {
	char * host;
} * IPQuery;
#define IP(m)  ((IPQuery)  SUBCLASSVALUE(m))

/*****************************************************************************************/
/*  The real pinger stuff  */
/*****************************************************************************************/

static int wait_sock();
static int check_packet();
static u_short checksum();
static char *pr_type();

static int verbose = 0;			/* verbose output */
static unsigned timeout = 2;			/* timeout in secs */
static long timelimit;				/* safety limit */

#define EX_ALIVE 0			/* host is alive */
#define EX_DEAD  1			/* host is dead */
#define EX_USAGE 2			/* illegal parameter */
#define EX_ERROR 3			/* system error */

#ifdef NOT_USING_THIS_JUST_NOW_ABCD
static int ping (host)
	String host;
{
	register int s;
	register struct hostent *hp;
	static struct protoent *proto = 0;
	struct sockaddr toaddr, fromaddr;
	struct sockaddr_in *to   = (struct sockaddr_in *) &toaddr;
	struct sockaddr_in *from = (struct sockaddr_in *) &fromaddr;
	int addrlen = sizeof(struct sockaddr);
	int status;

	static u_char packet[MAXPACKET];
	register struct timeval *tp = (struct timeval *) &packet[HDRLEN];
	register u_char *dp = &packet[HDRLEN+sizeof(struct timeval)];

	int cc = PACKETSIZE;
	int nc;
	int sequence = 0;
	int ident = getpid() & 0xFFFF;
	to->sin_family = AF_INET;

	/* Determine remote host address. */
	if (host == NULL)
		return 1;
	if ((hp = gethostbyname(host)) != NULL)	{
		host = hp->h_name;
		bcopy(hp->h_addr, (char *)&to->sin_addr, hp->h_length);
		to->sin_family = hp->h_addrtype;
	} else if ((to->sin_addr.s_addr = inet_addr(host)) == (u_long)-1) {
		sys_error("Unknown host: %s", host);
		return 1;
	}

	/* Prepare output packet. Store current time in data field after packet header. */
	tp = (struct timeval *) &packet[HDRLEN];
	(void) gettimeofday(tp, (struct timezone *)0);

	dp = &packet[HDRLEN+sizeof(struct timeval)];
	for (nc = sizeof(struct timeval); nc < DATALEN; nc++)
		*dp++ = nc;

	ICP = (struct icmp *) packet;
/*	ICP->icmp_type  = ICMP_ECHO;
	ICP->icmp_code  = 0;
*/
	ICP->icmp_cksum = 0;
	ICP->icmp_id    = ident;
	ICP->icmp_seq   = sequence;
	ICP->icmp_cksum = checksum((u_short *)ICP, cc);

	/* Send probe packet to remote destination */
	if ((nc = sendto(IcmpSocket, packet, cc, 0, &toaddr, sizeof(toaddr))) < 0) {
		perror("sendto");
		(void) close(s);
		return (EX_ERROR);
	}

	/* Wait until input packet arrives. Retry after timeout */
	do {
		if ((status = wait_sock(IcmpSocket, timeout)) < 0)
		{
			perror("select");
			(void) close(s);
			return (EX_ERROR);
		} else if (status == 0) {
			errno = ETIMEDOUT;
			if (verbose)
				perror("recvfrom");
			(void) close(IcmpSocket);
			return (EX_DEAD);
		}

	/*
	 * Fetch available input packet.
	 * Note. On Apollo, recvfrom() will not be interrupted by an alarm().
	 */
		addrlen = sizeof(fromaddr);
		if ((nc = recvfrom(IcmpSocket, packet, sizeof(packet), 0, &fromaddr, &addrlen)) < 0) {
			if (errno == EINTR)
				errno = ETIMEDOUT;
			perror("recvfrom");
			(void) close(IcmpSocket);
			return (EX_ERROR);
		}
		/* Analyze input packet. Reread if this is not what we want */
		status = check_packet(packet, nc, cc, ident, sequence, from);
	} while (status != EX_ALIVE && (time(0) <= timelimit));
	
	(void) close(IcmpSocket);
	return (status);
}

static int wait_sock(sock, secs)
	int sock;
	int secs;
{
	fd_set fds;
	struct timeval wait;
	int nreply;

	wait.tv_sec = secs;
	wait.tv_usec = 0;
	FD_ZERO(&fds);
	FD_SET(sock, &fds);

	nreply = select(FD_SETSIZE, &fds, (fd_set *)0, (fd_set *)0, &wait);
	return (nreply);
}

static int check_packet(buf, nc, cc, ident, sequence, from)
	u_char *buf;
	int nc;
	int cc;
	int ident;
	int sequence;
	struct sockaddr_in *from;
{
	register struct ip *ip;
	register struct icmp *icp;
	struct timeval *sendtime;
	struct timeval recvtime;
	int hdrlen, triptime;
	char *host;

	(void) gettimeofday(&recvtime, (struct timezone *)0);

	from->sin_addr.s_addr = ntohl(from->sin_addr.s_addr);
	host = inet_ntoa(from->sin_addr);

	ip = (struct ip *) buf;
	hdrlen = ip->ip_hl << 2;
	if (nc < hdrlen + ICMP_MINLEN)
	{
		if (verbose)
			printf("%d bytes from %s (packet too short)\n", nc, host);
		return (EX_DEAD);
	}
	nc -= hdrlen;
	icp = (struct icmp *)(buf + hdrlen);

	if (verbose)
		printf("%d bytes from %s ", nc, host);

	if (icp->icmp_type != ICMP_ECHOREPLY)
	{
		if (verbose)
			printf("(not echo reply: %s)\n", pr_type(icp->icmp_type, icp->icmp_code));
		return (EX_DEAD);
	}

	if (icp->icmp_id != ident)
		/* ("(not from us: ident %d should be %d)\n", icp->icmp_id, ident); */
		return (EX_DEAD);

	if (icp->icmp_seq != sequence)
	{
		if (verbose)
			printf("(out of sequence: seq %d should be %d)\n", icp->icmp_seq, sequence);
		return (EX_DEAD);
	}

	if (nc != cc)
	{
		if (verbose)
			printf("(packet size error: size %d should be %d)\n", nc, cc);
		return (EX_DEAD);
	}

	sendtime = (struct timeval *)&icp->icmp_data[0];
	if ((recvtime.tv_usec -= sendtime->tv_usec) < 0)
		recvtime.tv_sec--, recvtime.tv_usec += 1000000;
	recvtime.tv_sec -= sendtime->tv_sec;
	triptime = (recvtime.tv_sec*1000) + (recvtime.tv_usec/1000);
	if (verbose)
		printf("seq=%d time=%d ms\n", sequence, triptime);
	return (EX_ALIVE);
}

static char * pr_type(type, code)
int type;
int code;
{
	static char *ctab[] =
	{
	/*  0 */	"network is unreachable",
	/*  1 */	"host is unreachable",
	/*  2 */	"destination unreachable, protocol",
	/*  3 */	"destination unreachable, port",
	/*  4 */	"destination unreachable, needfrag",
	/*  5 */	"destination unreachable, srcfail"
	};

	static char *ttab[] =
	{
	/*  0 */	"echo reply",
	/*  1 */	"ICMP 1",
	/*  2 */	"ICMP 2",
	/*  3 */	"destination unreachable",
	/*  4 */	"source quench",
	/*  5 */	"redirect",
	/*  6 */	"ICMP 6",
	/*  7 */	"ICMP 7",
	/*  8 */	"echo request",
	/*  9 */	"ICMP 9",
	/* 10 */	"ICMP 10",
	/* 11 */	"time exceeded",
	/* 12 */	"parameter problem",
	/* 13 */	"timestamp request",
	/* 14 */	"timestamp reply",
	/* 15 */	"info request",
	/* 16 */	"info reply",
	/* 17 */	"mask request",
	/* 18 */	"mask reply"
	};

	if (type < 0 || type > 18 )
		return ("unknown packet type");

	if (type == ICMP_UNREACH)
		if (code >= 0 && code <= 5 )
			return (ctab[code]);

	return (ttab[type]);
}

static u_short checksum(addr, len)
u_short *addr;
int len;
{
	register int nleft = len;
	register u_short *w = addr;
	register int sum = 0;
	u_short answer = 0;

	/*
	 *  Our algorithm is simple, using a 32 bit accumulator (sum),
	 *  we add sequential 16 bit words to it, and at the end, fold
	 *  back all the carry bits from the top 16 bits into the lower
	 *  16 bits.
	 */
	while (nleft > 1)
	{
		sum += *w++;
		nleft -= 2;
	}

	/*
	 * mop up an odd byte, if necessary
	 */
	if (nleft == 1)
	{
		*(u_char *)(&answer) = *(u_char *)w;
		sum += answer;
	}

	/*
	 * add back carry outs from top 16 bits to low 16 bits
	 */
	sum = (sum >> 16) + (sum & 0xffff);	/* add hi 16 to low 16 */
	sum += (sum >> 16);			/* add carry */
	answer = ~sum;				/* truncate to 16 bits */
	return (answer);
}

#endif /*  NOT_USING_THIS_JUST_NOW_ABCD */

/*****************************************************************************************************/
/* Ping */
/*****************************************************************************************************/

static Boolean request (query)
	Query query;
{
	Entity mob = query->entity;
	VStatus status = V_STATUS(mob);
	/* !!! Should use ICMP Ping */

	String host = IP(query)->host;
	int	sock;
	struct	sockaddr_in server;
	struct	hostent *hp;
	Boolean result;
	int Error;
	char buffer [1024]; /* !!! */
	int ipstatus;
	Cardinal addresses;

	if (isdigit (host[0]))
		/*  If it is a number, should ping the address, not the host  !!! */
		hp = gethostbyaddr (host, MonetStrlen (host), AF_INET);
	else
		hp = gethostbyname (host);
	if (hp == NULL) {
		switch (h_errno) {
		case HOST_NOT_FOUND:
			debug_message ("[%s] unknown", host);
			SetReachStatusMask (status, status_invalid);
			break;
		case NO_DATA:
			debug_message ("[%s] known but no A Record", host);
			SetReachStatusMask (status, status_unknown);
			break;
		case NO_RECOVERY:
			debug_message ("Server failed on [%s]", host);
			SetReachStatusMask (status, status_unknown);
			break;
		case TRY_AGAIN:
			debug_message ("Temporary server failure on [%s]", host);
			SetReachStatusMask (status, status_unknown);
			break;
		default:
			sys_error ("Unknown h_errno [%d]", h_errno);
			SetReachStatusMask (status, status_invalid);
			break;
		}
		return True;
	}
	/*  Set official name */
	SetTitleStatusMask (status, hp->h_name);
	/*  Count the number of ip addresses this host has.  This is used  */
	/*  as the statisttic for the number of connections/interfaces.    */
	for (addresses = 0; hp->h_addr_list[addresses] != NULL; addresses++)
		;
	SetConnectionsStatusMask (status, addresses);

#ifdef NOT_USING_THIS_JUST_NOW_ABCD
	if (CanPing) {
		timelimit = time(0) + timeout;
		ipstatus = ping (host);
	} else
#endif /*  NOT_USING_THIS_JUST_NOW_ABCD */
	{
		(void) sprintf (buffer, "%s %s", Config_rping, host);
		ipstatus = system (buffer);
	}
	if (ipstatus == 0) {
		SetReachStatusMask (status, status_reachable);
	} else if (ipstatus == 127) {
		sys_error ("Could not execute [%s], do you have access to it?", Config_rping);
		Pinging = False;
		SetReachStatusMask (status, status_unknown);
	} else {
		SetReachStatusMask (status, status_unreachable);
	}
	return True;
}

/********************************************************************/
/*  Free  */
/********************************************************************/

static void free (query)
	Query query;
{
	MonetFree (IP(query)->host);
	MonetFree (IP(query));
}                           

/********************************************************************/
/*  Find  */
/********************************************************************/

static  Query find (host)
	String host;
{
	Query o;

	FORINSTANCES (o, QP)
		if (EQS(IP(o)->host, host))
			return o;

	return NULL;
}

/********************************************************************/
/*  Create  */
/********************************************************************/

static  SubValue create (query, host)
	Query query;
	String host;
{
	IPQuery new;
	new =  MonetNew (struct ip_query);
	new -> host = MonetNewString (host);
	return (SubValue) new;
}

/********************************************************************/
/*  Convert to text */
/********************************************************************/

static char * as_text (mob)
	Query mob;
{
	static char text [128]; /* !!! */
	(void) sprintf (text, "ip:%s", IP(mob)->host);
	return text;
}

/*****************************************************************************************/
/* Init and Finish */
/*****************************************************************************************/

static void finish ()
{
	if (IcmpSocket)
		close (IcmpSocket);
}

extern QueryClass init_ip ()
{
	struct protoent *proto;
	extern int sys_nerr;
	extern char *sys_errlist[];

	CanPing = False;
	MyPID = getpid() & 0xFFFF;
	ICP->icmp_type = ICMP_ECHO;
	ICP->icmp_code = 0;
	ICP->icmp_id = MyPID;	/* ID */
	if ((proto = getprotobyname("icmp")) == NULL) {
		sys_error ("icmp: unknown protocol");
		CanPing = False;
	} else 	if ((IcmpSocket = socket(AF_INET, SOCK_RAW, proto->p_proto)) < 0) {
		if (errno < sys_nerr)
			sys_error (sys_errlist[errno]);
		else
			sys_error ("Unknown error from socket call.");
		CanPing = False;
	} else {
		CanPing = True;
	}
	NEW_QUERYCLASS (QP, "ip");
	return QP;
}

/*****************************************************************************************/
/*****************************************************************************************/
/* $Header $ */
/*****************************************************************************************/
/*****************************************************************************************/

