/*****************************************************************************************/
/*****************************************************************************************/
/* $Header$ */
/*****************************************************************************************/
/*  This doesn't really make that much sense as yet because an snmp request will doesn't */
/*  return an up/down status, unless of course it is an interface.  Will deal with all   */
/*  this later.  */
/*****************************************************************************************/

#include "general/General.h"
#include <sys/types.h>
#include <netinet/in.h>
#include "query/SnmpLib.h"
#include "snmp.h"

/*****************************************************************************************/
/*  Globals  */
/*****************************************************************************************/

typedef struct snmp_interface_query {
	char *		host;
	char *		community;
	Cardinal	interface;
} * SnmpInterfaceQuery;
#define SNMP(m)  ((SnmpInterfaceQuery)  SUBCLASSVALUE(m))
static QueryClass QP;

/*****************************************************************************************/
/*  Ping an snmp object  */
/*****************************************************************************************/

#ifdef CMU_SNMP

#define BASEDIFF(a,b)  ((b>=a) ? 0 : a-b)

static Boolean request (query)
	Query query;
{
	Entity mob = query->entity;
	VStatus status = V_STATUS(mob);
	int interface = SNMP(query)->interface;
	String community = SNMP(query)->community;
	String host      = SNMP(query)->host;
	struct snmp_session session, *ss;
	struct snmp_pdu *pdu, *response;
	struct variable_list *vars;
	int	count;
	unsigned long lastchange, uptime = 0;
	Capacity trafficin, trafficout;
	Capacity capacity;
	unsigned long now;

	if (!QP->enabled) {
		SetReachStatusMask (status, status_unknown);
		return 0;
	}
	if (interface <1) {
		SetReachStatusMask (status, status_invalid);
		return 0;
	}

	bzero((char *)&session, sizeof(struct snmp_session));
	session.peername = host;
	session.community = (u_char *)community;
	session.community_len = MonetStrlen(community);
	session.retries = SNMP_DEFAULT_RETRIES;
	session.timeout = SNMP_DEFAULT_TIMEOUT;
	session.authenticator = NULL;
	snmp_synch_setup(&session);
	if ((ss = snmp_open(&session)) == NULL) {;
		user_error ("Couldn't start snmp");
		SetReachStatusMask (status, status_unknown);
		return 0;
	}
	pdu = snmp_pdu_create(GET_REQ_MSG);

	set_IfOid (ifInOctets, interface);
	set_IfOid (ifOutOctets, interface);
	set_IfOid (ifSpeed, interface);
	set_IfOid (ifOperStatus, interface);
	set_IfOid (ifLastChange, interface);

	add_oid2pdu(pdu, sysUpTime);
	add_oid2pdu(pdu, ifNumber);
	add_oid2pdu(pdu, ifOperStatus);
	add_oid2pdu(pdu, ifInOctets);
	add_oid2pdu(pdu, ifOutOctets);
	add_oid2pdu(pdu, ifSpeed);
	add_oid2pdu(pdu, ifLastChange);

	SetReachStatusMask (status, status_unknown);
	for (;;) {
		int snmpstatus;
		snmpstatus = snmp_synch_response(ss, pdu, &response);
		switch (snmpstatus) {
		case STAT_TIMEOUT:
			debug_message ("No Response from [%s]", host);
			SetReachStatusMask (status, status_unreachable);
			goto out;
		case STAT_ERROR:
		default:
			debug_message ("An error occurred");
			SetReachStatusMask (status, status_unknown);
			goto out;
		case STAT_SUCCESS:
			if (response->errstat == SNMP_ERR_NOERROR) {
				for(vars = response->variables; vars; vars = vars->next_variable) {
				IFOID(vars, sysUpTime)
					uptime = *vars->val.integer / 100;
				ELSIFOID(vars, ifLastChange)
					lastchange = *vars->val.integer / 100;
					/* Hmm?  */
				ELSIFOID(vars, ifNumber)
					if (*vars->val.integer < interface)
						SetReachStatusMask (status, status_invalid);
				ELSIFOID(vars, ifOperStatus)
					switch (*vars->val.integer) {
					case 1 : /* Up */
						SetReachStatusMask (status, status_reachable);
						break;
					case 2 : /* Down */
						SetReachStatusMask (status, status_unreachable);
						break;
					case 3 : /* Testing */
						/* !!! Should have something else here */
						SetReachStatusMask (status, status_unreachable);
						break;
					default :
						user_error ("Unknown snmp response value [%d]", *vars->val.integer);
 						SetReachStatusMask (status, status_unknown);
						break;
	                              	}
				ELSIFOID (vars, ifInOctets)
          				/*  Traffic in bytes  */
					trafficin = (Capacity)*vars->val.integer;
				ELSIFOID (vars, ifOutOctets)
					trafficout = (Capacity)*vars->val.integer;
				ELSIFOID (vars, ifSpeed)
          				/*  Speed in Bits / second  */
					capacity = (Capacity)*vars->val.integer;
				ELSEOID
					debug_message ("Unknown snmp variable returned");
				ENDOID
				}
				goto out;
			} else {
				debug_message ("Error in packet. Reason: %s", snmp_errstring(response->errstat));
				if (response->errstat == SNMP_ERR_NOSUCHNAME) {
					printf("This name doesn't exist: ");
					for(count = 1, vars = response->variables; vars && count != response->errindex;
						vars = vars->next_variable, count++);
					if (vars)
						print_objid(vars->name, vars->name_length);
					printf("\n");
					SetReachStatusMask (status, status_invalid);
					goto out;
				}
				if ((pdu = snmp_fix_pdu(response, GET_REQ_MSG)) == NULL) {
					SetReachStatusMask (status, status_unknown);
					goto out;
				}
			}
			break;
		}
	}
	out:
	if (response)
		snmp_free_pdu(response);
	snmp_close(ss);

	now = time(NULL);

	/*  Work out the load in and out */
	/*  Yes I am quite aware that the following is bad style   !!! */
	/*  but this lot needs changing anyway and besides I need  !!! */
	/*  a better load algorithm, one that gives recent load.   !!! */
	debug_message ("Traffic in  %u", trafficin);
	debug_message ("Traffic out %u", trafficout);
	debug_message ("Capacity %u", capacity);
	debug_message ("Uptime %u", uptime);
	if (capacity == 0 || uptime == 0) {
		trafficin  = 0;
		trafficout = 0;
	} else {
		trafficin /= capacity;
		trafficin *= 8 * FULL_LOAD;
		trafficin /= uptime;
		trafficout /= capacity;
		trafficout *= 8 * FULL_LOAD;
		trafficout /= uptime;
	}

	debug_message ("Load in   %u/%u", trafficin, FULL_LOAD);
	debug_message ("Load out  %u/%u", trafficin, FULL_LOAD);
	SetLoadInStatusMask  (status, trafficin);
	SetLoadOutStatusMask (status, trafficout);
	SetCapacityStatusMask (status, capacity);
	SetConnectionsStatusMask (status, 2);

	status->time_stamp = now;

	return 0;
}


#else /* CMU_SNMP */
static Boolean request (mob)
	Query mob;
{
	return 0;
}
#endif /* CMU_SNMP */

/*****************************************************************************************/
/* Init and Finish */
/*****************************************************************************************/

static void free (mob)
	Query mob;
{
	MonetFree (SNMP(mob)->host);
	MonetFree (SNMP(mob)->community);
	MonetFree (SNMP(mob));
}                           

/********************************************************************/
/*  Find  */
/********************************************************************/

static  Query find (text)
	String text;
{
	int cc;
	Query o;
	char community[512], host[512];
	int interface;

	cc = sscanf (text, "%[^ \t\n+]+%[^ \t\n]", community, host, &interface);
	if (cc != 2)
		return NULL;

	FORINSTANCES (o, QP)
		if (SNMP(o)->interface == interface && EQS(SNMP(o)->host, host))
			return o;

	return NULL;
}

/********************************************************************/
/*  Create  */
/********************************************************************/

static  SubValue create (query, text)
	String text;
	Query query;
{
	int cc;
	char community[512], host[512];
	int interface;

	cc = sscanf (text, "%[^ \t\n+]+%[^ \t\n+]+%u", community, host, &interface);
	if (cc == 3) {
		SnmpInterfaceQuery new;
		new = MonetNew (struct snmp_interface_query);
		new -> interface = interface;
		new -> host      = MonetNewString (host);
		new -> community = MonetNewString (community);
		return (SubValue) new;
	} else {
		user_error ("Bad request [%s]", text);
		return NULL;
	}
	/* NOTREACHED */
}

static char * as_text (mob)
	Query mob;
{
	static char text [1024]; /* !!! */
	(void) sprintf (text, "snmp_interface:%s+%s+%u",
		SNMP(mob)->community,
		SNMP(mob)->host,
		SNMP(mob)->interface);
	return text;
}

/*****************************************************************************************/

/*****************************************************************************************/

static void finish ()
{
}

extern QueryClass init_snmp_interface ()
{
	PROC ("init_snmp_interface");

	NEW_QUERYCLASS (QP, "snmp_interface");
	if (snmplib_enabled ())
		QP->enabled  = True;
	else
		QP->enabled  = False;

	return QP;
}

/*****************************************************************************************/
/*****************************************************************************************/
/* $Header$ */
/*****************************************************************************************/
/*****************************************************************************************/

