
/*****************************************************************************/
/*****************************************************************************/
/* $Header$ */
/*****************************************************************************/
/*  Useful Callback and Actions for use with X11                             */
/*****************************************************************************/

#include "general/General.h"
#include "xmonet/Widget.h"

/****************************************************************************/
/*  Take the name of a widget, find it, if it is a dialog or a text widget  */
/*  return its value, NULL is empty or error  */
/****************************************************************************/

extern String WidgetName2value (w_name)
	String w_name;
{
	Widget text_widget;
	String name;
	char  clean_name[1024]; /* !!! */

	if (w_name == NULL) {
		user_error ("text_widget name is NULL");
		return NULL;
	}
	WcCleanName (w_name, clean_name);
	text_widget = WcFullNameToWidget (toplevel, clean_name);
	if ( ! text_widget) {
		user_error ("No such text_widget [%s]", w_name);
		return NULL;
	}
	if (XtIsSubclass (text_widget, dialogWidgetClass)) {
		name = XawDialogGetValueString (text_widget);
	} else if (XtIsSubclass (text_widget, asciiTextWidgetClass)) {
		Arg args[1];
		XtSetArg(args[0], XtNstring, &name);
		XtGetValues(text_widget, args, ONE);
	} else {
		user_error ("Widget is not a dialog widget or an asciitext widget");
		return NULL;
	}
	return name;
}

/****************************************************************************/
/*  Take the name of a widget, find it, if it is a dialog or a text widget  */
/*  set and return its value, NULL is empty or error  */
/****************************************************************************/

extern String TextWidget_set_value (w_name, name)
	String w_name, name;
{
	Widget text_widget;
	char  clean_name[1024]; /* !!! */
	Arg args[1];

	if (w_name == NULL) {
		user_error ("text_widget name is NULL");
		return NULL;
	}
	WcCleanName (w_name, clean_name);
	text_widget = WcFullNameToWidget (toplevel, clean_name);
	if ( ! text_widget) {
		user_error ("No such widget [%s]", w_name);
		return NULL;
	}
	if (XtIsSubclass (text_widget, dialogWidgetClass)) {
		XtSetArg(args[0], XtNvalue, name);
	} else if (XtIsSubclass (text_widget, labelWidgetClass)) {
		XtSetArg(args[0], XtNlabel, name);
	} else if (XtIsSubclass (text_widget, asciiTextWidgetClass)) {
		XtSetArg(args[0], XtNstring, name);
	} else {
		user_error ("Widget is not a dialog widget or an asciitext widget");
		return NULL;
	}
	if (XtIsRealized(text_widget))
		XtSetValues(text_widget, args, ONE);
	return name;
}

/*****************************************************************************/
/*  Takes the name of a widget a returns a transient shell widget by that    */
/*  name if such a widget exists.                                            */
/*****************************************************************************/

extern Widget WidgetName2Popup (widget, w_name)
	Widget widget;
	String w_name;
{
	Widget popup;
	char clean_name[1024]; /* !!! */

	(void) WcCleanName (w_name, clean_name);
	popup = WcFullNameToWidget (widget, clean_name);
	if ( ! popup) {
		user_error ("No such popup widget [%s]", w_name);
		return NULL;
	}
	if ( ! (XtIsTransientShell (popup))) {
		user_error ("Widget is not a Transientshell widget [%s]", clean_name);
		return NULL;
	}
	return popup;
}

/*****************************************************************************/
/*  The following callbacks for positioning popups was adapted from the MIT   */
/*  source  */
/*****************************************************************************/
/*
 * Copyright 1989 Massachusetts Institute of Technology
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of M.I.T. not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  M.I.T. makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * M.I.T. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL M.I.T.
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */



/*	Function Name: MoveMenu
 *	Description: Actually moves the menu, may force it to
 *                   to be fully visable if menu_on_screen is TRUE.
 *	Arguments: w - the simple menu widget.
 *                 x, y - the current location of the widget.
 *	Returns: none 
 */

static void
MoveMenu(w, x, y)
Widget w;
Position x, y;
{
	Arg arglist[2];
	Cardinal num_args = 0;
    
	int width = w->core.width + 2 * w->core.border_width;
	int height = w->core.height + 2 * w->core.border_width;
	
	if (x < 0) 
	    x = 0;
	else {
	    int scr_width = WidthOfScreen(XtScreen(w));
	    if (x + width > scr_width)
		x = scr_width - width;
	}
	
	if (y < 0)
	    y = 0;
	else {
	    int scr_height = HeightOfScreen(XtScreen(w));
	    if (y + height > scr_height)
		y = scr_height - height;
	}
    
    XtSetArg(arglist[num_args], XtNx, x); num_args++;
    XtSetArg(arglist[num_args], XtNy, y); num_args++;
    XtSetValues(w, arglist, num_args);
}


CALLBACK(PopupAtCursorCB)
	String w_name = (String) client_data;
	Widget popup;
	Position x, y;
	Window junk1, junk2;
	int root_x, root_y, junkX, junkY;
	unsigned int junkM;

	if ( ! (popup = WidgetName2Popup (widget, w_name)))
		return;

	/* The width will not be correct unless it is realized. */
	if (!XtIsRealized(popup))
		XtRealizeWidget(popup);
	
	if (XQueryPointer(XtDisplay(popup), XtWindow(popup), &junk1, &junk2, 
		&root_x, &root_y, &junkX, &junkY, &junkM) == False) {
		user_error ("Could not find location of mouse pointer");
		return;
	}
	x = (Position) root_x;
	y = (Position) root_y;
    
	x -= (Position) popup->core.width/2;
    
	MoveMenu(popup, x, y);
	XtPopup (popup, XtGrabNone);
	if (XtWindow(popup))  /*  Even if already popped up, put it on top  */
		XRaiseWindow (display, XtWindow(popup));
ENDCB

CALLBACK(PopupCommandPopupCB)
	Boolean grab = (Boolean) call_data;
	String w_name = (String) client_data;
	Widget popup;
	Arg arglist[2];
	Cardinal num_args;
	int popup_x, popup_y, popup_width, popup_height, button_width, button_height;
	Position button_x, button_y;

	if ( ! (popup = WidgetName2Popup (widget, w_name)))
		return;
	
	if (!XtIsRealized(popup))
		XtRealizeWidget(popup);
  
	popup_width = popup->core.width + 2 * popup->core.border_width;
	button_width = widget->core.width + 2 * widget->core.border_width;
	button_height = widget->core.height + 2 * widget->core.border_width;
	popup_height = popup->core.height + 2 * popup->core.border_width;

	XtTranslateCoords(widget, 0, 0, &button_x, &button_y);
	popup_x = button_x;
	popup_y = button_y + button_height;

  if (popup_x < 0) 
    popup_x = 0;
  else {
    int scr_width = WidthOfScreen(XtScreen(popup));
    if (popup_x + popup_width > scr_width)
      popup_x = scr_width - popup_width;
  }

  if (popup_y < 0)
    popup_y = 0;
  else {
    int scr_height = HeightOfScreen(XtScreen(popup));
    if (popup_y + popup_height > scr_height)
      popup_y = scr_height - popup_height;
  }

  num_args = 0;
  XtSetArg(arglist[num_args], XtNx, popup_x); num_args++;
  XtSetArg(arglist[num_args], XtNy, popup_y); num_args++;
  XtSetValues(popup, arglist, num_args);

	if (grab)
		XtPopupSpringLoaded(popup);
	else
		XtPopup(popup, XtGrabNone);

	if (XtWindow(popup))  /*  Even if already popped up, put it on top  */
		XRaiseWindow (display, XtWindow(popup));
ENDCB

CALLBACK(CreateAndPopupCB)

ENDCB

CALLBACK(PopupCentralCB)
	String w_name = (String) client_data;
	Widget popup;

	if ( ! (popup = WidgetName2Popup (widget, w_name)))
		return;
	popup_central (toplevel, popup, XtGrabNonexclusive);
ENDCB

/******************************************************************************
    This function destroys the widget tree.
******************************************************************************/

CALLBACK(DestroyShellCB)
	String widgetName = (String) client_data;
	Widget condemned = WcFullNameToWidget( widget, widgetName );

	if (condemned && XtIsTransientShell (condemned))
		XtDestroyWidget( condemned );
	else
		user_error ("Widget [%s] is not a shell", widgetName);
ENDCB

CALLBACK(DestroyWidgetCB)
	String widgetName = (String) client_data;
	Widget condemned = WcFullNameToWidget( widget, widgetName );

	if (condemned)
		XtDestroyWidget( condemned );
	else
		XtDestroyWidget( WcRootWidget( widget ) );
ENDCB

/*****************************************************************************/
/*  Loads file into widget  */
/*****************************************************************************/

CALLBACK(MonetLoadFileInWidgetCB)
	FILE * f;
	String w_name, value;
	Widget text_widget;
	Arg args[2];
#define SIZEMESSAGEBUFFER  2048   /* !!! */
	char Buffer[SIZEMESSAGEBUFFER];
	char  clean_name[1024]; /* !!! */

	w_name = MonetNewString ((String) client_data);
	for (value = w_name; *value && *value != ';'; ++value);	
	if (value == NULL || *value == NULL) {
		user_error ("MonetLoadFileInWidgetC needs 2 args [%s]", w_name);
		MonetFree (w_name);
		return;
	}
	if (f = MonetFopen (value, "r")) {
		*value = '\0';
		++ value;
		WcCleanName (w_name, clean_name);
		text_widget = WcFullNameToWidget (toplevel, clean_name);
		if ( ! text_widget) {
			user_error ("No such widget [%s]", w_name);
		} else {
			char  buffer[1024]; /* !!! */
			while (fgets (buffer, sizeof(buffer), f))
				put_widget (text_widget, buffer);
		}
		fclose (f);
	} else {
		user_error ("Could not open file [%s]", value);
	}
	MonetFree (w_name);
	return;
ENDCB

/*****************************************************************************/
/*****************************************************************************/
/* $Header$ */
/*****************************************************************************/
/*****************************************************************************/
